#include "global.h"
#include "graphics.h"
#include "mail.h"
#include "palette.h"
#include "pokemon_icon.h"
#include "sprite.h"
#include "data.h"

#define POKE_ICON_BASE_PAL_TAG 56000

#define INVALID_ICON_SPECIES SPECIES_NONE // Oddly specific, used when an icon should be a ?. Any of the 'old unown' would work

struct MonIconSpriteTemplate
{
    const struct OamData *oam;
    const u8 *image;
    const union AnimCmd *const *anims;
    const union AffineAnimCmd *const *affineAnims;
    void (*callback)(struct Sprite *);
    u16 paletteTag;
};

// static functions
static u8 CreateMonIconSprite(struct MonIconSpriteTemplate *, s16, s16, u8);

// .rodata

const u8 *const gMonIconTable[] =
{
    [SPECIES_NONE] = gMonIcon_Bulbasaur,
    [SPECIES_BULBASAUR] = gMonIcon_Bulbasaur,
    [SPECIES_IVYSAUR] = gMonIcon_Ivysaur,
    [SPECIES_VENUSAUR] = gMonIcon_Venusaur,
    [SPECIES_CHARMANDER] = gMonIcon_Charmander,
    [SPECIES_CHARMELEON] = gMonIcon_Charmeleon,
    [SPECIES_CHARIZARD] = gMonIcon_Charizard,
    [SPECIES_SQUIRTLE] = gMonIcon_Squirtle,
    [SPECIES_WARTORTLE] = gMonIcon_Wartortle,
    [SPECIES_BLASTOISE] = gMonIcon_Blastoise,
    [SPECIES_CATERPIE] = gMonIcon_Caterpie,
    [SPECIES_METAPOD] = gMonIcon_Metapod,
    [SPECIES_BUTTERFREE] = gMonIcon_Butterfree,
    [SPECIES_WEEDLE] = gMonIcon_Weedle,
    [SPECIES_KAKUNA] = gMonIcon_Kakuna,
    [SPECIES_BEEDRILL] = gMonIcon_Beedrill,
    [SPECIES_PIDGEY] = gMonIcon_Pidgey,
    [SPECIES_PIDGEOTTO] = gMonIcon_Pidgeotto,
    [SPECIES_PIDGEOT] = gMonIcon_Pidgeot,
    [SPECIES_RATTATA] = gMonIcon_Rattata,
    [SPECIES_RATICATE] = gMonIcon_Raticate,
    [SPECIES_SPEAROW] = gMonIcon_Spearow,
    [SPECIES_FEAROW] = gMonIcon_Fearow,
    [SPECIES_EKANS] = gMonIcon_Ekans,
    [SPECIES_ARBOK] = gMonIcon_Arbok,
    [SPECIES_PIKACHU] = gMonIcon_Pikachu,
    [SPECIES_RAICHU] = gMonIcon_Raichu,
    [SPECIES_SANDSHREW] = gMonIcon_Sandshrew,
    [SPECIES_SANDSLASH] = gMonIcon_Sandslash,
    [SPECIES_NIDORAN_F] = gMonIcon_NidoranF,
    [SPECIES_NIDORINA] = gMonIcon_Nidorina,
    [SPECIES_NIDOQUEEN] = gMonIcon_Nidoqueen,
    [SPECIES_NIDORAN_M] = gMonIcon_NidoranM,
    [SPECIES_NIDORINO] = gMonIcon_Nidorino,
    [SPECIES_NIDOKING] = gMonIcon_Nidoking,
    [SPECIES_CLEFAIRY] = gMonIcon_Clefairy,
    [SPECIES_CLEFABLE] = gMonIcon_Clefable,
    [SPECIES_VULPIX] = gMonIcon_Vulpix,
    [SPECIES_NINETALES] = gMonIcon_Ninetales,
    [SPECIES_JIGGLYPUFF] = gMonIcon_Jigglypuff,
    [SPECIES_WIGGLYTUFF] = gMonIcon_Wigglytuff,
    [SPECIES_ZUBAT] = gMonIcon_Zubat,
    [SPECIES_GOLBAT] = gMonIcon_Golbat,
    [SPECIES_ODDISH] = gMonIcon_Oddish,
    [SPECIES_GLOOM] = gMonIcon_Gloom,
    [SPECIES_VILEPLUME] = gMonIcon_Vileplume,
    [SPECIES_PARAS] = gMonIcon_Paras,
    [SPECIES_PARASECT] = gMonIcon_Parasect,
    [SPECIES_VENONAT] = gMonIcon_Venonat,
    [SPECIES_VENOMOTH] = gMonIcon_Venomoth,
    [SPECIES_DIGLETT] = gMonIcon_Diglett,
    [SPECIES_DUGTRIO] = gMonIcon_Dugtrio,
    [SPECIES_MEOWTH] = gMonIcon_Meowth,
    [SPECIES_PERSIAN] = gMonIcon_Persian,
    [SPECIES_PSYDUCK] = gMonIcon_Psyduck,
    [SPECIES_GOLDUCK] = gMonIcon_Golduck,
    [SPECIES_MANKEY] = gMonIcon_Mankey,
    [SPECIES_PRIMEAPE] = gMonIcon_Primeape,
    [SPECIES_GROWLITHE] = gMonIcon_Growlithe,
    [SPECIES_ARCANINE] = gMonIcon_Arcanine,
    [SPECIES_POLIWAG] = gMonIcon_Poliwag,
    [SPECIES_POLIWHIRL] = gMonIcon_Poliwhirl,
    [SPECIES_POLIWRATH] = gMonIcon_Poliwrath,
    [SPECIES_ABRA] = gMonIcon_Abra,
    [SPECIES_KADABRA] = gMonIcon_Kadabra,
    [SPECIES_ALAKAZAM] = gMonIcon_Alakazam,
    [SPECIES_MACHOP] = gMonIcon_Machop,
    [SPECIES_MACHOKE] = gMonIcon_Machoke,
    [SPECIES_MACHAMP] = gMonIcon_Machamp,
    [SPECIES_BELLSPROUT] = gMonIcon_Bellsprout,
    [SPECIES_WEEPINBELL] = gMonIcon_Weepinbell,
    [SPECIES_VICTREEBEL] = gMonIcon_Victreebel,
    [SPECIES_TENTACOOL] = gMonIcon_Tentacool,
    [SPECIES_TENTACRUEL] = gMonIcon_Tentacruel,
    [SPECIES_GEODUDE] = gMonIcon_Geodude,
    [SPECIES_GRAVELER] = gMonIcon_Graveler,
    [SPECIES_GOLEM] = gMonIcon_Golem,
    [SPECIES_PONYTA] = gMonIcon_Ponyta,
    [SPECIES_RAPIDASH] = gMonIcon_Rapidash,
    [SPECIES_SLOWPOKE] = gMonIcon_Slowpoke,
    [SPECIES_SLOWBRO] = gMonIcon_Slowbro,
    [SPECIES_MAGNEMITE] = gMonIcon_Magnemite,
    [SPECIES_MAGNETON] = gMonIcon_Magneton,
    [SPECIES_FARFETCHD] = gMonIcon_Farfetchd,
    [SPECIES_DODUO] = gMonIcon_Doduo,
    [SPECIES_DODRIO] = gMonIcon_Dodrio,
    [SPECIES_SEEL] = gMonIcon_Seel,
    [SPECIES_DEWGONG] = gMonIcon_Dewgong,
    [SPECIES_GRIMER] = gMonIcon_Grimer,
    [SPECIES_MUK] = gMonIcon_Muk,
    [SPECIES_SHELLDER] = gMonIcon_Shellder,
    [SPECIES_CLOYSTER] = gMonIcon_Cloyster,
    [SPECIES_GASTLY] = gMonIcon_Gastly,
    [SPECIES_HAUNTER] = gMonIcon_Haunter,
    [SPECIES_GENGAR] = gMonIcon_Gengar,
    [SPECIES_ONIX] = gMonIcon_Onix,
    [SPECIES_DROWZEE] = gMonIcon_Drowzee,
    [SPECIES_HYPNO] = gMonIcon_Hypno,
    [SPECIES_KRABBY] = gMonIcon_Krabby,
    [SPECIES_KINGLER] = gMonIcon_Kingler,
    [SPECIES_VOLTORB] = gMonIcon_Voltorb,
    [SPECIES_ELECTRODE] = gMonIcon_Electrode,
    [SPECIES_EXEGGCUTE] = gMonIcon_Exeggcute,
    [SPECIES_EXEGGUTOR] = gMonIcon_Exeggutor,
    [SPECIES_CUBONE] = gMonIcon_Cubone,
    [SPECIES_MAROWAK] = gMonIcon_Marowak,
    [SPECIES_HITMONLEE] = gMonIcon_Hitmonlee,
    [SPECIES_HITMONCHAN] = gMonIcon_Hitmonchan,
    [SPECIES_LICKITUNG] = gMonIcon_Lickitung,
    [SPECIES_KOFFING] = gMonIcon_Koffing,
    [SPECIES_WEEZING] = gMonIcon_Weezing,
    [SPECIES_RHYHORN] = gMonIcon_Rhyhorn,
    [SPECIES_RHYDON] = gMonIcon_Rhydon,
    [SPECIES_CHANSEY] = gMonIcon_Chansey,
    [SPECIES_TANGELA] = gMonIcon_Tangela,
    [SPECIES_KANGASKHAN] = gMonIcon_Kangaskhan,
    [SPECIES_HORSEA] = gMonIcon_Horsea,
    [SPECIES_SEADRA] = gMonIcon_Seadra,
    [SPECIES_GOLDEEN] = gMonIcon_Goldeen,
    [SPECIES_SEAKING] = gMonIcon_Seaking,
    [SPECIES_STARYU] = gMonIcon_Staryu,
    [SPECIES_STARMIE] = gMonIcon_Starmie,
    [SPECIES_MR_MIME] = gMonIcon_MrMime,
    [SPECIES_SCYTHER] = gMonIcon_Scyther,
    [SPECIES_JYNX] = gMonIcon_Jynx,
    [SPECIES_ELECTABUZZ] = gMonIcon_Electabuzz,
    [SPECIES_MAGMAR] = gMonIcon_Magmar,
    [SPECIES_PINSIR] = gMonIcon_Pinsir,
    [SPECIES_TAUROS] = gMonIcon_Tauros,
    [SPECIES_MAGIKARP] = gMonIcon_Magikarp,
    [SPECIES_GYARADOS] = gMonIcon_Gyarados,
    [SPECIES_LAPRAS] = gMonIcon_Lapras,
    [SPECIES_DITTO] = gMonIcon_Ditto,
    [SPECIES_EEVEE] = gMonIcon_Eevee,
    [SPECIES_VAPOREON] = gMonIcon_Vaporeon,
    [SPECIES_JOLTEON] = gMonIcon_Jolteon,
    [SPECIES_FLAREON] = gMonIcon_Flareon,
    [SPECIES_PORYGON] = gMonIcon_Porygon,
    [SPECIES_OMANYTE] = gMonIcon_Omanyte,
    [SPECIES_OMASTAR] = gMonIcon_Omastar,
    [SPECIES_KABUTO] = gMonIcon_Kabuto,
    [SPECIES_KABUTOPS] = gMonIcon_Kabutops,
    [SPECIES_AERODACTYL] = gMonIcon_Aerodactyl,
    [SPECIES_SNORLAX] = gMonIcon_Snorlax,
    [SPECIES_ARTICUNO] = gMonIcon_Articuno,
    [SPECIES_ZAPDOS] = gMonIcon_Zapdos,
    [SPECIES_MOLTRES] = gMonIcon_Moltres,
    [SPECIES_DRATINI] = gMonIcon_Dratini,
    [SPECIES_DRAGONAIR] = gMonIcon_Dragonair,
    [SPECIES_DRAGONITE] = gMonIcon_Dragonite,
    [SPECIES_MEWTWO] = gMonIcon_Mewtwo,
    [SPECIES_MEW] = gMonIcon_Mew,
    [SPECIES_CHIKORITA] = gMonIcon_Chikorita,
    [SPECIES_BAYLEEF] = gMonIcon_Bayleef,
    [SPECIES_MEGANIUM] = gMonIcon_Meganium,
    [SPECIES_CYNDAQUIL] = gMonIcon_Cyndaquil,
    [SPECIES_QUILAVA] = gMonIcon_Quilava,
    [SPECIES_TYPHLOSION] = gMonIcon_Typhlosion,
    [SPECIES_TOTODILE] = gMonIcon_Totodile,
    [SPECIES_CROCONAW] = gMonIcon_Croconaw,
    [SPECIES_FERALIGATR] = gMonIcon_Feraligatr,
    [SPECIES_SENTRET] = gMonIcon_Sentret,
    [SPECIES_FURRET] = gMonIcon_Furret,
    [SPECIES_HOOTHOOT] = gMonIcon_Hoothoot,
    [SPECIES_NOCTOWL] = gMonIcon_Noctowl,
    [SPECIES_LEDYBA] = gMonIcon_Ledyba,
    [SPECIES_LEDIAN] = gMonIcon_Ledian,
    [SPECIES_SPINARAK] = gMonIcon_Spinarak,
    [SPECIES_ARIADOS] = gMonIcon_Ariados,
    [SPECIES_CROBAT] = gMonIcon_Crobat,
    [SPECIES_CHINCHOU] = gMonIcon_Chinchou,
    [SPECIES_LANTURN] = gMonIcon_Lanturn,
    [SPECIES_PICHU] = gMonIcon_Pichu,
    [SPECIES_CLEFFA] = gMonIcon_Cleffa,
    [SPECIES_IGGLYBUFF] = gMonIcon_Igglybuff,
    [SPECIES_TOGEPI] = gMonIcon_Togepi,
    [SPECIES_TOGETIC] = gMonIcon_Togetic,
    [SPECIES_NATU] = gMonIcon_Natu,
    [SPECIES_XATU] = gMonIcon_Xatu,
    [SPECIES_MAREEP] = gMonIcon_Mareep,
    [SPECIES_FLAAFFY] = gMonIcon_Flaaffy,
    [SPECIES_AMPHAROS] = gMonIcon_Ampharos,
    [SPECIES_BELLOSSOM] = gMonIcon_Bellossom,
    [SPECIES_MARILL] = gMonIcon_Marill,
    [SPECIES_AZUMARILL] = gMonIcon_Azumarill,
    [SPECIES_SUDOWOODO] = gMonIcon_Sudowoodo,
    [SPECIES_POLITOED] = gMonIcon_Politoed,
    [SPECIES_HOPPIP] = gMonIcon_Hoppip,
    [SPECIES_SKIPLOOM] = gMonIcon_Skiploom,
    [SPECIES_JUMPLUFF] = gMonIcon_Jumpluff,
    [SPECIES_AIPOM] = gMonIcon_Aipom,
    [SPECIES_SUNKERN] = gMonIcon_Sunkern,
    [SPECIES_SUNFLORA] = gMonIcon_Sunflora,
    [SPECIES_YANMA] = gMonIcon_Yanma,
    [SPECIES_WOOPER] = gMonIcon_Wooper,
    [SPECIES_QUAGSIRE] = gMonIcon_Quagsire,
    [SPECIES_ESPEON] = gMonIcon_Espeon,
    [SPECIES_UMBREON] = gMonIcon_Umbreon,
    [SPECIES_MURKROW] = gMonIcon_Murkrow,
    [SPECIES_SLOWKING] = gMonIcon_Slowking,
    [SPECIES_MISDREAVUS] = gMonIcon_Misdreavus,
    [SPECIES_UNOWN] = gMonIcon_Unown,
    [SPECIES_WOBBUFFET] = gMonIcon_Wobbuffet,
    [SPECIES_GIRAFARIG] = gMonIcon_Girafarig,
    [SPECIES_PINECO] = gMonIcon_Pineco,
    [SPECIES_FORRETRESS] = gMonIcon_Forretress,
    [SPECIES_DUNSPARCE] = gMonIcon_Dunsparce,
    [SPECIES_GLIGAR] = gMonIcon_Gligar,
    [SPECIES_STEELIX] = gMonIcon_Steelix,
    [SPECIES_SNUBBULL] = gMonIcon_Snubbull,
    [SPECIES_GRANBULL] = gMonIcon_Granbull,
    [SPECIES_QWILFISH] = gMonIcon_Qwilfish,
    [SPECIES_SCIZOR] = gMonIcon_Scizor,
    [SPECIES_SHUCKLE] = gMonIcon_Shuckle,
    [SPECIES_HERACROSS] = gMonIcon_Heracross,
    [SPECIES_SNEASEL] = gMonIcon_Sneasel,
    [SPECIES_TEDDIURSA] = gMonIcon_Teddiursa,
    [SPECIES_URSARING] = gMonIcon_Ursaring,
    [SPECIES_SLUGMA] = gMonIcon_Slugma,
    [SPECIES_MAGCARGO] = gMonIcon_Magcargo,
    [SPECIES_SWINUB] = gMonIcon_Swinub,
    [SPECIES_PILOSWINE] = gMonIcon_Piloswine,
    [SPECIES_CORSOLA] = gMonIcon_Corsola,
    [SPECIES_REMORAID] = gMonIcon_Remoraid,
    [SPECIES_OCTILLERY] = gMonIcon_Octillery,
    [SPECIES_DELIBIRD] = gMonIcon_Delibird,
    [SPECIES_MANTINE] = gMonIcon_Mantine,
    [SPECIES_SKARMORY] = gMonIcon_Skarmory,
    [SPECIES_HOUNDOUR] = gMonIcon_Houndour,
    [SPECIES_HOUNDOOM] = gMonIcon_Houndoom,
    [SPECIES_KINGDRA] = gMonIcon_Kingdra,
    [SPECIES_PHANPY] = gMonIcon_Phanpy,
    [SPECIES_DONPHAN] = gMonIcon_Donphan,
    [SPECIES_PORYGON2] = gMonIcon_Porygon2,
    [SPECIES_STANTLER] = gMonIcon_Stantler,
    [SPECIES_SMEARGLE] = gMonIcon_Smeargle,
    [SPECIES_TYROGUE] = gMonIcon_Tyrogue,
    [SPECIES_HITMONTOP] = gMonIcon_Hitmontop,
    [SPECIES_SMOOCHUM] = gMonIcon_Smoochum,
    [SPECIES_ELEKID] = gMonIcon_Elekid,
    [SPECIES_MAGBY] = gMonIcon_Magby,
    [SPECIES_MILTANK] = gMonIcon_Miltank,
    [SPECIES_BLISSEY] = gMonIcon_Blissey,
    [SPECIES_RAIKOU] = gMonIcon_Raikou,
    [SPECIES_ENTEI] = gMonIcon_Entei,
    [SPECIES_SUICUNE] = gMonIcon_Suicune,
    [SPECIES_LARVITAR] = gMonIcon_Larvitar,
    [SPECIES_PUPITAR] = gMonIcon_Pupitar,
    [SPECIES_TYRANITAR] = gMonIcon_Tyranitar,
    [SPECIES_LUGIA] = gMonIcon_Lugia,
    [SPECIES_HO_OH] = gMonIcon_HoOh,
    [SPECIES_CELEBI] = gMonIcon_Celebi,
    [SPECIES_TREECKO] = gMonIcon_Treecko,
    [SPECIES_GROVYLE] = gMonIcon_Grovyle,
    [SPECIES_SCEPTILE] = gMonIcon_Sceptile,
    [SPECIES_TORCHIC] = gMonIcon_Torchic,
    [SPECIES_COMBUSKEN] = gMonIcon_Combusken,
    [SPECIES_BLAZIKEN] = gMonIcon_Blaziken,
    [SPECIES_MUDKIP] = gMonIcon_Mudkip,
    [SPECIES_MARSHTOMP] = gMonIcon_Marshtomp,
    [SPECIES_SWAMPERT] = gMonIcon_Swampert,
    [SPECIES_POOCHYENA] = gMonIcon_Poochyena,
    [SPECIES_MIGHTYENA] = gMonIcon_Mightyena,
    [SPECIES_ZIGZAGOON] = gMonIcon_Zigzagoon,
    [SPECIES_LINOONE] = gMonIcon_Linoone,
    [SPECIES_WURMPLE] = gMonIcon_Wurmple,
    [SPECIES_SILCOON] = gMonIcon_Silcoon,
    [SPECIES_BEAUTIFLY] = gMonIcon_Beautifly,
    [SPECIES_CASCOON] = gMonIcon_Cascoon,
    [SPECIES_DUSTOX] = gMonIcon_Dustox,
    [SPECIES_LOTAD] = gMonIcon_Lotad,
    [SPECIES_LOMBRE] = gMonIcon_Lombre,
    [SPECIES_LUDICOLO] = gMonIcon_Ludicolo,
    [SPECIES_SEEDOT] = gMonIcon_Seedot,
    [SPECIES_NUZLEAF] = gMonIcon_Nuzleaf,
    [SPECIES_SHIFTRY] = gMonIcon_Shiftry,
    [SPECIES_NINCADA] = gMonIcon_Nincada,
    [SPECIES_NINJASK] = gMonIcon_Ninjask,
    [SPECIES_SHEDINJA] = gMonIcon_Shedinja,
    [SPECIES_TAILLOW] = gMonIcon_Taillow,
    [SPECIES_SWELLOW] = gMonIcon_Swellow,
    [SPECIES_SHROOMISH] = gMonIcon_Shroomish,
    [SPECIES_BRELOOM] = gMonIcon_Breloom,
    [SPECIES_SPINDA] = gMonIcon_Spinda,
    [SPECIES_WINGULL] = gMonIcon_Wingull,
    [SPECIES_PELIPPER] = gMonIcon_Pelipper,
    [SPECIES_SURSKIT] = gMonIcon_Surskit,
    [SPECIES_MASQUERAIN] = gMonIcon_Masquerain,
    [SPECIES_WAILMER] = gMonIcon_Wailmer,
    [SPECIES_WAILORD] = gMonIcon_Wailord,
    [SPECIES_SKITTY] = gMonIcon_Skitty,
    [SPECIES_DELCATTY] = gMonIcon_Delcatty,
    [SPECIES_KECLEON] = gMonIcon_Kecleon,
    [SPECIES_BALTOY] = gMonIcon_Baltoy,
    [SPECIES_CLAYDOL] = gMonIcon_Claydol,
    [SPECIES_NOSEPASS] = gMonIcon_Nosepass,
    [SPECIES_TORKOAL] = gMonIcon_Torkoal,
    [SPECIES_SABLEYE] = gMonIcon_Sableye,
    [SPECIES_BARBOACH] = gMonIcon_Barboach,
    [SPECIES_WHISCASH] = gMonIcon_Whiscash,
    [SPECIES_LUVDISC] = gMonIcon_Luvdisc,
    [SPECIES_CORPHISH] = gMonIcon_Corphish,
    [SPECIES_CRAWDAUNT] = gMonIcon_Crawdaunt,
    [SPECIES_FEEBAS] = gMonIcon_Feebas,
    [SPECIES_MILOTIC] = gMonIcon_Milotic,
    [SPECIES_CARVANHA] = gMonIcon_Carvanha,
    [SPECIES_SHARPEDO] = gMonIcon_Sharpedo,
    [SPECIES_TRAPINCH] = gMonIcon_Trapinch,
    [SPECIES_VIBRAVA] = gMonIcon_Vibrava,
    [SPECIES_FLYGON] = gMonIcon_Flygon,
    [SPECIES_MAKUHITA] = gMonIcon_Makuhita,
    [SPECIES_HARIYAMA] = gMonIcon_Hariyama,
    [SPECIES_ELECTRIKE] = gMonIcon_Electrike,
    [SPECIES_MANECTRIC] = gMonIcon_Manectric,
    [SPECIES_NUMEL] = gMonIcon_Numel,
    [SPECIES_CAMERUPT] = gMonIcon_Camerupt,
    [SPECIES_SPHEAL] = gMonIcon_Spheal,
    [SPECIES_SEALEO] = gMonIcon_Sealeo,
    [SPECIES_WALREIN] = gMonIcon_Walrein,
    [SPECIES_CACNEA] = gMonIcon_Cacnea,
    [SPECIES_CACTURNE] = gMonIcon_Cacturne,
    [SPECIES_SNORUNT] = gMonIcon_Snorunt,
    [SPECIES_GLALIE] = gMonIcon_Glalie,
    [SPECIES_LUNATONE] = gMonIcon_Lunatone,
    [SPECIES_SOLROCK] = gMonIcon_Solrock,
    [SPECIES_AZURILL] = gMonIcon_Azurill,
    [SPECIES_SPOINK] = gMonIcon_Spoink,
    [SPECIES_GRUMPIG] = gMonIcon_Grumpig,
    [SPECIES_PLUSLE] = gMonIcon_Plusle,
    [SPECIES_MINUN] = gMonIcon_Minun,
    [SPECIES_MAWILE] = gMonIcon_Mawile,
    [SPECIES_MEDITITE] = gMonIcon_Meditite,
    [SPECIES_MEDICHAM] = gMonIcon_Medicham,
    [SPECIES_SWABLU] = gMonIcon_Swablu,
    [SPECIES_ALTARIA] = gMonIcon_Altaria,
    [SPECIES_WYNAUT] = gMonIcon_Wynaut,
    [SPECIES_DUSKULL] = gMonIcon_Duskull,
    [SPECIES_DUSCLOPS] = gMonIcon_Dusclops,
    [SPECIES_ROSELIA] = gMonIcon_Roselia,
    [SPECIES_SLAKOTH] = gMonIcon_Slakoth,
    [SPECIES_VIGOROTH] = gMonIcon_Vigoroth,
    [SPECIES_SLAKING] = gMonIcon_Slaking,
    [SPECIES_GULPIN] = gMonIcon_Gulpin,
    [SPECIES_SWALOT] = gMonIcon_Swalot,
    [SPECIES_TROPIUS] = gMonIcon_Tropius,
    [SPECIES_WHISMUR] = gMonIcon_Whismur,
    [SPECIES_LOUDRED] = gMonIcon_Loudred,
    [SPECIES_EXPLOUD] = gMonIcon_Exploud,
    [SPECIES_CLAMPERL] = gMonIcon_Clamperl,
    [SPECIES_HUNTAIL] = gMonIcon_Huntail,
    [SPECIES_GOREBYSS] = gMonIcon_Gorebyss,
    [SPECIES_ABSOL] = gMonIcon_Absol,
    [SPECIES_SHUPPET] = gMonIcon_Shuppet,
    [SPECIES_BANETTE] = gMonIcon_Banette,
    [SPECIES_SEVIPER] = gMonIcon_Seviper,
    [SPECIES_ZANGOOSE] = gMonIcon_Zangoose,
    [SPECIES_RELICANTH] = gMonIcon_Relicanth,
    [SPECIES_ARON] = gMonIcon_Aron,
    [SPECIES_LAIRON] = gMonIcon_Lairon,
    [SPECIES_AGGRON] = gMonIcon_Aggron,
    [SPECIES_CASTFORM] = gMonIcon_Castform,
    [SPECIES_VOLBEAT] = gMonIcon_Volbeat,
    [SPECIES_ILLUMISE] = gMonIcon_Illumise,
    [SPECIES_LILEEP] = gMonIcon_Lileep,
    [SPECIES_CRADILY] = gMonIcon_Cradily,
    [SPECIES_ANORITH] = gMonIcon_Anorith,
    [SPECIES_ARMALDO] = gMonIcon_Armaldo,
    [SPECIES_RALTS] = gMonIcon_Ralts,
    [SPECIES_KIRLIA] = gMonIcon_Kirlia,
    [SPECIES_GARDEVOIR] = gMonIcon_Gardevoir,
    [SPECIES_BAGON] = gMonIcon_Bagon,
    [SPECIES_SHELGON] = gMonIcon_Shelgon,
    [SPECIES_SALAMENCE] = gMonIcon_Salamence,
    [SPECIES_BELDUM] = gMonIcon_Beldum,
    [SPECIES_METANG] = gMonIcon_Metang,
    [SPECIES_METAGROSS] = gMonIcon_Metagross,
    [SPECIES_REGIROCK] = gMonIcon_Regirock,
    [SPECIES_REGICE] = gMonIcon_Regice,
    [SPECIES_REGISTEEL] = gMonIcon_Registeel,
    [SPECIES_KYOGRE] = gMonIcon_Kyogre,
    [SPECIES_GROUDON] = gMonIcon_Groudon,
    [SPECIES_RAYQUAZA] = gMonIcon_Rayquaza,
    [SPECIES_LATIAS] = gMonIcon_Latias,
    [SPECIES_LATIOS] = gMonIcon_Latios,
    [SPECIES_JIRACHI] = gMonIcon_Jirachi,
    [SPECIES_DEOXYS] = gMonIcon_Deoxys,
    [SPECIES_CHIMECHO] = gMonIcon_Chimecho,
    [SPECIES_TURTWIG] = gMonIcon_Turtwig,
    [SPECIES_GROTLE] = gMonIcon_Grotle,
    [SPECIES_TORTERRA] = gMonIcon_Torterra,
    [SPECIES_CHIMCHAR] = gMonIcon_Chimchar,
    [SPECIES_MONFERNO] = gMonIcon_Monferno,
    [SPECIES_INFERNAPE] = gMonIcon_Infernape,
    [SPECIES_PIPLUP] = gMonIcon_Piplup,
    [SPECIES_PRINPLUP] = gMonIcon_Prinplup,
    [SPECIES_EMPOLEON] = gMonIcon_Empoleon,
    [SPECIES_STARLY] = gMonIcon_Starly,
    [SPECIES_STARAVIA] = gMonIcon_Staravia,
    [SPECIES_STARAPTOR] = gMonIcon_Staraptor,
    [SPECIES_BIDOOF] = gMonIcon_Bidoof,
    [SPECIES_BIBAREL] = gMonIcon_Bibarel,
    [SPECIES_KRICKETOT] = gMonIcon_Kricketot,
    [SPECIES_KRICKETUNE] = gMonIcon_Kricketune,
    [SPECIES_SHINX] = gMonIcon_Shinx,
    [SPECIES_LUXIO] = gMonIcon_Luxio,
    [SPECIES_LUXRAY] = gMonIcon_Luxray,
    [SPECIES_BUDEW] = gMonIcon_Budew,
    [SPECIES_ROSERADE] = gMonIcon_Roserade,
    [SPECIES_CRANIDOS] = gMonIcon_Cranidos,
    [SPECIES_RAMPARDOS] = gMonIcon_Rampardos,
    [SPECIES_SHIELDON] = gMonIcon_Shieldon,
    [SPECIES_BASTIODON] = gMonIcon_Bastiodon,
    [SPECIES_BURMY] = gMonIcon_Burmy,
    [SPECIES_WORMADAM] = gMonIcon_Wormadam,
    [SPECIES_MOTHIM] = gMonIcon_Mothim,
    [SPECIES_COMBEE] = gMonIcon_Combee,
    [SPECIES_VESPIQUEN] = gMonIcon_Vespiquen,
    [SPECIES_PACHIRISU] = gMonIcon_Pachirisu,
    [SPECIES_BUIZEL] = gMonIcon_Buizel,
    [SPECIES_FLOATZEL] = gMonIcon_Floatzel,
    [SPECIES_CHERUBI] = gMonIcon_Cherubi,
    [SPECIES_CHERRIM] = gMonIcon_Cherrim,
    [SPECIES_SHELLOS] = gMonIcon_Shellos,
    [SPECIES_GASTRODON] = gMonIcon_Gastrodon,
    [SPECIES_AMBIPOM] = gMonIcon_Ambipom,
    [SPECIES_DRIFLOON] = gMonIcon_Drifloon,
    [SPECIES_DRIFBLIM] = gMonIcon_Drifblim,
    [SPECIES_BUNEARY] = gMonIcon_Buneary,
    [SPECIES_LOPUNNY] = gMonIcon_Lopunny,
    [SPECIES_MISMAGIUS] = gMonIcon_Mismagius,
    [SPECIES_HONCHKROW] = gMonIcon_Honchkrow,
    [SPECIES_GLAMEOW] = gMonIcon_Glameow,
    [SPECIES_PURUGLY] = gMonIcon_Purugly,
    [SPECIES_CHINGLING] = gMonIcon_Chingling,
    [SPECIES_STUNKY] = gMonIcon_Stunky,
    [SPECIES_SKUNTANK] = gMonIcon_Skuntank,
    [SPECIES_BRONZOR] = gMonIcon_Bronzor,
    [SPECIES_BRONZONG] = gMonIcon_Bronzong,
    [SPECIES_BONSLY] = gMonIcon_Bonsly,
    [SPECIES_MIME_JR] = gMonIcon_MimeJr,
    [SPECIES_HAPPINY] = gMonIcon_Happiny,
    [SPECIES_CHATOT] = gMonIcon_Chatot,
    [SPECIES_SPIRITOMB] = gMonIcon_Spiritomb,
    [SPECIES_GIBLE] = gMonIcon_Gible,
    [SPECIES_GABITE] = gMonIcon_Gabite,
    [SPECIES_GARCHOMP] = gMonIcon_Garchomp,
    [SPECIES_MUNCHLAX] = gMonIcon_Munchlax,
    [SPECIES_RIOLU] = gMonIcon_Riolu,
    [SPECIES_LUCARIO] = gMonIcon_Lucario,
    [SPECIES_HIPPOPOTAS] = gMonIcon_Hippopotas,
    [SPECIES_HIPPOWDON] = gMonIcon_Hippowdon,
    [SPECIES_SKORUPI] = gMonIcon_Skorupi,
    [SPECIES_DRAPION] = gMonIcon_Drapion,
    [SPECIES_CROAGUNK] = gMonIcon_Croagunk,
    [SPECIES_TOXICROAK] = gMonIcon_Toxicroak,
    [SPECIES_CARNIVINE] = gMonIcon_Carnivine,
    [SPECIES_FINNEON] = gMonIcon_Finneon,
    [SPECIES_LUMINEON] = gMonIcon_Lumineon,
    [SPECIES_MANTYKE] = gMonIcon_Mantyke,
    [SPECIES_SNOVER] = gMonIcon_Snover,
    [SPECIES_ABOMASNOW] = gMonIcon_Abomasnow,
    [SPECIES_WEAVILE] = gMonIcon_Weavile,
    [SPECIES_MAGNEZONE] = gMonIcon_Magnezone,
    [SPECIES_LICKILICKY] = gMonIcon_Lickilicky,
    [SPECIES_RHYPERIOR] = gMonIcon_Rhyperior,
    [SPECIES_TANGROWTH] = gMonIcon_Tangrowth,
    [SPECIES_ELECTIVIRE] = gMonIcon_Electivire,
    [SPECIES_MAGMORTAR] = gMonIcon_Magmortar,
    [SPECIES_TOGEKISS] = gMonIcon_Togekiss,
    [SPECIES_YANMEGA] = gMonIcon_Yanmega,
    [SPECIES_LEAFEON] = gMonIcon_Leafeon,
    [SPECIES_GLACEON] = gMonIcon_Glaceon,
    [SPECIES_GLISCOR] = gMonIcon_Gliscor,
    [SPECIES_MAMOSWINE] = gMonIcon_Mamoswine,
    [SPECIES_PORYGON_Z] = gMonIcon_Porygon_Z,
    [SPECIES_GALLADE] = gMonIcon_Gallade,
    [SPECIES_PROBOPASS] = gMonIcon_Probopass,
    [SPECIES_DUSKNOIR] = gMonIcon_Dusknoir,
    [SPECIES_FROSLASS] = gMonIcon_Froslass,
    [SPECIES_ROTOM] = gMonIcon_Rotom,
    [SPECIES_UXIE] = gMonIcon_Uxie,
    [SPECIES_MESPRIT] = gMonIcon_Mesprit,
    [SPECIES_AZELF] = gMonIcon_Azelf,
    [SPECIES_DIALGA] = gMonIcon_Dialga,
    [SPECIES_PALKIA] = gMonIcon_Palkia,
    [SPECIES_HEATRAN] = gMonIcon_Heatran,
    [SPECIES_REGIGIGAS] = gMonIcon_Regigigas,
    [SPECIES_GIRATINA] = gMonIcon_Giratina,
    [SPECIES_CRESSELIA] = gMonIcon_Cresselia,
    [SPECIES_PHIONE] = gMonIcon_Phione,
    [SPECIES_MANAPHY] = gMonIcon_Manaphy,
    [SPECIES_DARKRAI] = gMonIcon_Darkrai,
    [SPECIES_SHAYMIN] = gMonIcon_Shaymin,
    [SPECIES_ARCEUS] = gMonIcon_Arceus,
    [SPECIES_VICTINI] = gMonIcon_Victini,
    [SPECIES_SNIVY] = gMonIcon_Snivy,
    [SPECIES_SERVINE] = gMonIcon_Servine,
    [SPECIES_SERPERIOR] = gMonIcon_Serperior,
    [SPECIES_TEPIG] = gMonIcon_Tepig,
    [SPECIES_PIGNITE] = gMonIcon_Pignite,
    [SPECIES_EMBOAR] = gMonIcon_Emboar,
    [SPECIES_OSHAWOTT] = gMonIcon_Oshawott,
    [SPECIES_DEWOTT] = gMonIcon_Dewott,
    [SPECIES_SAMUROTT] = gMonIcon_Samurott,
    [SPECIES_PATRAT] = gMonIcon_Patrat,
    [SPECIES_WATCHOG] = gMonIcon_Watchog,
    [SPECIES_LILLIPUP] = gMonIcon_Lillipup,
    [SPECIES_HERDIER] = gMonIcon_Herdier,
    [SPECIES_STOUTLAND] = gMonIcon_Stoutland,
    [SPECIES_PURRLOIN] = gMonIcon_Purrloin,
    [SPECIES_LIEPARD] = gMonIcon_Liepard,
    [SPECIES_PANSAGE] = gMonIcon_Pansage,
    [SPECIES_SIMISAGE] = gMonIcon_Simisage,
    [SPECIES_PANSEAR] = gMonIcon_Pansear,
    [SPECIES_SIMISEAR] = gMonIcon_Simisear,
    [SPECIES_PANPOUR] = gMonIcon_Panpour,
    [SPECIES_SIMIPOUR] = gMonIcon_Simipour,
    [SPECIES_MUNNA] = gMonIcon_Munna,
    [SPECIES_MUSHARNA] = gMonIcon_Musharna,
    [SPECIES_PIDOVE] = gMonIcon_Pidove,
    [SPECIES_TRANQUILL] = gMonIcon_Tranquill,
    [SPECIES_UNFEZANT] = gMonIcon_Unfezant,
    [SPECIES_BLITZLE] = gMonIcon_Blitzle,
    [SPECIES_ZEBSTRIKA] = gMonIcon_Zebstrika,
    [SPECIES_ROGGENROLA] = gMonIcon_Roggenrola,
    [SPECIES_BOLDORE] = gMonIcon_Boldore,
    [SPECIES_GIGALITH] = gMonIcon_Gigalith,
    [SPECIES_WOOBAT] = gMonIcon_Woobat,
    [SPECIES_SWOOBAT] = gMonIcon_Swoobat,
    [SPECIES_DRILBUR] = gMonIcon_Drilbur,
    [SPECIES_EXCADRILL] = gMonIcon_Excadrill,
    [SPECIES_AUDINO] = gMonIcon_Audino,
    [SPECIES_TIMBURR] = gMonIcon_Timburr,
    [SPECIES_GURDURR] = gMonIcon_Gurdurr,
    [SPECIES_CONKELDURR] = gMonIcon_Conkeldurr,
    [SPECIES_TYMPOLE] = gMonIcon_Tympole,
    [SPECIES_PALPITOAD] = gMonIcon_Palpitoad,
    [SPECIES_SEISMITOAD] = gMonIcon_Seismitoad,
    [SPECIES_THROH] = gMonIcon_Throh,
    [SPECIES_SAWK] = gMonIcon_Sawk,
    [SPECIES_SEWADDLE] = gMonIcon_Sewaddle,
    [SPECIES_SWADLOON] = gMonIcon_Swadloon,
    [SPECIES_LEAVANNY] = gMonIcon_Leavanny,
    [SPECIES_VENIPEDE] = gMonIcon_Venipede,
    [SPECIES_WHIRLIPEDE] = gMonIcon_Whirlipede,
    [SPECIES_SCOLIPEDE] = gMonIcon_Scolipede,
    [SPECIES_COTTONEE] = gMonIcon_Cottonee,
    [SPECIES_WHIMSICOTT] = gMonIcon_Whimsicott,
    [SPECIES_PETILIL] = gMonIcon_Petilil,
    [SPECIES_LILLIGANT] = gMonIcon_Lilligant,
    [SPECIES_BASCULIN] = gMonIcon_Basculin,
    [SPECIES_SANDILE] = gMonIcon_Sandile,
    [SPECIES_KROKOROK] = gMonIcon_Krokorok,
    [SPECIES_KROOKODILE] = gMonIcon_Krookodile,
    [SPECIES_DARUMAKA] = gMonIcon_Darumaka,
    [SPECIES_DARMANITAN] = gMonIcon_Darmanitan,
    [SPECIES_MARACTUS] = gMonIcon_Maractus,
    [SPECIES_DWEBBLE] = gMonIcon_Dwebble,
    [SPECIES_CRUSTLE] = gMonIcon_Crustle,
    [SPECIES_SCRAGGY] = gMonIcon_Scraggy,
    [SPECIES_SCRAFTY] = gMonIcon_Scrafty,
    [SPECIES_SIGILYPH] = gMonIcon_Sigilyph,
    [SPECIES_YAMASK] = gMonIcon_Yamask,
    [SPECIES_COFAGRIGUS] = gMonIcon_Cofagrigus,
    [SPECIES_TIRTOUGA] = gMonIcon_Tirtouga,
    [SPECIES_CARRACOSTA] = gMonIcon_Carracosta,
    [SPECIES_ARCHEN] = gMonIcon_Archen,
    [SPECIES_ARCHEOPS] = gMonIcon_Archeops,
    [SPECIES_TRUBBISH] = gMonIcon_Trubbish,
    [SPECIES_GARBODOR] = gMonIcon_Garbodor,
    [SPECIES_ZORUA] = gMonIcon_Zorua,
    [SPECIES_ZOROARK] = gMonIcon_Zoroark,
    [SPECIES_MINCCINO] = gMonIcon_Minccino,
    [SPECIES_CINCCINO] = gMonIcon_Cinccino,
    [SPECIES_GOTHITA] = gMonIcon_Gothita,
    [SPECIES_GOTHORITA] = gMonIcon_Gothorita,
    [SPECIES_GOTHITELLE] = gMonIcon_Gothitelle,
    [SPECIES_SOLOSIS] = gMonIcon_Solosis,
    [SPECIES_DUOSION] = gMonIcon_Duosion,
    [SPECIES_REUNICLUS] = gMonIcon_Reuniclus,
    [SPECIES_DUCKLETT] = gMonIcon_Ducklett,
    [SPECIES_SWANNA] = gMonIcon_Swanna,
    [SPECIES_VANILLITE] = gMonIcon_Vanillite,
    [SPECIES_VANILLISH] = gMonIcon_Vanillish,
    [SPECIES_VANILLUXE] = gMonIcon_Vanilluxe,
    [SPECIES_DEERLING] = gMonIcon_Deerling,
    [SPECIES_SAWSBUCK] = gMonIcon_Sawsbuck,
    [SPECIES_EMOLGA] = gMonIcon_Emolga,
    [SPECIES_KARRABLAST] = gMonIcon_Karrablast,
    [SPECIES_ESCAVALIER] = gMonIcon_Escavalier,
    [SPECIES_FOONGUS] = gMonIcon_Foongus,
    [SPECIES_AMOONGUSS] = gMonIcon_Amoonguss,
    [SPECIES_FRILLISH] = gMonIcon_Frillish,
    [SPECIES_JELLICENT] = gMonIcon_Jellicent,
    [SPECIES_ALOMOMOLA] = gMonIcon_Alomomola,
    [SPECIES_JOLTIK] = gMonIcon_Joltik,
    [SPECIES_GALVANTULA] = gMonIcon_Galvantula,
    [SPECIES_FERROSEED] = gMonIcon_Ferroseed,
    [SPECIES_FERROTHORN] = gMonIcon_Ferrothorn,
    [SPECIES_KLINK] = gMonIcon_Klink,
    [SPECIES_KLANG] = gMonIcon_Klang,
    [SPECIES_KLINKLANG] = gMonIcon_Klinklang,
    [SPECIES_TYNAMO] = gMonIcon_Tynamo,
    [SPECIES_EELEKTRIK] = gMonIcon_Eelektrik,
    [SPECIES_EELEKTROSS] = gMonIcon_Eelektross,
    [SPECIES_ELGYEM] = gMonIcon_Elgyem,
    [SPECIES_BEHEEYEM] = gMonIcon_Beheeyem,
    [SPECIES_LITWICK] = gMonIcon_Litwick,
    [SPECIES_LAMPENT] = gMonIcon_Lampent,
    [SPECIES_CHANDELURE] = gMonIcon_Chandelure,
    [SPECIES_AXEW] = gMonIcon_Axew,
    [SPECIES_FRAXURE] = gMonIcon_Fraxure,
    [SPECIES_HAXORUS] = gMonIcon_Haxorus,
    [SPECIES_CUBCHOO] = gMonIcon_Cubchoo,
    [SPECIES_BEARTIC] = gMonIcon_Beartic,
    [SPECIES_CRYOGONAL] = gMonIcon_Cryogonal,
    [SPECIES_SHELMET] = gMonIcon_Shelmet,
    [SPECIES_ACCELGOR] = gMonIcon_Accelgor,
    [SPECIES_STUNFISK] = gMonIcon_Stunfisk,
    [SPECIES_MIENFOO] = gMonIcon_Mienfoo,
    [SPECIES_MIENSHAO] = gMonIcon_Mienshao,
    [SPECIES_DRUDDIGON] = gMonIcon_Druddigon,
    [SPECIES_GOLETT] = gMonIcon_Golett,
    [SPECIES_GOLURK] = gMonIcon_Golurk,
    [SPECIES_PAWNIARD] = gMonIcon_Pawniard,
    [SPECIES_BISHARP] = gMonIcon_Bisharp,
    [SPECIES_BOUFFALANT] = gMonIcon_Bouffalant,
    [SPECIES_RUFFLET] = gMonIcon_Rufflet,
    [SPECIES_BRAVIARY] = gMonIcon_Braviary,
    [SPECIES_VULLABY] = gMonIcon_Vullaby,
    [SPECIES_MANDIBUZZ] = gMonIcon_Mandibuzz,
    [SPECIES_HEATMOR] = gMonIcon_Heatmor,
    [SPECIES_DURANT] = gMonIcon_Durant,
    [SPECIES_DEINO] = gMonIcon_Deino,
    [SPECIES_ZWEILOUS] = gMonIcon_Zweilous,
    [SPECIES_HYDREIGON] = gMonIcon_Hydreigon,
    [SPECIES_LARVESTA] = gMonIcon_Larvesta,
    [SPECIES_VOLCARONA] = gMonIcon_Volcarona,
    [SPECIES_COBALION] = gMonIcon_Cobalion,
    [SPECIES_TERRAKION] = gMonIcon_Terrakion,
    [SPECIES_VIRIZION] = gMonIcon_Virizion,
    [SPECIES_TORNADUS] = gMonIcon_Tornadus,
    [SPECIES_THUNDURUS] = gMonIcon_Thundurus,
    [SPECIES_RESHIRAM] = gMonIcon_Reshiram,
    [SPECIES_ZEKROM] = gMonIcon_Zekrom,
    [SPECIES_LANDORUS] = gMonIcon_Landorus,
    [SPECIES_KYUREM] = gMonIcon_Kyurem,
    [SPECIES_KELDEO] = gMonIcon_Keldeo,
    [SPECIES_MELOETTA] = gMonIcon_Meloetta,
    [SPECIES_GENESECT] = gMonIcon_Genesect,
    [SPECIES_CHESPIN] = gMonIcon_Chespin,
    [SPECIES_QUILLADIN] = gMonIcon_Quilladin,
    [SPECIES_CHESNAUGHT] = gMonIcon_Chesnaught,
    [SPECIES_FENNEKIN] = gMonIcon_Fennekin,
    [SPECIES_BRAIXEN] = gMonIcon_Braixen,
    [SPECIES_DELPHOX] = gMonIcon_Delphox,
    [SPECIES_FROAKIE] = gMonIcon_Froakie,
    [SPECIES_FROGADIER] = gMonIcon_Frogadier,
    [SPECIES_GRENINJA] = gMonIcon_Greninja,
    [SPECIES_BUNNELBY] = gMonIcon_Bunnelby,
    [SPECIES_DIGGERSBY] = gMonIcon_Diggersby,
    [SPECIES_FLETCHLING] = gMonIcon_Fletchling,
    [SPECIES_FLETCHINDER] = gMonIcon_Fletchinder,
    [SPECIES_TALONFLAME] = gMonIcon_Talonflame,
    [SPECIES_SCATTERBUG] = gMonIcon_Scatterbug,
    [SPECIES_SPEWPA] = gMonIcon_Spewpa,
    [SPECIES_VIVILLON] = gMonIcon_Vivillon,
    [SPECIES_LITLEO] = gMonIcon_Litleo,
    [SPECIES_PYROAR] = gMonIcon_Pyroar,
    [SPECIES_FLABEBE] = gMonIcon_Flabebe,
    [SPECIES_FLOETTE] = gMonIcon_Floette,
    [SPECIES_FLORGES] = gMonIcon_Florges,
    [SPECIES_SKIDDO] = gMonIcon_Skiddo,
    [SPECIES_GOGOAT] = gMonIcon_Gogoat,
    [SPECIES_PANCHAM] = gMonIcon_Pancham,
    [SPECIES_PANGORO] = gMonIcon_Pangoro,
    [SPECIES_FURFROU] = gMonIcon_Furfrou,
    [SPECIES_ESPURR] = gMonIcon_Espurr,
    [SPECIES_MEOWSTIC] = gMonIcon_Meowstic,
    [SPECIES_HONEDGE] = gMonIcon_Honedge,
    [SPECIES_DOUBLADE] = gMonIcon_Doublade,
    [SPECIES_AEGISLASH] = gMonIcon_Aegislash,
    [SPECIES_SPRITZEE] = gMonIcon_Spritzee,
    [SPECIES_AROMATISSE] = gMonIcon_Aromatisse,
    [SPECIES_SWIRLIX] = gMonIcon_Swirlix,
    [SPECIES_SLURPUFF] = gMonIcon_Slurpuff,
    [SPECIES_INKAY] = gMonIcon_Inkay,
    [SPECIES_MALAMAR] = gMonIcon_Malamar,
    [SPECIES_BINACLE] = gMonIcon_Binacle,
    [SPECIES_BARBARACLE] = gMonIcon_Barbaracle,
    [SPECIES_SKRELP] = gMonIcon_Skrelp,
    [SPECIES_DRAGALGE] = gMonIcon_Dragalge,
    [SPECIES_CLAUNCHER] = gMonIcon_Clauncher,
    [SPECIES_CLAWITZER] = gMonIcon_Clawitzer,
    [SPECIES_HELIOPTILE] = gMonIcon_Helioptile,
    [SPECIES_HELIOLISK] = gMonIcon_Heliolisk,
    [SPECIES_TYRUNT] = gMonIcon_Tyrunt,
    [SPECIES_TYRANTRUM] = gMonIcon_Tyrantrum,
    [SPECIES_AMAURA] = gMonIcon_Amaura,
    [SPECIES_AURORUS] = gMonIcon_Aurorus,
    [SPECIES_SYLVEON] = gMonIcon_Sylveon,
    [SPECIES_HAWLUCHA] = gMonIcon_Hawlucha,
    [SPECIES_DEDENNE] = gMonIcon_Dedenne,
    [SPECIES_CARBINK] = gMonIcon_Carbink,
    [SPECIES_GOOMY] = gMonIcon_Goomy,
    [SPECIES_SLIGGOO] = gMonIcon_Sliggoo,
    [SPECIES_GOODRA] = gMonIcon_Goodra,
    [SPECIES_KLEFKI] = gMonIcon_Klefki,
    [SPECIES_PHANTUMP] = gMonIcon_Phantump,
    [SPECIES_TREVENANT] = gMonIcon_Trevenant,
    [SPECIES_PUMPKABOO] = gMonIcon_Pumpkaboo,
    [SPECIES_GOURGEIST] = gMonIcon_Gourgeist,
    [SPECIES_BERGMITE] = gMonIcon_Bergmite,
    [SPECIES_AVALUGG] = gMonIcon_Avalugg,
    [SPECIES_NOIBAT] = gMonIcon_Noibat,
    [SPECIES_NOIVERN] = gMonIcon_Noivern,
    [SPECIES_XERNEAS] = gMonIcon_Xerneas,
    [SPECIES_YVELTAL] = gMonIcon_Yveltal,
    [SPECIES_ZYGARDE] = gMonIcon_Zygarde,
    [SPECIES_DIANCIE] = gMonIcon_Diancie,
    [SPECIES_HOOPA] = gMonIcon_Hoopa,
    [SPECIES_VOLCANION] = gMonIcon_Volcanion,
    [SPECIES_ROWLET] = gMonIcon_Rowlet,
    [SPECIES_DARTRIX] = gMonIcon_Dartrix,
    [SPECIES_DECIDUEYE] = gMonIcon_Decidueye,
    [SPECIES_LITTEN] = gMonIcon_Litten,
    [SPECIES_TORRACAT] = gMonIcon_Torracat,
    [SPECIES_INCINEROAR] = gMonIcon_Incineroar,
    [SPECIES_POPPLIO] = gMonIcon_Popplio,
    [SPECIES_BRIONNE] = gMonIcon_Brionne,
    [SPECIES_PRIMARINA] = gMonIcon_Primarina,
    [SPECIES_PIKIPEK] = gMonIcon_Pikipek,
    [SPECIES_TRUMBEAK] = gMonIcon_Trumbeak,
    [SPECIES_TOUCANNON] = gMonIcon_Toucannon,
    [SPECIES_YUNGOOS] = gMonIcon_Yungoos,
    [SPECIES_GUMSHOOS] = gMonIcon_Gumshoos,
    [SPECIES_GRUBBIN] = gMonIcon_Grubbin,
    [SPECIES_CHARJABUG] = gMonIcon_Charjabug,
    [SPECIES_VIKAVOLT] = gMonIcon_Vikavolt,
    [SPECIES_CRABRAWLER] = gMonIcon_Crabrawler,
    [SPECIES_CRABOMINABLE] = gMonIcon_Crabominable,
    [SPECIES_ORICORIO] = gMonIcon_Oricorio,
    [SPECIES_CUTIEFLY] = gMonIcon_Cutiefly,
    [SPECIES_RIBOMBEE] = gMonIcon_Ribombee,
    [SPECIES_ROCKRUFF] = gMonIcon_Rockruff,
    [SPECIES_LYCANROC] = gMonIcon_Lycanroc,
    [SPECIES_WISHIWASHI] = gMonIcon_Wishiwashi,
    [SPECIES_MAREANIE] = gMonIcon_Mareanie,
    [SPECIES_TOXAPEX] = gMonIcon_Toxapex,
    [SPECIES_MUDBRAY] = gMonIcon_Mudbray,
    [SPECIES_MUDSDALE] = gMonIcon_Mudsdale,
    [SPECIES_DEWPIDER] = gMonIcon_Dewpider,
    [SPECIES_ARAQUANID] = gMonIcon_Araquanid,
    [SPECIES_FOMANTIS] = gMonIcon_Fomantis,
    [SPECIES_LURANTIS] = gMonIcon_Lurantis,
    [SPECIES_MORELULL] = gMonIcon_Morelull,
    [SPECIES_SHIINOTIC] = gMonIcon_Shiinotic,
    [SPECIES_SALANDIT] = gMonIcon_Salandit,
    [SPECIES_SALAZZLE] = gMonIcon_Salazzle,
    [SPECIES_STUFFUL] = gMonIcon_Stufful,
    [SPECIES_BEWEAR] = gMonIcon_Bewear,
    [SPECIES_BOUNSWEET] = gMonIcon_Bounsweet,
    [SPECIES_STEENEE] = gMonIcon_Steenee,
    [SPECIES_TSAREENA] = gMonIcon_Tsareena,
    [SPECIES_COMFEY] = gMonIcon_Comfey,
    [SPECIES_ORANGURU] = gMonIcon_Oranguru,
    [SPECIES_PASSIMIAN] = gMonIcon_Passimian,
    [SPECIES_WIMPOD] = gMonIcon_Wimpod,
    [SPECIES_GOLISOPOD] = gMonIcon_Golisopod,
    [SPECIES_SANDYGAST] = gMonIcon_Sandygast,
    [SPECIES_PALOSSAND] = gMonIcon_Palossand,
    [SPECIES_PYUKUMUKU] = gMonIcon_Pyukumuku,
    [SPECIES_TYPE_NULL] = gMonIcon_TypeNull,
    [SPECIES_SILVALLY] = gMonIcon_Silvally,
    [SPECIES_MINIOR] = gMonIcon_Minior,
    [SPECIES_KOMALA] = gMonIcon_Komala,
    [SPECIES_TURTONATOR] = gMonIcon_Turtonator,
    [SPECIES_TOGEDEMARU] = gMonIcon_Togedemaru,
    [SPECIES_MIMIKYU] = gMonIcon_Mimikyu,
    [SPECIES_BRUXISH] = gMonIcon_Bruxish,
    [SPECIES_DRAMPA] = gMonIcon_Drampa,
    [SPECIES_DHELMISE] = gMonIcon_Dhelmise,
    [SPECIES_JANGMO_O] = gMonIcon_Jangmoo,
    [SPECIES_HAKAMO_O] = gMonIcon_Hakamoo,
    [SPECIES_KOMMO_O] = gMonIcon_Kommoo,
    [SPECIES_TAPU_KOKO] = gMonIcon_TapuKoko,
    [SPECIES_TAPU_LELE] = gMonIcon_TapuLele,
    [SPECIES_TAPU_BULU] = gMonIcon_TapuBulu,
    [SPECIES_TAPU_FINI] = gMonIcon_TapuFini,
    [SPECIES_COSMOG] = gMonIcon_Cosmog,
    [SPECIES_COSMOEM] = gMonIcon_Cosmoem,
    [SPECIES_SOLGALEO] = gMonIcon_Solgaleo,
    [SPECIES_LUNALA] = gMonIcon_Lunala,
    [SPECIES_NIHILEGO] = gMonIcon_Nihilego,
    [SPECIES_BUZZWOLE] = gMonIcon_Buzzwole,
    [SPECIES_PHEROMOSA] = gMonIcon_Pheromosa,
    [SPECIES_XURKITREE] = gMonIcon_Xurkitree,
    [SPECIES_CELESTEELA] = gMonIcon_Celesteela,
    [SPECIES_KARTANA] = gMonIcon_Kartana,
    [SPECIES_GUZZLORD] = gMonIcon_Guzzlord,
    [SPECIES_NECROZMA] = gMonIcon_Necrozma,
    [SPECIES_MAGEARNA] = gMonIcon_Magearna,
    [SPECIES_MARSHADOW] = gMonIcon_Marshadow,
    [SPECIES_POIPOLE] = gMonIcon_Poipole,
    [SPECIES_NAGANADEL] = gMonIcon_Naganadel,
    [SPECIES_STAKATAKA] = gMonIcon_Stakataka,
    [SPECIES_BLACEPHALON] = gMonIcon_Blacephalon,
    [SPECIES_ZERAORA] = gMonIcon_Zeraora,
    [SPECIES_MELTAN] = gMonIcon_Meltan,
    [SPECIES_MELMETAL] = gMonIcon_Melmetal,
    [SPECIES_GROOKEY] = gMonIcon_QuestionMark,
    [SPECIES_THWACKEY] = gMonIcon_QuestionMark,
    [SPECIES_RILLABOOM] = gMonIcon_QuestionMark,
    [SPECIES_SCORBUNNY] = gMonIcon_QuestionMark,
    [SPECIES_RABOOT] = gMonIcon_QuestionMark,
    [SPECIES_CINDERACE] = gMonIcon_QuestionMark,
    [SPECIES_SOBBLE] = gMonIcon_QuestionMark,
    [SPECIES_DRIZZILE] = gMonIcon_QuestionMark,
    [SPECIES_INTELEON] = gMonIcon_QuestionMark,
    [SPECIES_SKWOVET] = gMonIcon_QuestionMark,
    [SPECIES_GREEDENT] = gMonIcon_QuestionMark,
    [SPECIES_ROOKIDEE] = gMonIcon_QuestionMark,
    [SPECIES_CORVISQUIRE] = gMonIcon_QuestionMark,
    [SPECIES_CORVIKNIGHT] = gMonIcon_QuestionMark,
    [SPECIES_BLIPBUG] = gMonIcon_QuestionMark,
    [SPECIES_DOTTLER] = gMonIcon_QuestionMark,
    [SPECIES_ORBEETLE] = gMonIcon_QuestionMark,
    [SPECIES_NICKIT] = gMonIcon_QuestionMark,
    [SPECIES_THIEVUL] = gMonIcon_QuestionMark,
    [SPECIES_GOSSIFLEUR] = gMonIcon_QuestionMark,
    [SPECIES_ELDEGOSS] = gMonIcon_QuestionMark,
    [SPECIES_WOOLOO] = gMonIcon_QuestionMark,
    [SPECIES_DUBWOOL] = gMonIcon_QuestionMark,
    [SPECIES_CHEWTLE] = gMonIcon_QuestionMark,
    [SPECIES_DREDNAW] = gMonIcon_QuestionMark,
    [SPECIES_YAMPER] = gMonIcon_QuestionMark,
    [SPECIES_BOLTUND] = gMonIcon_QuestionMark,
    [SPECIES_ROLYCOLY] = gMonIcon_QuestionMark,
    [SPECIES_CARKOL] = gMonIcon_QuestionMark,
    [SPECIES_COALOSSAL] = gMonIcon_QuestionMark,
    [SPECIES_APPLIN] = gMonIcon_QuestionMark,
    [SPECIES_FLAPPLE] = gMonIcon_QuestionMark,
    [SPECIES_APPLETUN] = gMonIcon_QuestionMark,
    [SPECIES_SILICOBRA] = gMonIcon_QuestionMark,
    [SPECIES_SANDACONDA] = gMonIcon_QuestionMark,
    [SPECIES_CRAMORANT] = gMonIcon_QuestionMark,
    [SPECIES_ARROKUDA] = gMonIcon_QuestionMark,
    [SPECIES_BARRASKEWDA] = gMonIcon_QuestionMark,
    [SPECIES_TOXEL] = gMonIcon_QuestionMark,
    [SPECIES_TOXTRICITY] = gMonIcon_QuestionMark,
    [SPECIES_SIZZLIPEDE] = gMonIcon_QuestionMark,
    [SPECIES_CENTISKORCH] = gMonIcon_QuestionMark,
    [SPECIES_CLOBBOPUS] = gMonIcon_QuestionMark,
    [SPECIES_GRAPPLOCT] = gMonIcon_QuestionMark,
    [SPECIES_SINISTEA] = gMonIcon_QuestionMark,
    [SPECIES_POLTEAGEIST] = gMonIcon_QuestionMark,
    [SPECIES_HATENNA] = gMonIcon_QuestionMark,
    [SPECIES_HATTREM] = gMonIcon_QuestionMark,
    [SPECIES_HATTERENE] = gMonIcon_QuestionMark,
    [SPECIES_IMPIDIMP] = gMonIcon_QuestionMark,
    [SPECIES_MORGREM] = gMonIcon_QuestionMark,
    [SPECIES_GRIMMSNARL] = gMonIcon_QuestionMark,
    [SPECIES_OBSTAGOON] = gMonIcon_QuestionMark,
    [SPECIES_PERRSERKER] = gMonIcon_QuestionMark,
    [SPECIES_CURSOLA] = gMonIcon_QuestionMark,
    [SPECIES_SIRFETCHD] = gMonIcon_QuestionMark,
    [SPECIES_MR_RIME] = gMonIcon_QuestionMark,
    [SPECIES_RUNERIGUS] = gMonIcon_QuestionMark,
    [SPECIES_MILCERY] = gMonIcon_QuestionMark,
    [SPECIES_ALCREMIE] = gMonIcon_QuestionMark,
    [SPECIES_FALINKS] = gMonIcon_QuestionMark,
    [SPECIES_PINCURCHIN] = gMonIcon_QuestionMark,
    [SPECIES_SNOM] = gMonIcon_QuestionMark,
    [SPECIES_FROSMOTH] = gMonIcon_QuestionMark,
    [SPECIES_STONJOURNER] = gMonIcon_QuestionMark,
    [SPECIES_EISCUE] = gMonIcon_QuestionMark,
    [SPECIES_INDEEDEE] = gMonIcon_QuestionMark,
    [SPECIES_MORPEKO] = gMonIcon_QuestionMark,
    [SPECIES_CUFANT] = gMonIcon_QuestionMark,
    [SPECIES_COPPERAJAH] = gMonIcon_QuestionMark,
    [SPECIES_DRACOZOLT] = gMonIcon_QuestionMark,
    [SPECIES_ARCTOZOLT] = gMonIcon_QuestionMark,
    [SPECIES_DRACOVISH] = gMonIcon_QuestionMark,
    [SPECIES_ARCTOVISH] = gMonIcon_QuestionMark,
    [SPECIES_DURALUDON] = gMonIcon_QuestionMark,
    [SPECIES_DREEPY] = gMonIcon_QuestionMark,
    [SPECIES_DRAKLOAK] = gMonIcon_QuestionMark,
    [SPECIES_DRAGAPULT] = gMonIcon_QuestionMark,
    [SPECIES_ZACIAN] = gMonIcon_QuestionMark,
    [SPECIES_ZAMAZENTA] = gMonIcon_QuestionMark,
    [SPECIES_ETERNATUS] = gMonIcon_QuestionMark,
    [SPECIES_KUBFU] = gMonIcon_QuestionMark,
    [SPECIES_URSHIFU] = gMonIcon_QuestionMark,
    [SPECIES_ZARUDE] = gMonIcon_QuestionMark,
    [SPECIES_REGIELEKI] = gMonIcon_QuestionMark,
    [SPECIES_REGIDRAGO] = gMonIcon_QuestionMark,
    [SPECIES_GLASTRIER] = gMonIcon_QuestionMark,
    [SPECIES_SPECTRIER] = gMonIcon_QuestionMark,
    [SPECIES_CALYREX] = gMonIcon_QuestionMark,
    [SPECIES_VENUSAUR_MEGA] = gMonIcon_VenusaurMega,
    [SPECIES_CHARIZARD_MEGA_X] = gMonIcon_CharizardMegaX,
    [SPECIES_CHARIZARD_MEGA_Y] = gMonIcon_CharizardMegaY,
    [SPECIES_BLASTOISE_MEGA] = gMonIcon_BlastoiseMega,
    [SPECIES_BEEDRILL_MEGA] = gMonIcon_BeedrillMega,
    [SPECIES_PIDGEOT_MEGA] = gMonIcon_PidgeotMega,
    [SPECIES_ALAKAZAM_MEGA] = gMonIcon_AlakazamMega,
    [SPECIES_SLOWBRO_MEGA] = gMonIcon_SlowbroMega,
    [SPECIES_GENGAR_MEGA] = gMonIcon_GengarMega,
    [SPECIES_KANGASKHAN_MEGA] = gMonIcon_KangaskhanMega,
    [SPECIES_PINSIR_MEGA] = gMonIcon_PinsirMega,
    [SPECIES_GYARADOS_MEGA] = gMonIcon_GyaradosMega,
    [SPECIES_AERODACTYL_MEGA] = gMonIcon_AerodactylMega,
    [SPECIES_MEWTWO_MEGA_X] = gMonIcon_MewtwoMegaX,
    [SPECIES_MEWTWO_MEGA_Y] = gMonIcon_MewtwoMegaY,
    [SPECIES_AMPHAROS_MEGA] = gMonIcon_AmpharosMega,
    [SPECIES_STEELIX_MEGA] = gMonIcon_SteelixMega,
    [SPECIES_SCIZOR_MEGA] = gMonIcon_ScizorMega,
    [SPECIES_HERACROSS_MEGA] = gMonIcon_HeracrossMega,
    [SPECIES_HOUNDOOM_MEGA] = gMonIcon_HoundoomMega,
    [SPECIES_TYRANITAR_MEGA] = gMonIcon_TyranitarMega,
    [SPECIES_SCEPTILE_MEGA] = gMonIcon_SceptileMega,
    [SPECIES_BLAZIKEN_MEGA] = gMonIcon_BlazikenMega,
    [SPECIES_SWAMPERT_MEGA] = gMonIcon_SwampertMega,
    [SPECIES_GARDEVOIR_MEGA] = gMonIcon_GardevoirMega,
    [SPECIES_SABLEYE_MEGA] = gMonIcon_SableyeMega,
    [SPECIES_MAWILE_MEGA] = gMonIcon_MawileMega,
    [SPECIES_AGGRON_MEGA] = gMonIcon_AggronMega,
    [SPECIES_MEDICHAM_MEGA] = gMonIcon_MedichamMega,
    [SPECIES_MANECTRIC_MEGA] = gMonIcon_ManectricMega,
    [SPECIES_SHARPEDO_MEGA] = gMonIcon_SharpedoMega,
    [SPECIES_CAMERUPT_MEGA] = gMonIcon_CameruptMega,
    [SPECIES_ALTARIA_MEGA] = gMonIcon_AltariaMega,
    [SPECIES_BANETTE_MEGA] = gMonIcon_BanetteMega,
    [SPECIES_ABSOL_MEGA] = gMonIcon_AbsolMega,
    [SPECIES_GLALIE_MEGA] = gMonIcon_GlalieMega,
    [SPECIES_SALAMENCE_MEGA] = gMonIcon_SalamenceMega,
    [SPECIES_METAGROSS_MEGA] = gMonIcon_MetagrossMega,
    [SPECIES_LATIAS_MEGA] = gMonIcon_LatiasMega,
    [SPECIES_LATIOS_MEGA] = gMonIcon_LatiosMega,
    [SPECIES_LOPUNNY_MEGA] = gMonIcon_LopunnyMega,
    [SPECIES_GARCHOMP_MEGA] = gMonIcon_GarchompMega,
    [SPECIES_LUCARIO_MEGA] = gMonIcon_LucarioMega,
    [SPECIES_ABOMASNOW_MEGA] = gMonIcon_AbomasnowMega,
    [SPECIES_GALLADE_MEGA] = gMonIcon_GalladeMega,
    [SPECIES_AUDINO_MEGA] = gMonIcon_AudinoMega,
    [SPECIES_DIANCIE_MEGA] = gMonIcon_DiancieMega,
    [SPECIES_RAYQUAZA_MEGA] = gMonIcon_RayquazaMega,
    [SPECIES_KYOGRE_PRIMAL] = gMonIcon_KyogrePrimal,
    [SPECIES_GROUDON_PRIMAL] = gMonIcon_GroudonPrimal,
    [SPECIES_RATTATA_ALOLAN] = gMonIcon_RattataAlolan,
    [SPECIES_RATICATE_ALOLAN] = gMonIcon_RaticateAlolan,
    [SPECIES_RAICHU_ALOLAN] = gMonIcon_RaichuAlolan,
    [SPECIES_SANDSHREW_ALOLAN] = gMonIcon_SandshrewAlolan,
    [SPECIES_SANDSLASH_ALOLAN] = gMonIcon_SandslashAlolan,
    [SPECIES_VULPIX_ALOLAN] = gMonIcon_VulpixAlolan,
    [SPECIES_NINETALES_ALOLAN] = gMonIcon_NinetalesAlolan,
    [SPECIES_DIGLETT_ALOLAN] = gMonIcon_DiglettAlolan,
    [SPECIES_DUGTRIO_ALOLAN] = gMonIcon_DugtrioAlolan,
    [SPECIES_MEOWTH_ALOLAN] = gMonIcon_MeowthAlolan,
    [SPECIES_PERSIAN_ALOLAN] = gMonIcon_PersianAlolan,
    [SPECIES_GEODUDE_ALOLAN] = gMonIcon_GeodudeAlolan,
    [SPECIES_GRAVELER_ALOLAN] = gMonIcon_GravelerAlolan,
    [SPECIES_GOLEM_ALOLAN] = gMonIcon_GolemAlolan,
    [SPECIES_GRIMER_ALOLAN] = gMonIcon_GrimerAlolan,
    [SPECIES_MUK_ALOLAN] = gMonIcon_MukAlolan,
    [SPECIES_EXEGGUTOR_ALOLAN] = gMonIcon_ExeggutorAlolan,
    [SPECIES_MAROWAK_ALOLAN] = gMonIcon_MarowakAlolan,
    [SPECIES_MEOWTH_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_PONYTA_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_RAPIDASH_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_SLOWPOKE_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_SLOWBRO_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_FARFETCHD_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_WEEZING_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_MR_MIME_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_ARTICUNO_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_ZAPDOS_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_MOLTRES_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_SLOWKING_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_CORSOLA_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_ZIGZAGOON_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_LINOONE_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_DARUMAKA_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_DARMANITAN_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_YAMASK_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_STUNFISK_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_PIKACHU_COSPLAY] = gMonIcon_PikachuCosplay,
    [SPECIES_PIKACHU_ROCK_STAR] = gMonIcon_PikachuRockStar,
    [SPECIES_PIKACHU_BELLE] = gMonIcon_PikachuBelle,
    [SPECIES_PIKACHU_POP_STAR] = gMonIcon_PikachuPopStar,
    [SPECIES_PIKACHU_PH_D] = gMonIcon_PikachuPhD,
    [SPECIES_PIKACHU_LIBRE] = gMonIcon_PikachuLibre,
    [SPECIES_PIKACHU_ORIGINAL_CAP] = gMonIcon_PikachuOriginalCap,
    [SPECIES_PIKACHU_HOENN_CAP] = gMonIcon_PikachuHoennCap,
    [SPECIES_PIKACHU_SINNOH_CAP] = gMonIcon_PikachuSinnohCap,
    [SPECIES_PIKACHU_UNOVA_CAP] = gMonIcon_PikachuUnovaCap,
    [SPECIES_PIKACHU_KALOS_CAP] = gMonIcon_PikachuKalosCap,
    [SPECIES_PIKACHU_ALOLA_CAP] = gMonIcon_PikachuAlolaCap,
    [SPECIES_PIKACHU_PARTNER_CAP] = gMonIcon_PikachuPartnerCap,
    [SPECIES_PIKACHU_WORLD_CAP] = gMonIcon_QuestionMark,
    [SPECIES_PICHU_SPIKY_EARED] = gMonIcon_PichuSpikyEared,
    [SPECIES_UNOWN_B] = gMonIcon_UnownB,
    [SPECIES_UNOWN_C] = gMonIcon_UnownC,
    [SPECIES_UNOWN_D] = gMonIcon_UnownD,
    [SPECIES_UNOWN_E] = gMonIcon_UnownE,
    [SPECIES_UNOWN_F] = gMonIcon_UnownF,
    [SPECIES_UNOWN_G] = gMonIcon_UnownG,
    [SPECIES_UNOWN_H] = gMonIcon_UnownH,
    [SPECIES_UNOWN_I] = gMonIcon_UnownI,
    [SPECIES_UNOWN_J] = gMonIcon_UnownJ,
    [SPECIES_UNOWN_K] = gMonIcon_UnownK,
    [SPECIES_UNOWN_L] = gMonIcon_UnownL,
    [SPECIES_UNOWN_M] = gMonIcon_UnownM,
    [SPECIES_UNOWN_N] = gMonIcon_UnownN,
    [SPECIES_UNOWN_O] = gMonIcon_UnownO,
    [SPECIES_UNOWN_P] = gMonIcon_UnownP,
    [SPECIES_UNOWN_Q] = gMonIcon_UnownQ,
    [SPECIES_UNOWN_R] = gMonIcon_UnownR,
    [SPECIES_UNOWN_S] = gMonIcon_UnownS,
    [SPECIES_UNOWN_T] = gMonIcon_UnownT,
    [SPECIES_UNOWN_U] = gMonIcon_UnownU,
    [SPECIES_UNOWN_V] = gMonIcon_UnownV,
    [SPECIES_UNOWN_W] = gMonIcon_UnownW,
    [SPECIES_UNOWN_X] = gMonIcon_UnownX,
    [SPECIES_UNOWN_Y] = gMonIcon_UnownY,
    [SPECIES_UNOWN_Z] = gMonIcon_UnownZ,
    [SPECIES_UNOWN_EMARK] = gMonIcon_UnownExclamationMark,
    [SPECIES_UNOWN_QMARK] = gMonIcon_UnownQuestionMark,
    [SPECIES_CASTFORM_SUNNY] = gMonIcon_CastformSunny,
    [SPECIES_CASTFORM_RAINY] = gMonIcon_CastformRainy,
    [SPECIES_CASTFORM_SNOWY] = gMonIcon_CastformSnowy,
    [SPECIES_DEOXYS_ATTACK] = gMonIcon_DeoxysAttack,
    [SPECIES_DEOXYS_DEFENSE] = gMonIcon_DeoxysDefense,
    [SPECIES_DEOXYS_SPEED] = gMonIcon_DeoxysSpeed,
    [SPECIES_BURMY_SANDY_CLOAK] = gMonIcon_BurmySandyCloak,
    [SPECIES_BURMY_TRASH_CLOAK] = gMonIcon_BurmyTrashCloak,
    [SPECIES_WORMADAM_SANDY_CLOAK] = gMonIcon_WormadamSandyCloak,
    [SPECIES_WORMADAM_TRASH_CLOAK] = gMonIcon_WormadamTrashCloak,
    [SPECIES_CHERRIM_SUNSHINE] = gMonIcon_CherrimSunshine,
    [SPECIES_SHELLOS_EAST_SEA] = gMonIcon_ShellosEastSea,
    [SPECIES_GASTRODON_EAST_SEA] = gMonIcon_GastrodonEastSea,
    [SPECIES_ROTOM_HEAT] = gMonIcon_RotomHeat,
    [SPECIES_ROTOM_WASH] = gMonIcon_RotomWash,
    [SPECIES_ROTOM_FROST] = gMonIcon_RotomFrost,
    [SPECIES_ROTOM_FAN] = gMonIcon_RotomFan,
    [SPECIES_ROTOM_MOW] = gMonIcon_RotomMow,
    [SPECIES_GIRATINA_ORIGIN] = gMonIcon_GiratinaOrigin,
    [SPECIES_SHAYMIN_SKY] = gMonIcon_ShayminSky,
    [SPECIES_ARCEUS_FIGHTING] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_FLYING] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_POISON] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_GROUND] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_ROCK] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_BUG] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_GHOST] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_STEEL] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_FIRE] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_WATER] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_GRASS] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_ELECTRIC] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_PSYCHIC] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_ICE] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_DRAGON] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_DARK] = gMonIcon_Arceus,
    [SPECIES_ARCEUS_FAIRY] = gMonIcon_Arceus,
    [SPECIES_BASCULIN_BLUE_STRIPED] = gMonIcon_BasculinBlueStriped,
    [SPECIES_DARMANITAN_ZEN_MODE] = gMonIcon_DarmanitanZenMode,
    [SPECIES_DARMANITAN_ZEN_MODE_GALARIAN] = gMonIcon_QuestionMark,
    [SPECIES_DEERLING_SUMMER] = gMonIcon_DeerlingSummer,
    [SPECIES_DEERLING_AUTUMN] = gMonIcon_DeerlingAutumn,
    [SPECIES_DEERLING_WINTER] = gMonIcon_DeerlingWinter,
    [SPECIES_SAWSBUCK_SUMMER] = gMonIcon_SawsbuckSummer,
    [SPECIES_SAWSBUCK_AUTUMN] = gMonIcon_SawsbuckAutumn,
    [SPECIES_SAWSBUCK_WINTER] = gMonIcon_SawsbuckWinter,
    [SPECIES_TORNADUS_THERIAN] = gMonIcon_TornadusTherian,
    [SPECIES_THUNDURUS_THERIAN] = gMonIcon_ThundurusTherian,
    [SPECIES_LANDORUS_THERIAN] = gMonIcon_LandorusTherian,
    [SPECIES_KYUREM_WHITE] = gMonIcon_KyuremWhite,
    [SPECIES_KYUREM_BLACK] = gMonIcon_KyuremBlack,
    [SPECIES_KELDEO_RESOLUTE] = gMonIcon_KeldeoResolute,
    [SPECIES_MELOETTA_PIROUETTE] = gMonIcon_MeloettaPirouette,
    [SPECIES_GENESECT_DOUSE_DRIVE] = gMonIcon_Genesect,
    [SPECIES_GENESECT_SHOCK_DRIVE] = gMonIcon_Genesect,
    [SPECIES_GENESECT_BURN_DRIVE] = gMonIcon_Genesect,
    [SPECIES_GENESECT_CHILL_DRIVE] = gMonIcon_Genesect,
    [SPECIES_GRENINJA_BATTLE_BOND] = gMonIcon_Greninja,
    [SPECIES_GRENINJA_ASH] = gMonIcon_GreninjaAsh,
    [SPECIES_VIVILLON_POLAR] = gMonIcon_VivillonPolar,
    [SPECIES_VIVILLON_TUNDRA] = gMonIcon_VivillonTundra,
    [SPECIES_VIVILLON_CONTINENTAL] = gMonIcon_VivillonContinental,
    [SPECIES_VIVILLON_GARDEN] = gMonIcon_VivillonGarden,
    [SPECIES_VIVILLON_ELEGANT] = gMonIcon_VivillonElegant,
    [SPECIES_VIVILLON_MEADOW] = gMonIcon_VivillonMeadow,
    [SPECIES_VIVILLON_MODERN] = gMonIcon_VivillonModern,
    [SPECIES_VIVILLON_MARINE] = gMonIcon_VivillonMarine,
    [SPECIES_VIVILLON_ARCHIPELAGO] = gMonIcon_VivillonArchipelago,
    [SPECIES_VIVILLON_HIGH_PLAINS] = gMonIcon_VivillonHighPlains,
    [SPECIES_VIVILLON_SANDSTORM] = gMonIcon_VivillonSandstorm,
    [SPECIES_VIVILLON_RIVER] = gMonIcon_VivillonRiver,
    [SPECIES_VIVILLON_MONSOON] = gMonIcon_VivillonMonsoon,
    [SPECIES_VIVILLON_SAVANNA] = gMonIcon_VivillonSavanna,
    [SPECIES_VIVILLON_SUN] = gMonIcon_VivillonSun,
    [SPECIES_VIVILLON_OCEAN] = gMonIcon_VivillonOcean,
    [SPECIES_VIVILLON_JUNGLE] = gMonIcon_VivillonJungle,
    [SPECIES_VIVILLON_FANCY] = gMonIcon_VivillonFancy,
    [SPECIES_VIVILLON_POKE_BALL] = gMonIcon_VivillonPokeBall,
    [SPECIES_FLABEBE_YELLOW_FLOWER] = gMonIcon_FlabebeYellowFlower,
    [SPECIES_FLABEBE_ORANGE_FLOWER] = gMonIcon_FlabebeOrangeFlower,
    [SPECIES_FLABEBE_BLUE_FLOWER] = gMonIcon_FlabebeBlueFlower,
    [SPECIES_FLABEBE_WHITE_FLOWER] = gMonIcon_FlabebeWhiteFlower,
    [SPECIES_FLOETTE_YELLOW_FLOWER] = gMonIcon_FloetteYellowFlower,
    [SPECIES_FLOETTE_ORANGE_FLOWER] = gMonIcon_FloetteOrangeFlower,
    [SPECIES_FLOETTE_BLUE_FLOWER] = gMonIcon_FloetteBlueFlower,
    [SPECIES_FLOETTE_WHITE_FLOWER] = gMonIcon_FloetteWhiteFlower,
    [SPECIES_FLOETTE_ETERNAL_FLOWER] = gMonIcon_FloetteEternalFlower,
    [SPECIES_FLORGES_YELLOW_FLOWER] = gMonIcon_FlorgesYellowFlower,
    [SPECIES_FLORGES_ORANGE_FLOWER] = gMonIcon_FlorgesOrangeFlower,
    [SPECIES_FLORGES_BLUE_FLOWER] = gMonIcon_FlorgesBlueFlower,
    [SPECIES_FLORGES_WHITE_FLOWER] = gMonIcon_FlorgesWhiteFlower,
    [SPECIES_FURFROU_HEART_TRIM] = gMonIcon_FurfrouHeartTrim,
    [SPECIES_FURFROU_STAR_TRIM] = gMonIcon_FurfrouStarTrim,
    [SPECIES_FURFROU_DIAMOND_TRIM] = gMonIcon_FurfrouDiamondTrim,
    [SPECIES_FURFROU_DEBUTANTE_TRIM] = gMonIcon_FurfrouDebutanteTrim,
    [SPECIES_FURFROU_MATRON_TRIM] = gMonIcon_FurfrouMatronTrim,
    [SPECIES_FURFROU_DANDY_TRIM] = gMonIcon_FurfrouDandyTrim,
    [SPECIES_FURFROU_LA_REINE_TRIM] = gMonIcon_FurfrouLaReineTrim,
    [SPECIES_FURFROU_KABUKI_TRIM] = gMonIcon_FurfrouKabukiTrim,
    [SPECIES_FURFROU_PHARAOH_TRIM] = gMonIcon_FurfrouPharaohTrim,
    [SPECIES_MEOWSTIC_FEMALE] = gMonIcon_MeowsticFemale,
    [SPECIES_AEGISLASH_BLADE] = gMonIcon_AegislashBlade,
    [SPECIES_PUMPKABOO_SMALL] = gMonIcon_Pumpkaboo,
    [SPECIES_PUMPKABOO_LARGE] = gMonIcon_Pumpkaboo,
    [SPECIES_PUMPKABOO_SUPER] = gMonIcon_Pumpkaboo,
    [SPECIES_GOURGEIST_SMALL] = gMonIcon_Gourgeist,
    [SPECIES_GOURGEIST_LARGE] = gMonIcon_Gourgeist,
    [SPECIES_GOURGEIST_SUPER] = gMonIcon_Gourgeist,
    [SPECIES_XERNEAS_ACTIVE] = gMonIcon_XerneasActive,
    [SPECIES_ZYGARDE_10] = gMonIcon_Zygarde10,
    [SPECIES_ZYGARDE_10_POWER_CONSTRUCT] = gMonIcon_Zygarde10,
    [SPECIES_ZYGARDE_50_POWER_CONSTRUCT] = gMonIcon_Zygarde,
    [SPECIES_ZYGARDE_COMPLETE] = gMonIcon_ZygardeComplete,
    [SPECIES_HOOPA_UNBOUND] = gMonIcon_HoopaUnbound,
    [SPECIES_ORICORIO_POM_POM] = gMonIcon_OricorioPomPom,
    [SPECIES_ORICORIO_PAU] = gMonIcon_OricorioPau,
    [SPECIES_ORICORIO_SENSU] = gMonIcon_OricorioSensu,
    [SPECIES_ROCKRUFF_OWN_TEMPO] = gMonIcon_Rockruff,
    [SPECIES_LYCANROC_MIDNIGHT] = gMonIcon_LycanrocMidnight,
    [SPECIES_LYCANROC_DUSK] = gMonIcon_LycanrocDusk,
    [SPECIES_WISHIWASHI_SCHOOL] = gMonIcon_WishiwashiSchool,
    [SPECIES_SILVALLY_FIGHTING] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_FLYING] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_POISON] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_GROUND] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_ROCK] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_BUG] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_GHOST] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_STEEL] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_FIRE] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_WATER] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_GRASS] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_ELECTRIC] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_PSYCHIC] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_ICE] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_DRAGON] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_DARK] = gMonIcon_Silvally,
    [SPECIES_SILVALLY_FAIRY] = gMonIcon_Silvally,
    [SPECIES_MINIOR_METEOR_ORANGE] = gMonIcon_Minior,
    [SPECIES_MINIOR_METEOR_YELLOW] = gMonIcon_Minior,
    [SPECIES_MINIOR_METEOR_GREEN] = gMonIcon_Minior,
    [SPECIES_MINIOR_METEOR_BLUE] = gMonIcon_Minior,
    [SPECIES_MINIOR_METEOR_INDIGO] = gMonIcon_Minior,
    [SPECIES_MINIOR_METEOR_VIOLET] = gMonIcon_Minior,
    [SPECIES_MINIOR_CORE_RED] = gMonIcon_MiniorCoreRed,
    [SPECIES_MINIOR_CORE_ORANGE] = gMonIcon_MiniorCoreOrange,
    [SPECIES_MINIOR_CORE_YELLOW] = gMonIcon_MiniorCoreYellow,
    [SPECIES_MINIOR_CORE_GREEN] = gMonIcon_MiniorCoreGreen,
    [SPECIES_MINIOR_CORE_BLUE] = gMonIcon_MiniorCoreBlue,
    [SPECIES_MINIOR_CORE_INDIGO] = gMonIcon_MiniorCoreIndigo,
    [SPECIES_MINIOR_CORE_VIOLET] = gMonIcon_MiniorCoreViolet,
    [SPECIES_MIMIKYU_BUSTED] = gMonIcon_MimikyuBusted,
    [SPECIES_NECROZMA_DUSK_MANE] = gMonIcon_NecrozmaDuskMane,
    [SPECIES_NECROZMA_DAWN_WINGS] = gMonIcon_NecrozmaDawnWings,
    [SPECIES_NECROZMA_ULTRA] = gMonIcon_NecrozmaUltra,
    [SPECIES_MAGEARNA_ORIGINAL_COLOR] = gMonIcon_MagearnaOriginalColor,
    [SPECIES_CRAMORANT_GULPING] = gMonIcon_QuestionMark,
    [SPECIES_CRAMORANT_GORGING] = gMonIcon_QuestionMark,
    [SPECIES_TOXTRICITY_LOW_KEY] = gMonIcon_QuestionMark,
    [SPECIES_SINISTEA_ANTIQUE] = gMonIcon_QuestionMark,
    [SPECIES_POLTEAGEIST_ANTIQUE] = gMonIcon_QuestionMark,
    [SPECIES_ALCREMIE_RUBY_CREAM] = gMonIcon_QuestionMark,
    [SPECIES_ALCREMIE_MATCHA_CREAM] = gMonIcon_QuestionMark,
    [SPECIES_ALCREMIE_MINT_CREAM] = gMonIcon_QuestionMark,
    [SPECIES_ALCREMIE_LEMON_CREAM] = gMonIcon_QuestionMark,
    [SPECIES_ALCREMIE_SALTED_CREAM] = gMonIcon_QuestionMark,
    [SPECIES_ALCREMIE_RUBY_SWIRL] = gMonIcon_QuestionMark,
    [SPECIES_ALCREMIE_CARAMEL_SWIRL] = gMonIcon_QuestionMark,
    [SPECIES_ALCREMIE_RAINBOW_SWIRL] = gMonIcon_QuestionMark,
    [SPECIES_EISCUE_NOICE_FACE] = gMonIcon_QuestionMark,
    [SPECIES_INDEEDEE_FEMALE] = gMonIcon_QuestionMark,
    [SPECIES_MORPEKO_HANGRY] = gMonIcon_QuestionMark,
    [SPECIES_ZACIAN_CROWNED_SWORD] = gMonIcon_QuestionMark,
    [SPECIES_ZAMAZENTA_CROWNED_SHIELD] = gMonIcon_QuestionMark,
    [SPECIES_ETERNATUS_ETERNAMAX] = gMonIcon_QuestionMark,
    [SPECIES_URSHIFU_RAPID_STRIKE_STYLE] = gMonIcon_QuestionMark,
    [SPECIES_ZARUDE_DADA] = gMonIcon_QuestionMark,
    [SPECIES_CALYREX_ICE_RIDER] = gMonIcon_QuestionMark,
    [SPECIES_CALYREX_SHADOW_RIDER] = gMonIcon_QuestionMark,
    [SPECIES_EGG] = gMonIcon_Egg,
};

const u8 *const gMonIconTableFemale[] =
{
    [SPECIES_EEVEE] = gMonIcon_Eevee,
};

const u8 gMonIconPaletteIndices[] =
{
    [SPECIES_BULBASAUR] = 4,
    [SPECIES_IVYSAUR] = 4,
    [SPECIES_VENUSAUR] = 4,
    [SPECIES_CHARMANDER] = 0,
    [SPECIES_CHARMELEON] = 0,
    [SPECIES_CHARIZARD] = 0,
    [SPECIES_SQUIRTLE] = 0,
    [SPECIES_WARTORTLE] = 2,
    [SPECIES_BLASTOISE] = 2,
    [SPECIES_CATERPIE] = 1,
    [SPECIES_METAPOD] = 1,
    [SPECIES_BUTTERFREE] = 0,
    [SPECIES_WEEDLE] = 2,
    [SPECIES_KAKUNA] = 2,
    [SPECIES_BEEDRILL] = 2,
    [SPECIES_PIDGEY] = 0,
    [SPECIES_PIDGEOTTO] = 0,
    [SPECIES_PIDGEOT] = 0,
    [SPECIES_RATTATA] = 2,
    [SPECIES_RATICATE] = 2,
    [SPECIES_SPEAROW] = 0,
    [SPECIES_FEAROW] = 0,
    [SPECIES_EKANS] = 2,
    [SPECIES_ARBOK] = 2,
    [SPECIES_PIKACHU] = 2,
    [SPECIES_RAICHU] = 0,
    [SPECIES_SANDSHREW] = 2,
    [SPECIES_SANDSLASH] = 2,
    [SPECIES_NIDORAN_F] = 0,
    [SPECIES_NIDORINA] = 0,
    [SPECIES_NIDOQUEEN] = 2,
    [SPECIES_NIDORAN_M] = 2,
    [SPECIES_NIDORINO] = 2,
    [SPECIES_NIDOKING] = 2,
    [SPECIES_CLEFAIRY] = 0,
    [SPECIES_CLEFABLE] = 0,
    [SPECIES_VULPIX] = 5,
    [SPECIES_NINETALES] = 3,
    [SPECIES_JIGGLYPUFF] = 0,
    [SPECIES_WIGGLYTUFF] = 0,
    [SPECIES_ZUBAT] = 2,
    [SPECIES_GOLBAT] = 2,
    [SPECIES_ODDISH] = 4,
    [SPECIES_GLOOM] = 0,
    [SPECIES_VILEPLUME] = 0,
    [SPECIES_PARAS] = 0,
    [SPECIES_PARASECT] = 0,
    [SPECIES_VENONAT] = 2,
    [SPECIES_VENOMOTH] = 2,
    [SPECIES_DIGLETT] = 2,
    [SPECIES_DUGTRIO] = 2,
    [SPECIES_MEOWTH] = 1,
    [SPECIES_PERSIAN] = 1,
    [SPECIES_PSYDUCK] = 1,
    [SPECIES_GOLDUCK] = 0,
    [SPECIES_MANKEY] = 1,
    [SPECIES_PRIMEAPE] = 2,
    [SPECIES_GROWLITHE] = 3,
    [SPECIES_ARCANINE] = 3,
    [SPECIES_POLIWAG] = 0,
    [SPECIES_POLIWHIRL] = 0,
    [SPECIES_POLIWRATH] = 0,
    [SPECIES_ABRA] = 2,
    [SPECIES_KADABRA] = 2,
    [SPECIES_ALAKAZAM] = 2,
    [SPECIES_MACHOP] = 0,
    [SPECIES_MACHOKE] = 2,
    [SPECIES_MACHAMP] = 0,
    [SPECIES_BELLSPROUT] = 1,
    [SPECIES_WEEPINBELL] = 1,
    [SPECIES_VICTREEBEL] = 1,
    [SPECIES_TENTACOOL] = 0,
    [SPECIES_TENTACRUEL] = 0,
    [SPECIES_GEODUDE] = 1,
    [SPECIES_GRAVELER] = 1,
    [SPECIES_GOLEM] = 2,
    [SPECIES_PONYTA] = 3,
    [SPECIES_RAPIDASH] = 3,
    [SPECIES_SLOWPOKE] = 0,
    [SPECIES_SLOWBRO] = 0,
    [SPECIES_MAGNEMITE] = 0,
    [SPECIES_MAGNETON] = 0,
    [SPECIES_FARFETCHD] = 1,
    [SPECIES_DODUO] = 2,
    [SPECIES_DODRIO] = 2,
    [SPECIES_SEEL] = 0,
    [SPECIES_DEWGONG] = 2,
    [SPECIES_GRIMER] = 2,
    [SPECIES_MUK] = 2,
    [SPECIES_SHELLDER] = 2,
    [SPECIES_CLOYSTER] = 2,
    [SPECIES_GASTLY] = 2,
    [SPECIES_HAUNTER] = 2,
    [SPECIES_GENGAR] = 2,
    [SPECIES_ONIX] = 2,
    [SPECIES_DROWZEE] = 2,
    [SPECIES_HYPNO] = 2,
    [SPECIES_KRABBY] = 0,
    [SPECIES_KINGLER] = 0,
    [SPECIES_VOLTORB] = 0,
    [SPECIES_ELECTRODE] = 0,
    [SPECIES_EXEGGCUTE] = 0,
    [SPECIES_EXEGGUTOR] = 1,
    [SPECIES_CUBONE] = 2,
    [SPECIES_MAROWAK] = 2,
    [SPECIES_HITMONLEE] = 2,
    [SPECIES_HITMONCHAN] = 2,
    [SPECIES_LICKITUNG] = 0,
    [SPECIES_KOFFING] = 2,
    [SPECIES_WEEZING] = 2,
    [SPECIES_RHYHORN] = 1,
    [SPECIES_RHYDON] = 1,
    [SPECIES_CHANSEY] = 0,
    [SPECIES_TANGELA] = 0,
    [SPECIES_KANGASKHAN] = 2,
    [SPECIES_HORSEA] = 0,
    [SPECIES_SEADRA] = 0,
    [SPECIES_GOLDEEN] = 0,
    [SPECIES_SEAKING] = 0,
    [SPECIES_STARYU] = 2,
    [SPECIES_STARMIE] = 2,
    [SPECIES_MR_MIME] = 0,
    [SPECIES_SCYTHER] = 1,
    [SPECIES_JYNX] = 2,
    [SPECIES_ELECTABUZZ] = 1,
    [SPECIES_MAGMAR] = 0,
    [SPECIES_PINSIR] = 2,
    [SPECIES_TAUROS] = 2,
    [SPECIES_MAGIKARP] = 0,
    [SPECIES_GYARADOS] = 0,
    [SPECIES_LAPRAS] = 2,
    [SPECIES_DITTO] = 2,
    [SPECIES_EEVEE] = 2,
    [SPECIES_VAPOREON] = 0,
    [SPECIES_JOLTEON] = 2,
    [SPECIES_FLAREON] = 3,
    [SPECIES_PORYGON] = 0,
    [SPECIES_OMANYTE] = 0,
    [SPECIES_OMASTAR] = 0,
    [SPECIES_KABUTO] = 2,
    [SPECIES_KABUTOPS] = 2,
    [SPECIES_AERODACTYL] = 2,
    [SPECIES_SNORLAX] = 3,
    [SPECIES_ARTICUNO] = 2,
    [SPECIES_ZAPDOS] = 0,
    [SPECIES_MOLTRES] = 0,
    [SPECIES_DRATINI] = 0,
    [SPECIES_DRAGONAIR] = 0,
    [SPECIES_DRAGONITE] = 2,
    [SPECIES_MEWTWO] = 2,
    [SPECIES_MEW] = 0,
    [SPECIES_CHIKORITA] = 1,
    [SPECIES_BAYLEEF] = 1,
    [SPECIES_MEGANIUM] = 1,
    [SPECIES_CYNDAQUIL] = 3,
    [SPECIES_QUILAVA] = 3,
    [SPECIES_TYPHLOSION] = 3,
    [SPECIES_TOTODILE] = 0,
    [SPECIES_CROCONAW] = 0,
    [SPECIES_FERALIGATR] = 0,
    [SPECIES_SENTRET] = 2,
    [SPECIES_FURRET] = 2,
    [SPECIES_HOOTHOOT] = 2,
    [SPECIES_NOCTOWL] = 2,
    [SPECIES_LEDYBA] = 0,
    [SPECIES_LEDIAN] = 0,
    [SPECIES_SPINARAK] = 1,
    [SPECIES_ARIADOS] = 0,
    [SPECIES_CROBAT] = 2,
    [SPECIES_CHINCHOU] = 2,
    [SPECIES_LANTURN] = 0,
    [SPECIES_PICHU] = 1,
    [SPECIES_CLEFFA] = 0,
    [SPECIES_IGGLYBUFF] = 1,
    [SPECIES_TOGEPI] = 0,
    [SPECIES_TOGETIC] = 0,
    [SPECIES_NATU] = 1,
    [SPECIES_XATU] = 1,
    [SPECIES_MAREEP] = 0,
    [SPECIES_FLAAFFY] = 0,
    [SPECIES_AMPHAROS] = 0,
    [SPECIES_BELLOSSOM] = 1,
    [SPECIES_MARILL] = 0,
    [SPECIES_AZUMARILL] = 0,
    [SPECIES_SUDOWOODO] = 1,
    [SPECIES_POLITOED] = 1,
    [SPECIES_HOPPIP] = 1,
    [SPECIES_SKIPLOOM] = 1,
    [SPECIES_JUMPLUFF] = 2,
    [SPECIES_AIPOM] = 2,
    [SPECIES_SUNKERN] = 1,
    [SPECIES_SUNFLORA] = 1,
    [SPECIES_YANMA] = 1,
    [SPECIES_WOOPER] = 0,
    [SPECIES_QUAGSIRE] = 0,
    [SPECIES_ESPEON] = 2,
    [SPECIES_UMBREON] = 0,
    [SPECIES_MURKROW] = 2,
    [SPECIES_SLOWKING] = 0,
    [SPECIES_MISDREAVUS] = 0,
    [SPECIES_UNOWN] = 0,
    [SPECIES_WOBBUFFET] = 0,
    [SPECIES_GIRAFARIG] = 1,
    [SPECIES_PINECO] = 0,
    [SPECIES_FORRETRESS] = 2,
    [SPECIES_DUNSPARCE] = 0,
    [SPECIES_GLIGAR] = 0,
    [SPECIES_STEELIX] = 0,
    [SPECIES_SNUBBULL] = 0,
    [SPECIES_GRANBULL] = 2,
    [SPECIES_QWILFISH] = 0,
    [SPECIES_SCIZOR] = 0,
    [SPECIES_SHUCKLE] = 1,
    [SPECIES_HERACROSS] = 0,
    [SPECIES_SNEASEL] = 0,
    [SPECIES_TEDDIURSA] = 0,
    [SPECIES_URSARING] = 2,
    [SPECIES_SLUGMA] = 0,
    [SPECIES_MAGCARGO] = 0,
    [SPECIES_SWINUB] = 2,
    [SPECIES_PILOSWINE] = 2,
    [SPECIES_CORSOLA] = 0,
    [SPECIES_REMORAID] = 0,
    [SPECIES_OCTILLERY] = 0,
    [SPECIES_DELIBIRD] = 1,
    [SPECIES_MANTINE] = 2,
    [SPECIES_SKARMORY] = 0,
    [SPECIES_HOUNDOUR] = 0,
    [SPECIES_HOUNDOOM] = 0,
    [SPECIES_KINGDRA] = 0,
    [SPECIES_PHANPY] = 0,
    [SPECIES_DONPHAN] = 0,
    [SPECIES_PORYGON2] = 0,
    [SPECIES_STANTLER] = 2,
    [SPECIES_SMEARGLE] = 1,
    [SPECIES_TYROGUE] = 2,
    [SPECIES_HITMONTOP] = 2,
    [SPECIES_SMOOCHUM] = 1,
    [SPECIES_ELEKID] = 1,
    [SPECIES_MAGBY] = 0,
    [SPECIES_MILTANK] = 0,
    [SPECIES_BLISSEY] = 0,
    [SPECIES_RAIKOU] = 2,
    [SPECIES_ENTEI] = 2,
    [SPECIES_SUICUNE] = 2,
    [SPECIES_LARVITAR] = 1,
    [SPECIES_PUPITAR] = 2,
    [SPECIES_TYRANITAR] = 4,
    [SPECIES_LUGIA] = 0,
    [SPECIES_HO_OH] = 1,
    [SPECIES_CELEBI] = 1,
    [SPECIES_TREECKO] = 1,
    [SPECIES_GROVYLE] = 1,
    [SPECIES_SCEPTILE] = 1,
    [SPECIES_TORCHIC] = 0,
    [SPECIES_COMBUSKEN] = 0,
    [SPECIES_BLAZIKEN] = 0,
    [SPECIES_MUDKIP] = 0,
    [SPECIES_MARSHTOMP] = 0,
    [SPECIES_SWAMPERT] = 0,
    [SPECIES_POOCHYENA] = 2,
    [SPECIES_MIGHTYENA] = 2,
    [SPECIES_ZIGZAGOON] = 2,
    [SPECIES_LINOONE] = 2,
    [SPECIES_WURMPLE] = 0,
    [SPECIES_SILCOON] = 2,
    [SPECIES_BEAUTIFLY] = 0,
    [SPECIES_CASCOON] = 2,
    [SPECIES_DUSTOX] = 5,
    [SPECIES_LOTAD] = 4,
    [SPECIES_LOMBRE] = 1,
    [SPECIES_LUDICOLO] = 1,
    [SPECIES_SEEDOT] = 2,
    [SPECIES_NUZLEAF] = 1,
    [SPECIES_SHIFTRY] = 5,
    [SPECIES_NINCADA] = 1,
    [SPECIES_NINJASK] = 1,
    [SPECIES_SHEDINJA] = 1,
    [SPECIES_TAILLOW] = 2,
    [SPECIES_SWELLOW] = 2,
    [SPECIES_SHROOMISH] = 1,
    [SPECIES_BRELOOM] = 1,
    [SPECIES_SPINDA] = 1,
    [SPECIES_WINGULL] = 0,
    [SPECIES_PELIPPER] = 2,
    [SPECIES_SURSKIT] = 0,
    [SPECIES_MASQUERAIN] = 0,
    [SPECIES_WAILMER] = 2,
    [SPECIES_WAILORD] = 0,
    [SPECIES_SKITTY] = 0,
    [SPECIES_DELCATTY] = 2,
    [SPECIES_KECLEON] = 1,
    [SPECIES_BALTOY] = 2,
    [SPECIES_CLAYDOL] = 0,
    [SPECIES_NOSEPASS] = 0,
    [SPECIES_TORKOAL] = 2,
    [SPECIES_SABLEYE] = 2,
    [SPECIES_BARBOACH] = 0,
    [SPECIES_WHISCASH] = 0,
    [SPECIES_LUVDISC] = 0,
    [SPECIES_CORPHISH] = 0,
    [SPECIES_CRAWDAUNT] = 0,
    [SPECIES_FEEBAS] = 2,
    [SPECIES_MILOTIC] = 2,
    [SPECIES_CARVANHA] = 0,
    [SPECIES_SHARPEDO] = 0,
    [SPECIES_TRAPINCH] = 0,
    [SPECIES_VIBRAVA] = 1,
    [SPECIES_FLYGON] = 1,
    [SPECIES_MAKUHITA] = 1,
    [SPECIES_HARIYAMA] = 2,
    [SPECIES_ELECTRIKE] = 1,
    [SPECIES_MANECTRIC] = 0,
    [SPECIES_NUMEL] = 1,
    [SPECIES_CAMERUPT] = 0,
    [SPECIES_SPHEAL] = 2,
    [SPECIES_SEALEO] = 2,
    [SPECIES_WALREIN] = 0,
    [SPECIES_CACNEA] = 1,
    [SPECIES_CACTURNE] = 1,
    [SPECIES_SNORUNT] = 2,
    [SPECIES_GLALIE] = 0,
    [SPECIES_LUNATONE] = 1,
    [SPECIES_SOLROCK] = 0,
    [SPECIES_AZURILL] = 0,
    [SPECIES_SPOINK] = 0,
    [SPECIES_GRUMPIG] = 2,
    [SPECIES_PLUSLE] = 0,
    [SPECIES_MINUN] = 0,
    [SPECIES_MAWILE] = 2,
    [SPECIES_MEDITITE] = 0,
    [SPECIES_MEDICHAM] = 0,
    [SPECIES_SWABLU] = 0,
    [SPECIES_ALTARIA] = 0,
    [SPECIES_WYNAUT] = 0,
    [SPECIES_DUSKULL] = 0,
    [SPECIES_DUSCLOPS] = 0,
    [SPECIES_ROSELIA] = 4,
    [SPECIES_SLAKOTH] = 2,
    [SPECIES_VIGOROTH] = 2,
    [SPECIES_SLAKING] = 2,
    [SPECIES_GULPIN] = 1,
    [SPECIES_SWALOT] = 2,
    [SPECIES_TROPIUS] = 1,
    [SPECIES_WHISMUR] = 1,
    [SPECIES_LOUDRED] = 2,
    [SPECIES_EXPLOUD] = 2,
    [SPECIES_CLAMPERL] = 0,
    [SPECIES_HUNTAIL] = 0,
    [SPECIES_GOREBYSS] = 0,
    [SPECIES_ABSOL] = 0,
    [SPECIES_SHUPPET] = 0,
    [SPECIES_BANETTE] = 0,
    [SPECIES_SEVIPER] = 2,
    [SPECIES_ZANGOOSE] = 0,
    [SPECIES_RELICANTH] = 2,
    [SPECIES_ARON] = 2,
    [SPECIES_LAIRON] = 2,
    [SPECIES_AGGRON] = 2,
    [SPECIES_CASTFORM] = 0,
    [SPECIES_VOLBEAT] = 0,
    [SPECIES_ILLUMISE] = 2,
    [SPECIES_LILEEP] = 2,
    [SPECIES_CRADILY] = 1,
    [SPECIES_ANORITH] = 0,
    [SPECIES_ARMALDO] = 2,
    [SPECIES_RALTS] = 1,
    [SPECIES_KIRLIA] = 1,
    [SPECIES_GARDEVOIR] = 1,
    [SPECIES_BAGON] = 0,
    [SPECIES_SHELGON] = 2,
    [SPECIES_SALAMENCE] = 0,
    [SPECIES_BELDUM] = 0,
    [SPECIES_METANG] = 0,
    [SPECIES_METAGROSS] = 0,
    [SPECIES_REGIROCK] = 2,
    [SPECIES_REGICE] = 0,
    [SPECIES_REGISTEEL] = 2,
    [SPECIES_KYOGRE] = 2,
    [SPECIES_GROUDON] = 0,
    [SPECIES_RAYQUAZA] = 1,
    [SPECIES_LATIAS] = 0,
    [SPECIES_LATIOS] = 0,
    [SPECIES_JIRACHI] = 0,
    [SPECIES_DEOXYS] = 0,
    [SPECIES_CHIMECHO] = 0,
    [SPECIES_TURTWIG] = 1,
    [SPECIES_GROTLE] = 1,
    [SPECIES_TORTERRA] = 1,
    [SPECIES_CHIMCHAR] = 1,
    [SPECIES_MONFERNO] = 0,
    [SPECIES_INFERNAPE] = 0,
    [SPECIES_PIPLUP] = 0,
    [SPECIES_PRINPLUP] = 0,
    [SPECIES_EMPOLEON] = 0,
    [SPECIES_STARLY] = 0,
    [SPECIES_STARAVIA] = 0,
    [SPECIES_STARAPTOR] = 0,
    [SPECIES_BIDOOF] = 2,
    [SPECIES_BIBAREL] = 2,
    [SPECIES_KRICKETOT] = 2,
    [SPECIES_KRICKETUNE] = 2,
    [SPECIES_SHINX] = 0,
    [SPECIES_LUXIO] = 0,
    [SPECIES_LUXRAY] = 0,
    [SPECIES_BUDEW] = 1,
    [SPECIES_ROSERADE] = 0,
    [SPECIES_CRANIDOS] = 0,
    [SPECIES_RAMPARDOS] = 0,
    [SPECIES_SHIELDON] = 1,
    [SPECIES_BASTIODON] = 1,
    [SPECIES_BURMY] = 1,
    [SPECIES_WORMADAM] = 1,
    [SPECIES_MOTHIM] = 0,
    [SPECIES_COMBEE] = 0,
    [SPECIES_VESPIQUEN] = 0,
    [SPECIES_PACHIRISU] = 0,
    [SPECIES_BUIZEL] = 0,
    [SPECIES_FLOATZEL] = 0,
    [SPECIES_CHERUBI] = 1,
    [SPECIES_CHERRIM] = 0,
    [SPECIES_SHELLOS] = 0,
    [SPECIES_GASTRODON] = 0,
    [SPECIES_AMBIPOM] = 2,
    [SPECIES_DRIFLOON] = 2,
    [SPECIES_DRIFBLIM] = 2,
    [SPECIES_BUNEARY] = 2,
    [SPECIES_LOPUNNY] = 2,
    [SPECIES_MISMAGIUS] = 2,
    [SPECIES_HONCHKROW] = 2,
    [SPECIES_GLAMEOW] = 0,
    [SPECIES_PURUGLY] = 0,
    [SPECIES_CHINGLING] = 1,
    [SPECIES_STUNKY] = 2,
    [SPECIES_SKUNTANK] = 2,
    [SPECIES_BRONZOR] = 0,
    [SPECIES_BRONZONG] = 0,
    [SPECIES_BONSLY] = 1,
    [SPECIES_MIME_JR] = 0,
    [SPECIES_HAPPINY] = 0,
    [SPECIES_CHATOT] = 0,
    [SPECIES_SPIRITOMB] = 5,
    [SPECIES_GIBLE] = 0,
    [SPECIES_GABITE] = 0,
    [SPECIES_GARCHOMP] = 0,
    [SPECIES_MUNCHLAX] = 3,
    [SPECIES_RIOLU] = 2,
    [SPECIES_LUCARIO] = 2,
    [SPECIES_HIPPOPOTAS] = 1,
    [SPECIES_HIPPOWDON] = 1,
    [SPECIES_SKORUPI] = 0,
    [SPECIES_DRAPION] = 2,
    [SPECIES_CROAGUNK] = 0,
    [SPECIES_TOXICROAK] = 0,
    [SPECIES_CARNIVINE] = 1,
    [SPECIES_FINNEON] = 0,
    [SPECIES_LUMINEON] = 0,
    [SPECIES_MANTYKE] = 0,
    [SPECIES_SNOVER] = 1,
    [SPECIES_ABOMASNOW] = 1,
    [SPECIES_WEAVILE] = 0,
    [SPECIES_MAGNEZONE] = 0,
    [SPECIES_LICKILICKY] = 1,
    [SPECIES_RHYPERIOR] = 0,
    [SPECIES_TANGROWTH] = 0,
    [SPECIES_ELECTIVIRE] = 1,
    [SPECIES_MAGMORTAR] = 0,
    [SPECIES_TOGEKISS] = 2,
    [SPECIES_YANMEGA] = 1,
    [SPECIES_LEAFEON] = 1,
    [SPECIES_GLACEON] = 0,
    [SPECIES_GLISCOR] = 2,
    [SPECIES_MAMOSWINE] = 2,
    [SPECIES_PORYGON_Z] = 0,
    [SPECIES_GALLADE] = 1,
    [SPECIES_PROBOPASS] = 0,
    [SPECIES_DUSKNOIR] = 2,
    [SPECIES_FROSLASS] = 0,
    [SPECIES_ROTOM] = 0,
    [SPECIES_UXIE] = 0,
    [SPECIES_MESPRIT] = 0,
    [SPECIES_AZELF] = 0,
    [SPECIES_DIALGA] = 2,
    [SPECIES_PALKIA] = 2,
    [SPECIES_HEATRAN] = 0,
    [SPECIES_REGIGIGAS] = 0,
    [SPECIES_GIRATINA] = 0,
    [SPECIES_CRESSELIA] = 0,
    [SPECIES_PHIONE] = 0,
    [SPECIES_MANAPHY] = 0,
    [SPECIES_DARKRAI] = 0,
    [SPECIES_SHAYMIN] = 1,
    [SPECIES_ARCEUS] = 1,
    [SPECIES_VICTINI] = 0,
    [SPECIES_SNIVY] = 1,
    [SPECIES_SERVINE] = 1,
    [SPECIES_SERPERIOR] = 1,
    [SPECIES_TEPIG] = 0,
    [SPECIES_PIGNITE] = 0,
    [SPECIES_EMBOAR] = 0,
    [SPECIES_OSHAWOTT] = 0,
    [SPECIES_DEWOTT] = 0,
    [SPECIES_SAMUROTT] = 2,
    [SPECIES_PATRAT] = 2,
    [SPECIES_WATCHOG] = 2,
    [SPECIES_LILLIPUP] = 2,
    [SPECIES_HERDIER] = 2,
    [SPECIES_STOUTLAND] = 2,
    [SPECIES_PURRLOIN] = 0,
    [SPECIES_LIEPARD] = 0,
    [SPECIES_PANSAGE] = 1,
    [SPECIES_SIMISAGE] = 1,
    [SPECIES_PANSEAR] = 2,
    [SPECIES_SIMISEAR] = 2,
    [SPECIES_PANPOUR] = 2,
    [SPECIES_SIMIPOUR] = 2,
    [SPECIES_MUNNA] = 0,
    [SPECIES_MUSHARNA] = 0,
    [SPECIES_PIDOVE] = 0,
    [SPECIES_TRANQUILL] = 0,
    [SPECIES_UNFEZANT] = 1,
    [SPECIES_BLITZLE] = 2,
    [SPECIES_ZEBSTRIKA] = 2,
    [SPECIES_ROGGENROLA] = 2,
    [SPECIES_BOLDORE] = 0,
    [SPECIES_GIGALITH] = 0,
    [SPECIES_WOOBAT] = 0,
    [SPECIES_SWOOBAT] = 0,
    [SPECIES_DRILBUR] = 0,
    [SPECIES_EXCADRILL] = 0,
    [SPECIES_AUDINO] = 1,
    [SPECIES_TIMBURR] = 1,
    [SPECIES_GURDURR] = 1,
    [SPECIES_CONKELDURR] = 1,
    [SPECIES_TYMPOLE] = 2,
    [SPECIES_PALPITOAD] = 2,
    [SPECIES_SEISMITOAD] = 0,
    [SPECIES_THROH] = 0,
    [SPECIES_SAWK] = 0,
    [SPECIES_SEWADDLE] = 1,
    [SPECIES_SWADLOON] = 1,
    [SPECIES_LEAVANNY] = 1,
    [SPECIES_VENIPEDE] = 1,
    [SPECIES_WHIRLIPEDE] = 2,
    [SPECIES_SCOLIPEDE] = 2,
    [SPECIES_COTTONEE] = 1,
    [SPECIES_WHIMSICOTT] = 1,
    [SPECIES_PETILIL] = 1,
    [SPECIES_LILLIGANT] = 1,
    [SPECIES_BASCULIN] = 1,
    [SPECIES_SANDILE] = 1,
    [SPECIES_KROKOROK] = 1,
    [SPECIES_KROOKODILE] = 0,
    [SPECIES_DARUMAKA] = 0,
    [SPECIES_DARMANITAN] = 0,
    [SPECIES_MARACTUS] = 1,
    [SPECIES_DWEBBLE] = 0,
    [SPECIES_CRUSTLE] = 2,
    [SPECIES_SCRAGGY] = 2,
    [SPECIES_SCRAFTY] = 0,
    [SPECIES_SIGILYPH] = 0,
    [SPECIES_YAMASK] = 0,
    [SPECIES_COFAGRIGUS] = 0,
    [SPECIES_TIRTOUGA] = 2,
    [SPECIES_CARRACOSTA] = 2,
    [SPECIES_ARCHEN] = 0,
    [SPECIES_ARCHEOPS] = 0,
    [SPECIES_TRUBBISH] = 1,
    [SPECIES_GARBODOR] = 1,
    [SPECIES_ZORUA] = 0,
    [SPECIES_ZOROARK] = 0,
    [SPECIES_MINCCINO] = 0,
    [SPECIES_CINCCINO] = 0,
    [SPECIES_GOTHITA] = 2,
    [SPECIES_GOTHORITA] = 2,
    [SPECIES_GOTHITELLE] = 2,
    [SPECIES_SOLOSIS] = 1,
    [SPECIES_DUOSION] = 1,
    [SPECIES_REUNICLUS] = 1,
    [SPECIES_DUCKLETT] = 0,
    [SPECIES_SWANNA] = 2,
    [SPECIES_VANILLITE] = 0,
    [SPECIES_VANILLISH] = 2,
    [SPECIES_VANILLUXE] = 2,
    [SPECIES_DEERLING] = 1,
    [SPECIES_SAWSBUCK] = 1,
    [SPECIES_EMOLGA] = 2,
    [SPECIES_KARRABLAST] = 0,
    [SPECIES_ESCAVALIER] = 0,
    [SPECIES_FOONGUS] = 0,
    [SPECIES_AMOONGUSS] = 1,
    [SPECIES_FRILLISH] = 0,
    [SPECIES_JELLICENT] = 0,
    [SPECIES_ALOMOMOLA] = 0,
    [SPECIES_JOLTIK] = 0,
    [SPECIES_GALVANTULA] = 2,
    [SPECIES_FERROSEED] = 1,
    [SPECIES_FERROTHORN] = 1,
    [SPECIES_KLINK] = 0,
    [SPECIES_KLANG] = 0,
    [SPECIES_KLINKLANG] = 0,
    [SPECIES_TYNAMO] = 0,
    [SPECIES_EELEKTRIK] = 0,
    [SPECIES_EELEKTROSS] = 0,
    [SPECIES_ELGYEM] = 0,
    [SPECIES_BEHEEYEM] = 2,
    [SPECIES_LITWICK] = 2,
    [SPECIES_LAMPENT] = 2,
    [SPECIES_CHANDELURE] = 2,
    [SPECIES_AXEW] = 1,
    [SPECIES_FRAXURE] = 1,
    [SPECIES_HAXORUS] = 2,
    [SPECIES_CUBCHOO] = 0,
    [SPECIES_BEARTIC] = 0,
    [SPECIES_CRYOGONAL] = 0,
    [SPECIES_SHELMET] = 1,
    [SPECIES_ACCELGOR] = 1,
    [SPECIES_STUNFISK] = 2,
    [SPECIES_MIENFOO] = 1,
    [SPECIES_MIENSHAO] = 2,
    [SPECIES_DRUDDIGON] = 0,
    [SPECIES_GOLETT] = 0,
    [SPECIES_GOLURK] = 0,
    [SPECIES_PAWNIARD] = 0,
    [SPECIES_BISHARP] = 0,
    [SPECIES_BOUFFALANT] = 2,
    [SPECIES_RUFFLET] = 2,
    [SPECIES_BRAVIARY] = 0,
    [SPECIES_VULLABY] = 0,
    [SPECIES_MANDIBUZZ] = 1,
    [SPECIES_HEATMOR] = 2,
    [SPECIES_DURANT] = 0,
    [SPECIES_DEINO] = 2,
    [SPECIES_ZWEILOUS] = 2,
    [SPECIES_HYDREIGON] = 2,
    [SPECIES_LARVESTA] = 0,
    [SPECIES_VOLCARONA] = 0,
    [SPECIES_COBALION] = 0,
    [SPECIES_TERRAKION] = 2,
    [SPECIES_VIRIZION] = 1,
    [SPECIES_TORNADUS] = 1,
    [SPECIES_THUNDURUS] = 0,
    [SPECIES_RESHIRAM] = 0,
    [SPECIES_ZEKROM] = 2,
    [SPECIES_LANDORUS] = 0,
    [SPECIES_KYUREM] = 0,
    [SPECIES_KELDEO] = 0,
    [SPECIES_MELOETTA] = 4,
    [SPECIES_GENESECT] = 2,
    [SPECIES_CHESPIN] = 1,
    [SPECIES_QUILLADIN] = 1,
    [SPECIES_CHESNAUGHT] = 1,
    [SPECIES_FENNEKIN] = 0,
    [SPECIES_BRAIXEN] = 0,
    [SPECIES_DELPHOX] = 0,
    [SPECIES_FROAKIE] = 0,
    [SPECIES_FROGADIER] = 0,
    [SPECIES_GRENINJA] = 0,
    [SPECIES_BUNNELBY] = 2,
    [SPECIES_DIGGERSBY] = 2,
    [SPECIES_FLETCHLING] = 2,
    [SPECIES_FLETCHINDER] = 2,
    [SPECIES_TALONFLAME] = 2,
    [SPECIES_SCATTERBUG] = 1,
    [SPECIES_SPEWPA] = 1,
    [SPECIES_VIVILLON] = 0,
    [SPECIES_LITLEO] = 2,
    [SPECIES_PYROAR] = 2,
    [SPECIES_FLABEBE] = 1,
    [SPECIES_FLOETTE] = 1,
    [SPECIES_FLORGES] = 0,
    [SPECIES_SKIDDO] = 1,
    [SPECIES_GOGOAT] = 1,
    [SPECIES_PANCHAM] = 1,
    [SPECIES_PANGORO] = 1,
    [SPECIES_FURFROU] = 0,
    [SPECIES_ESPURR] = 2,
    [SPECIES_MEOWSTIC] = 0,
    [SPECIES_HONEDGE] = 2,
    [SPECIES_DOUBLADE] = 2,
    [SPECIES_AEGISLASH] = 2,
    [SPECIES_SPRITZEE] = 0,
    [SPECIES_AROMATISSE] = 0,
    [SPECIES_SWIRLIX] = 1,
    [SPECIES_SLURPUFF] = 1,
    [SPECIES_INKAY] = 0,
    [SPECIES_MALAMAR] = 2,
    [SPECIES_BINACLE] = 2,
    [SPECIES_BARBARACLE] = 2,
    [SPECIES_SKRELP] = 2,
    [SPECIES_DRAGALGE] = 5,
    [SPECIES_CLAUNCHER] = 0,
    [SPECIES_CLAWITZER] = 0,
    [SPECIES_HELIOPTILE] = 2,
    [SPECIES_HELIOLISK] = 2,
    [SPECIES_TYRUNT] = 2,
    [SPECIES_TYRANTRUM] = 0,
    [SPECIES_AMAURA] = 0,
    [SPECIES_AURORUS] = 0,
    [SPECIES_SYLVEON] = 0,
    [SPECIES_HAWLUCHA] = 0,
    [SPECIES_DEDENNE] = 0,
    [SPECIES_CARBINK] = 2,
    [SPECIES_GOOMY] = 5,
    [SPECIES_SLIGGOO] = 5,
    [SPECIES_GOODRA] = 5,
    [SPECIES_KLEFKI] = 0,
    [SPECIES_PHANTUMP] = 1,
    [SPECIES_TREVENANT] = 1,
    [SPECIES_PUMPKABOO] = 2,
    [SPECIES_GOURGEIST] = 2,
    [SPECIES_BERGMITE] = 0,
    [SPECIES_AVALUGG] = 0,
    [SPECIES_NOIBAT] = 2,
    [SPECIES_NOIVERN] = 2,
    [SPECIES_XERNEAS] = 0,
    [SPECIES_YVELTAL] = 0,
    [SPECIES_ZYGARDE] = 1,
    [SPECIES_DIANCIE] = 1,
    [SPECIES_HOOPA] = 0,
    [SPECIES_VOLCANION] = 0,
    [SPECIES_ROWLET] = 0,
    [SPECIES_DARTRIX] = 1,
    [SPECIES_DECIDUEYE] = 1,
    [SPECIES_LITTEN] = 0,
    [SPECIES_TORRACAT] = 0,
    [SPECIES_INCINEROAR] = 0,
    [SPECIES_POPPLIO] = 0,
    [SPECIES_BRIONNE] = 0,
    [SPECIES_PRIMARINA] = 0,
    [SPECIES_PIKIPEK] = 2,
    [SPECIES_TRUMBEAK] = 0,
    [SPECIES_TOUCANNON] = 0,
    [SPECIES_YUNGOOS] = 2,
    [SPECIES_GUMSHOOS] = 2,
    [SPECIES_GRUBBIN] = 0,
    [SPECIES_CHARJABUG] = 1,
    [SPECIES_VIKAVOLT] = 0,
    [SPECIES_CRABRAWLER] = 2,
    [SPECIES_CRABOMINABLE] = 2,
    [SPECIES_ORICORIO] = 0,
    [SPECIES_CUTIEFLY] = 2,
    [SPECIES_RIBOMBEE] = 2,
    [SPECIES_ROCKRUFF] = 2,
    [SPECIES_LYCANROC] = 2,
    [SPECIES_WISHIWASHI] = 2,
    [SPECIES_MAREANIE] = 2,
    [SPECIES_TOXAPEX] = 0,
    [SPECIES_MUDBRAY] = 2,
    [SPECIES_MUDSDALE] = 0,
    [SPECIES_DEWPIDER] = 0,
    [SPECIES_ARAQUANID] = 2,
    [SPECIES_FOMANTIS] = 1,
    [SPECIES_LURANTIS] = 1,
    [SPECIES_MORELULL] = 0,
    [SPECIES_SHIINOTIC] = 1,
    [SPECIES_SALANDIT] = 2,
    [SPECIES_SALAZZLE] = 0,
    [SPECIES_STUFFUL] = 0,
    [SPECIES_BEWEAR] = 0,
    [SPECIES_BOUNSWEET] = 1,
    [SPECIES_STEENEE] = 1,
    [SPECIES_TSAREENA] = 1,
    [SPECIES_COMFEY] = 1,
    [SPECIES_ORANGURU] = 0,
    [SPECIES_PASSIMIAN] = 1,
    [SPECIES_WIMPOD] = 2,
    [SPECIES_GOLISOPOD] = 2,
    [SPECIES_SANDYGAST] = 1,
    [SPECIES_PALOSSAND] = 2,
    [SPECIES_PYUKUMUKU] = 0,
    [SPECIES_TYPE_NULL] = 0,
    [SPECIES_SILVALLY] = 0,
    [SPECIES_MINIOR] = 0,
    [SPECIES_KOMALA] = 2,
    [SPECIES_TURTONATOR] = 0,
    [SPECIES_TOGEDEMARU] = 2,
    [SPECIES_MIMIKYU] = 1,
    [SPECIES_BRUXISH] = 0,
    [SPECIES_DRAMPA] = 0,
    [SPECIES_DHELMISE] = 1,
    [SPECIES_JANGMO_O] = 2,
    [SPECIES_HAKAMO_O] = 2,
    [SPECIES_KOMMO_O] = 2,
    [SPECIES_TAPU_KOKO] = 0,
    [SPECIES_TAPU_LELE] = 0,
    [SPECIES_TAPU_BULU] = 2,
    [SPECIES_TAPU_FINI] = 0,
    [SPECIES_COSMOG] = 2,
    [SPECIES_COSMOEM] = 0,
    [SPECIES_SOLGALEO] = 0,
    [SPECIES_LUNALA] = 2,
    [SPECIES_NIHILEGO] = 0,
    [SPECIES_BUZZWOLE] = 0,
    [SPECIES_PHEROMOSA] = 2,
    [SPECIES_XURKITREE] = 0,
    [SPECIES_CELESTEELA] = 0,
    [SPECIES_KARTANA] = 0,
    [SPECIES_GUZZLORD] = 0,
    [SPECIES_NECROZMA] = 0,
    [SPECIES_MAGEARNA] = 0,
    [SPECIES_MARSHADOW] = 0,
    [SPECIES_POIPOLE] = 0,
    [SPECIES_NAGANADEL] = 0,
    [SPECIES_STAKATAKA] = 0,
    [SPECIES_BLACEPHALON] = 0,
    [SPECIES_ZERAORA] = 0,
    [SPECIES_MELTAN] = 2,
    [SPECIES_MELMETAL] = 2,
    [SPECIES_GROOKEY] = 0,
    [SPECIES_THWACKEY] = 0,
    [SPECIES_RILLABOOM] = 0,
    [SPECIES_SCORBUNNY] = 0,
    [SPECIES_RABOOT] = 0,
    [SPECIES_CINDERACE] = 0,
    [SPECIES_SOBBLE] = 0,
    [SPECIES_DRIZZILE] = 0,
    [SPECIES_INTELEON] = 0,
    [SPECIES_SKWOVET] = 0,
    [SPECIES_GREEDENT] = 0,
    [SPECIES_ROOKIDEE] = 0,
    [SPECIES_CORVISQUIRE] = 0,
    [SPECIES_CORVIKNIGHT] = 0,
    [SPECIES_BLIPBUG] = 0,
    [SPECIES_DOTTLER] = 0,
    [SPECIES_ORBEETLE] = 0,
    [SPECIES_NICKIT] = 0,
    [SPECIES_THIEVUL] = 0,
    [SPECIES_GOSSIFLEUR] = 0,
    [SPECIES_ELDEGOSS] = 0,
    [SPECIES_WOOLOO] = 0,
    [SPECIES_DUBWOOL] = 0,
    [SPECIES_CHEWTLE] = 0,
    [SPECIES_DREDNAW] = 0,
    [SPECIES_YAMPER] = 0,
    [SPECIES_BOLTUND] = 0,
    [SPECIES_ROLYCOLY] = 0,
    [SPECIES_CARKOL] = 0,
    [SPECIES_COALOSSAL] = 0,
    [SPECIES_APPLIN] = 0,
    [SPECIES_FLAPPLE] = 0,
    [SPECIES_APPLETUN] = 0,
    [SPECIES_SILICOBRA] = 0,
    [SPECIES_SANDACONDA] = 0,
    [SPECIES_CRAMORANT] = 0,
    [SPECIES_ARROKUDA] = 0,
    [SPECIES_BARRASKEWDA] = 0,
    [SPECIES_TOXEL] = 0,
    [SPECIES_TOXTRICITY] = 0,
    [SPECIES_SIZZLIPEDE] = 0,
    [SPECIES_CENTISKORCH] = 0,
    [SPECIES_CLOBBOPUS] = 0,
    [SPECIES_GRAPPLOCT] = 0,
    [SPECIES_SINISTEA] = 0,
    [SPECIES_POLTEAGEIST] = 0,
    [SPECIES_HATENNA] = 0,
    [SPECIES_HATTREM] = 0,
    [SPECIES_HATTERENE] = 0,
    [SPECIES_IMPIDIMP] = 0,
    [SPECIES_MORGREM] = 0,
    [SPECIES_GRIMMSNARL] = 0,
    [SPECIES_OBSTAGOON] = 0,
    [SPECIES_PERRSERKER] = 0,
    [SPECIES_CURSOLA] = 0,
    [SPECIES_SIRFETCHD] = 0,
    [SPECIES_MR_RIME] = 0,
    [SPECIES_RUNERIGUS] = 0,
    [SPECIES_MILCERY] = 0,
    [SPECIES_ALCREMIE] = 0,
    [SPECIES_FALINKS] = 0,
    [SPECIES_PINCURCHIN] = 0,
    [SPECIES_SNOM] = 0,
    [SPECIES_FROSMOTH] = 0,
    [SPECIES_STONJOURNER] = 0,
    [SPECIES_EISCUE] = 0,
    [SPECIES_INDEEDEE] = 0,
    [SPECIES_MORPEKO] = 0,
    [SPECIES_CUFANT] = 0,
    [SPECIES_COPPERAJAH] = 0,
    [SPECIES_DRACOZOLT] = 0,
    [SPECIES_ARCTOZOLT] = 0,
    [SPECIES_DRACOVISH] = 0,
    [SPECIES_ARCTOVISH] = 0,
    [SPECIES_DURALUDON] = 0,
    [SPECIES_DREEPY] = 0,
    [SPECIES_DRAKLOAK] = 0,
    [SPECIES_DRAGAPULT] = 0,
    [SPECIES_ZACIAN] = 0,
    [SPECIES_ZAMAZENTA] = 0,
    [SPECIES_ETERNATUS] = 0,
    [SPECIES_KUBFU] = 0,
    [SPECIES_URSHIFU] = 0,
    [SPECIES_ZARUDE] = 0,
    [SPECIES_REGIELEKI] = 0,
    [SPECIES_REGIDRAGO] = 0,
    [SPECIES_GLASTRIER] = 0,
    [SPECIES_SPECTRIER] = 0,
    [SPECIES_CALYREX] = 0,
    [SPECIES_VENUSAUR_MEGA] = 1,
    [SPECIES_CHARIZARD_MEGA_X] = 0,
    [SPECIES_CHARIZARD_MEGA_Y] = 0,
    [SPECIES_BLASTOISE_MEGA] = 2,
    [SPECIES_BEEDRILL_MEGA] = 2,
    [SPECIES_PIDGEOT_MEGA] = 0,
    [SPECIES_ALAKAZAM_MEGA] = 2,
    [SPECIES_SLOWBRO_MEGA] = 0,
    [SPECIES_GENGAR_MEGA] = 2,
    [SPECIES_KANGASKHAN_MEGA] = 2,
    [SPECIES_PINSIR_MEGA] = 2,
    [SPECIES_GYARADOS_MEGA] = 0,
    [SPECIES_AERODACTYL_MEGA] = 2,
    [SPECIES_MEWTWO_MEGA_X] = 2,
    [SPECIES_MEWTWO_MEGA_Y] = 2,
    [SPECIES_AMPHAROS_MEGA] = 0,
    [SPECIES_STEELIX_MEGA] = 0,
    [SPECIES_SCIZOR_MEGA] = 0,
    [SPECIES_HERACROSS_MEGA] = 0,
    [SPECIES_HOUNDOOM_MEGA] = 0,
    [SPECIES_TYRANITAR_MEGA] = 1,
    [SPECIES_SCEPTILE_MEGA] = 1,
    [SPECIES_BLAZIKEN_MEGA] = 0,
    [SPECIES_SWAMPERT_MEGA] = 0,
    [SPECIES_GARDEVOIR_MEGA] = 1,
    [SPECIES_SABLEYE_MEGA] = 2,
    [SPECIES_MAWILE_MEGA] = 0,
    [SPECIES_AGGRON_MEGA] = 2,
    [SPECIES_MEDICHAM_MEGA] = 0,
    [SPECIES_MANECTRIC_MEGA] = 0,
    [SPECIES_SHARPEDO_MEGA] = 0,
    [SPECIES_CAMERUPT_MEGA] = 0,
    [SPECIES_ALTARIA_MEGA] = 0,
    [SPECIES_BANETTE_MEGA] = 0,
    [SPECIES_ABSOL_MEGA] = 0,
    [SPECIES_GLALIE_MEGA] = 0,
    [SPECIES_SALAMENCE_MEGA] = 0,
    [SPECIES_METAGROSS_MEGA] = 0,
    [SPECIES_LATIAS_MEGA] = 2,
    [SPECIES_LATIOS_MEGA] = 2,
    [SPECIES_KYOGRE_PRIMAL] = 0,
    [SPECIES_GROUDON_PRIMAL] = 0,
    [SPECIES_RAYQUAZA_MEGA] = 1,
    [SPECIES_LOPUNNY_MEGA] = 2,
    [SPECIES_GARCHOMP_MEGA] = 0,
    [SPECIES_LUCARIO_MEGA] = 2,
    [SPECIES_ABOMASNOW_MEGA] = 1,
    [SPECIES_GALLADE_MEGA] = 1,
    [SPECIES_AUDINO_MEGA] = 1,
    [SPECIES_DIANCIE_MEGA] = 0,
    [SPECIES_RATTATA_ALOLAN] = 2,
    [SPECIES_RATICATE_ALOLAN] = 2,
    [SPECIES_RAICHU_ALOLAN] = 2,
    [SPECIES_SANDSHREW_ALOLAN] = 0,
    [SPECIES_SANDSLASH_ALOLAN] = 0,
    [SPECIES_VULPIX_ALOLAN] = 2,
    [SPECIES_NINETALES_ALOLAN] = 2,
    [SPECIES_DIGLETT_ALOLAN] = 2,
    [SPECIES_DUGTRIO_ALOLAN] = 2,
    [SPECIES_MEOWTH_ALOLAN] = 2,
    [SPECIES_PERSIAN_ALOLAN] = 2,
    [SPECIES_GEODUDE_ALOLAN] = 2,
    [SPECIES_GRAVELER_ALOLAN] = 2,
    [SPECIES_GOLEM_ALOLAN] = 2,
    [SPECIES_GRIMER_ALOLAN] = 1,
    [SPECIES_MUK_ALOLAN] = 0,
    [SPECIES_EXEGGUTOR_ALOLAN] = 1,
    [SPECIES_MAROWAK_ALOLAN] = 1,
    [SPECIES_MEOWTH_GALARIAN] = 0,
    [SPECIES_PONYTA_GALARIAN] = 0,
    [SPECIES_RAPIDASH_GALARIAN] = 0,
    [SPECIES_SLOWPOKE_GALARIAN] = 0,
    [SPECIES_SLOWBRO_GALARIAN] = 0,
    [SPECIES_FARFETCHD_GALARIAN] = 0,
    [SPECIES_WEEZING_GALARIAN] = 0,
    [SPECIES_MR_MIME_GALARIAN] = 0,
    [SPECIES_ARTICUNO_GALARIAN] = 0,
    [SPECIES_ZAPDOS_GALARIAN] = 0,
    [SPECIES_MOLTRES_GALARIAN] = 0,
    [SPECIES_SLOWKING_GALARIAN] = 0,
    [SPECIES_CORSOLA_GALARIAN] = 0,
    [SPECIES_ZIGZAGOON_GALARIAN] = 0,
    [SPECIES_LINOONE_GALARIAN] = 0,
    [SPECIES_DARUMAKA_GALARIAN] = 0,
    [SPECIES_DARMANITAN_GALARIAN] = 0,
    [SPECIES_YAMASK_GALARIAN] = 0,
    [SPECIES_STUNFISK_GALARIAN] = 0,
    [SPECIES_PIKACHU_COSPLAY] = 2,
    [SPECIES_PIKACHU_ROCK_STAR] = 1,
    [SPECIES_PIKACHU_BELLE] = 0,
    [SPECIES_PIKACHU_POP_STAR] = 0,
    [SPECIES_PIKACHU_PH_D] = 1,
    [SPECIES_PIKACHU_LIBRE] = 0,
    [SPECIES_PIKACHU_ORIGINAL_CAP] = 0,
    [SPECIES_PIKACHU_HOENN_CAP] = 0,
    [SPECIES_PIKACHU_SINNOH_CAP] = 0,
    [SPECIES_PIKACHU_UNOVA_CAP] = 0,
    [SPECIES_PIKACHU_KALOS_CAP] = 0,
    [SPECIES_PIKACHU_ALOLA_CAP] = 0,
    [SPECIES_PIKACHU_PARTNER_CAP] = 0,
    [SPECIES_PIKACHU_WORLD_CAP] = 0,
    [SPECIES_PICHU_SPIKY_EARED] = 1,
    [SPECIES_UNOWN_B ... SPECIES_UNOWN_QMARK] = 0,
    [SPECIES_CASTFORM_SUNNY] = 0,
    [SPECIES_CASTFORM_RAINY] = 0,
    [SPECIES_CASTFORM_SNOWY] = 0,
    [SPECIES_BURMY_SANDY_CLOAK] = 1,
    [SPECIES_BURMY_TRASH_CLOAK] = 0,
    [SPECIES_WORMADAM_SANDY_CLOAK] = 1,
    [SPECIES_WORMADAM_TRASH_CLOAK] = 0,
    [SPECIES_CHERRIM_SUNSHINE] = 1,
    [SPECIES_SHELLOS_EAST_SEA] = 0,
    [SPECIES_GASTRODON_EAST_SEA] = 0,
    [SPECIES_ROTOM_HEAT] = 0,
    [SPECIES_ROTOM_WASH] = 0,
    [SPECIES_ROTOM_FROST] = 5,
    [SPECIES_ROTOM_FAN] = 0,
    [SPECIES_ROTOM_MOW] = 0,
    [SPECIES_GIRATINA_ORIGIN] = 0,
    [SPECIES_SHAYMIN_SKY] = 1,
    [SPECIES_ARCEUS_FIGHTING] = 1,
    [SPECIES_ARCEUS_FLYING] = 1,
    [SPECIES_ARCEUS_POISON] = 1,
    [SPECIES_ARCEUS_GROUND] = 1,
    [SPECIES_ARCEUS_ROCK] = 1,
    [SPECIES_ARCEUS_BUG] = 1,
    [SPECIES_ARCEUS_GHOST] = 1,
    [SPECIES_ARCEUS_STEEL] = 1,
    [SPECIES_ARCEUS_FIRE] = 1,
    [SPECIES_ARCEUS_WATER] = 1,
    [SPECIES_ARCEUS_GRASS] = 1,
    [SPECIES_ARCEUS_ELECTRIC] = 1,
    [SPECIES_ARCEUS_PSYCHIC] = 1,
    [SPECIES_ARCEUS_ICE] = 1,
    [SPECIES_ARCEUS_DRAGON] = 1,
    [SPECIES_ARCEUS_DARK] = 1,
    [SPECIES_ARCEUS_FAIRY] = 1,
    [SPECIES_BASCULIN_BLUE_STRIPED] = 0,
    [SPECIES_DARMANITAN_ZEN_MODE] = 0,
    [SPECIES_DARMANITAN_ZEN_MODE_GALARIAN] = 0,
    [SPECIES_DEERLING_SUMMER] = 1,
    [SPECIES_DEERLING_AUTUMN] = 0,
    [SPECIES_DEERLING_WINTER] = 2,
    [SPECIES_SAWSBUCK_SUMMER] = 1,
    [SPECIES_SAWSBUCK_AUTUMN] = 1,
    [SPECIES_SAWSBUCK_WINTER] = 1,
    [SPECIES_TORNADUS_THERIAN] = 1,
    [SPECIES_THUNDURUS_THERIAN] = 0,
    [SPECIES_LANDORUS_THERIAN] = 0,
    [SPECIES_KYUREM_WHITE] = 0,
    [SPECIES_KYUREM_BLACK] = 0,
    [SPECIES_KELDEO_RESOLUTE] = 0,
    [SPECIES_MELOETTA_PIROUETTE] = 0,
    [SPECIES_GENESECT_DOUSE_DRIVE] = 2,
    [SPECIES_GENESECT_SHOCK_DRIVE] = 2,
    [SPECIES_GENESECT_BURN_DRIVE] = 2,
    [SPECIES_GENESECT_CHILL_DRIVE] = 2,
    [SPECIES_GRENINJA_BATTLE_BOND] = 0,
    [SPECIES_GRENINJA_ASH] = 0,
    [SPECIES_VIVILLON_POLAR] = 0,
    [SPECIES_VIVILLON_TUNDRA] = 0,
    [SPECIES_VIVILLON_CONTINENTAL] = 2,
    [SPECIES_VIVILLON_GARDEN] = 1,
    [SPECIES_VIVILLON_ELEGANT] = 0,
    [SPECIES_VIVILLON_MEADOW] = 0,
    [SPECIES_VIVILLON_MODERN] = 2,
    [SPECIES_VIVILLON_MARINE] = 0,
    [SPECIES_VIVILLON_ARCHIPELAGO] = 0,
    [SPECIES_VIVILLON_HIGH_PLAINS] = 0,
    [SPECIES_VIVILLON_SANDSTORM] = 1,
    [SPECIES_VIVILLON_RIVER] = 2,
    [SPECIES_VIVILLON_MONSOON] = 0,
    [SPECIES_VIVILLON_SAVANNA] = 0,
    [SPECIES_VIVILLON_SUN] = 0,
    [SPECIES_VIVILLON_OCEAN] = 0,
    [SPECIES_VIVILLON_JUNGLE] = 0,
    [SPECIES_VIVILLON_FANCY] = 1,
    [SPECIES_VIVILLON_POKE_BALL] = 2,
    [SPECIES_FLABEBE_YELLOW_FLOWER] = 1,
    [SPECIES_FLABEBE_ORANGE_FLOWER] = 0,
    [SPECIES_FLABEBE_BLUE_FLOWER] = 0,
    [SPECIES_FLABEBE_WHITE_FLOWER] = 1,
    [SPECIES_FLOETTE_YELLOW_FLOWER] = 1,
    [SPECIES_FLOETTE_ORANGE_FLOWER] = 0,
    [SPECIES_FLOETTE_BLUE_FLOWER] = 0,
    [SPECIES_FLOETTE_WHITE_FLOWER] = 1,
    [SPECIES_FLOETTE_ETERNAL_FLOWER] = 0,
    [SPECIES_FLORGES_YELLOW_FLOWER] = 1,
    [SPECIES_FLORGES_ORANGE_FLOWER] = 0,
    [SPECIES_FLORGES_BLUE_FLOWER] = 0,
    [SPECIES_FLORGES_WHITE_FLOWER] = 0,
    [SPECIES_FURFROU_HEART_TRIM] = 0,
    [SPECIES_FURFROU_STAR_TRIM] = 0,
    [SPECIES_FURFROU_DIAMOND_TRIM] = 0,
    [SPECIES_FURFROU_DEBUTANTE_TRIM] = 2,
    [SPECIES_FURFROU_MATRON_TRIM] = 2,
    [SPECIES_FURFROU_DANDY_TRIM] = 1,
    [SPECIES_FURFROU_LA_REINE_TRIM] = 0,
    [SPECIES_FURFROU_KABUKI_TRIM] = 0,
    [SPECIES_FURFROU_PHARAOH_TRIM] = 0,
    [SPECIES_MEOWSTIC_FEMALE] = 0,
    [SPECIES_AEGISLASH_BLADE] = 2,
    [SPECIES_PUMPKABOO_SMALL] = 2,
    [SPECIES_PUMPKABOO_LARGE] = 2,
    [SPECIES_PUMPKABOO_SUPER] = 2,
    [SPECIES_GOURGEIST_SMALL] = 2,
    [SPECIES_GOURGEIST_LARGE] = 2,
    [SPECIES_GOURGEIST_SUPER] = 2,
    [SPECIES_XERNEAS_ACTIVE] = 0,
    [SPECIES_ZYGARDE_10] = 1,
    [SPECIES_ZYGARDE_10_POWER_CONSTRUCT] = 1,
    [SPECIES_ZYGARDE_50_POWER_CONSTRUCT] = 1,
    [SPECIES_ZYGARDE_COMPLETE] = 1,
    [SPECIES_HOOPA_UNBOUND] = 0,
    [SPECIES_ORICORIO_POM_POM] = 1,
    [SPECIES_ORICORIO_PAU] = 1,
    [SPECIES_ORICORIO_SENSU] = 0,
    [SPECIES_ROCKRUFF_OWN_TEMPO] = 2,
    [SPECIES_LYCANROC_MIDNIGHT] = 0,
    [SPECIES_LYCANROC_DUSK] = 0,
    [SPECIES_WISHIWASHI_SCHOOL] = 0,
    [SPECIES_SILVALLY_FIGHTING] = 0,
    [SPECIES_SILVALLY_FLYING] = 0,
    [SPECIES_SILVALLY_POISON] = 0,
    [SPECIES_SILVALLY_GROUND] = 0,
    [SPECIES_SILVALLY_ROCK] = 0,
    [SPECIES_SILVALLY_BUG] = 0,
    [SPECIES_SILVALLY_GHOST] = 0,
    [SPECIES_SILVALLY_STEEL] = 0,
    [SPECIES_SILVALLY_FIRE] = 0,
    [SPECIES_SILVALLY_WATER] = 0,
    [SPECIES_SILVALLY_GRASS] = 0,
    [SPECIES_SILVALLY_ELECTRIC] = 0,
    [SPECIES_SILVALLY_PSYCHIC] = 0,
    [SPECIES_SILVALLY_ICE] = 0,
    [SPECIES_SILVALLY_DRAGON] = 0,
    [SPECIES_SILVALLY_DARK] = 0,
    [SPECIES_SILVALLY_FAIRY] = 0,
    [SPECIES_MINIOR_CORE_RED] = 0,
    [SPECIES_MINIOR_CORE_ORANGE] = 0,
    [SPECIES_MINIOR_CORE_YELLOW] = 0,
    [SPECIES_MINIOR_CORE_GREEN] = 1,
    [SPECIES_MINIOR_CORE_BLUE] = 0,
    [SPECIES_MINIOR_CORE_INDIGO] = 0,
    [SPECIES_MINIOR_CORE_VIOLET] = 2,
    [SPECIES_MIMIKYU_BUSTED] = 1,
    [SPECIES_NECROZMA_DUSK_MANE] = 0,
    [SPECIES_NECROZMA_DAWN_WINGS] = 0,
    [SPECIES_NECROZMA_ULTRA] = 2,
    [SPECIES_MAGEARNA_ORIGINAL_COLOR] = 0,
    [SPECIES_CRAMORANT_GULPING] = 0,
    [SPECIES_CRAMORANT_GORGING] = 0,
    [SPECIES_TOXTRICITY_LOW_KEY] = 0,
    [SPECIES_SINISTEA_ANTIQUE] = 0,
    [SPECIES_POLTEAGEIST_ANTIQUE] = 0,
    [SPECIES_ALCREMIE_RUBY_CREAM] = 0,
    [SPECIES_ALCREMIE_MATCHA_CREAM] = 0,
    [SPECIES_ALCREMIE_MINT_CREAM] = 0,
    [SPECIES_ALCREMIE_LEMON_CREAM] = 0,
    [SPECIES_ALCREMIE_SALTED_CREAM] = 0,
    [SPECIES_ALCREMIE_RUBY_SWIRL] = 0,
    [SPECIES_ALCREMIE_CARAMEL_SWIRL] = 0,
    [SPECIES_ALCREMIE_RAINBOW_SWIRL] = 0,
    [SPECIES_EISCUE_NOICE_FACE] = 0,
    [SPECIES_INDEEDEE_FEMALE] = 0,
    [SPECIES_MORPEKO_HANGRY] = 0,
    [SPECIES_ZACIAN_CROWNED_SWORD] = 0,
    [SPECIES_ZAMAZENTA_CROWNED_SHIELD] = 0,
    [SPECIES_ETERNATUS_ETERNAMAX] = 0,
    [SPECIES_URSHIFU_RAPID_STRIKE_STYLE] = 0,
    [SPECIES_ZARUDE_DADA] = 0,
    [SPECIES_CALYREX_ICE_RIDER] = 0,
    [SPECIES_CALYREX_SHADOW_RIDER] = 0,
    [SPECIES_EGG] = 1,
};

const u8 gMonIconPaletteIndicesFemale[] =
{
    [SPECIES_EEVEE] = 2,
};

const struct SpritePalette gMonIconPaletteTable[] =
{
    { gMonIconPalettes[0], POKE_ICON_BASE_PAL_TAG + 0 },
    { gMonIconPalettes[1], POKE_ICON_BASE_PAL_TAG + 1 },
    { gMonIconPalettes[2], POKE_ICON_BASE_PAL_TAG + 2 },
    { gMonIconPalettes[3], POKE_ICON_BASE_PAL_TAG + 3 },
    { gMonIconPalettes[4], POKE_ICON_BASE_PAL_TAG + 4 },
    { gMonIconPalettes[5], POKE_ICON_BASE_PAL_TAG + 5 },
};

const struct OamData sMonIconOamData =
{
    .y = 0,
    .affineMode = ST_OAM_AFFINE_OFF,
    .objMode = ST_OAM_OBJ_NORMAL,
    .bpp = ST_OAM_4BPP,
    .shape = SPRITE_SHAPE(32x32),
    .x = 0,
    .size = SPRITE_SIZE(32x32),
    .tileNum = 0,
    .priority = 1,
    .paletteNum = 0,
};

// fastest to slowest

static const union AnimCmd sAnim_0[] =
{
    ANIMCMD_FRAME(0, 6),
    ANIMCMD_FRAME(1, 6),
    ANIMCMD_JUMP(0),
};

static const union AnimCmd sAnim_1[] =
{
    ANIMCMD_FRAME(0, 8),
    ANIMCMD_FRAME(1, 8),
    ANIMCMD_JUMP(0),
};

static const union AnimCmd sAnim_2[] =
{
    ANIMCMD_FRAME(0, 14),
    ANIMCMD_FRAME(1, 14),
    ANIMCMD_JUMP(0),
};

static const union AnimCmd sAnim_3[] =
{
    ANIMCMD_FRAME(0, 22),
    ANIMCMD_FRAME(1, 22),
    ANIMCMD_JUMP(0),
};

static const union AnimCmd sAnim_4[] =
{
    ANIMCMD_FRAME(0, 29),
    ANIMCMD_FRAME(0, 29), // frame 0 is repeated
    ANIMCMD_JUMP(0),
};

const union AnimCmd *const sMonIconAnims[] =
{
    sAnim_0,
    sAnim_1,
    sAnim_2,
    sAnim_3,
    sAnim_4,
};

static const union AffineAnimCmd sAffineAnim_0[] =
{
    AFFINEANIMCMD_FRAME(0, 0, 0, 10),
    AFFINEANIMCMD_END,
};

static const union AffineAnimCmd sAffineAnim_1[] =
{
    AFFINEANIMCMD_FRAME(-2, -2, 0, 122),
    AFFINEANIMCMD_END,
};

const union AffineAnimCmd *const sMonIconAffineAnims[] =
{
    sAffineAnim_0,
    sAffineAnim_1,
};

const u16 sSpriteImageSizes[3][4] =
{
    [ST_OAM_SQUARE] = 
    {
        [SPRITE_SIZE(8x8)]   =  0x20,
        [SPRITE_SIZE(16x16)] =  0x80,
        [SPRITE_SIZE(32x32)] = 0x200,
        [SPRITE_SIZE(64x64)] = 0x800,
    },
    [ST_OAM_H_RECTANGLE] = 
    {
        [SPRITE_SIZE(16x8)]  =  0x40,
        [SPRITE_SIZE(32x8)]  =  0x80,
        [SPRITE_SIZE(32x16)] = 0x100,
        [SPRITE_SIZE(64x32)] = 0x400,
    },
    [ST_OAM_V_RECTANGLE] = 
    {
        [SPRITE_SIZE(8x16)]  =  0x40,
        [SPRITE_SIZE(8x32)]  =  0x80,
        [SPRITE_SIZE(16x32)] = 0x100,
        [SPRITE_SIZE(32x64)] = 0x400,
    },
};

u8 CreateMonIcon(u16 species, void (*callback)(struct Sprite *), s16 x, s16 y, u8 subpriority, u32 personality)
{
    u8 spriteId;
    struct MonIconSpriteTemplate iconTemplate =
    {
        .oam = &sMonIconOamData,
        .image = GetMonIconPtr(species, personality),
        .anims = sMonIconAnims,
        .affineAnims = sMonIconAffineAnims,
        .callback = callback,
        .paletteTag = POKE_ICON_BASE_PAL_TAG + gMonIconPaletteIndices[species],
    };

    if (species > NUM_SPECIES)
        iconTemplate.paletteTag = POKE_ICON_BASE_PAL_TAG;
    else if (SpeciesHasGenderDifference[species] && GetGenderFromSpeciesAndPersonality(species, personality) == MON_FEMALE)
        iconTemplate.paletteTag = POKE_ICON_BASE_PAL_TAG + gMonIconPaletteIndicesFemale[species];

    spriteId = CreateMonIconSprite(&iconTemplate, x, y, subpriority);

    UpdateMonIconFrame(&gSprites[spriteId]);

    return spriteId;
}

u8 CreateMonIconNoPersonality(u16 species, void (*callback)(struct Sprite *), s16 x, s16 y, u8 subpriority)
{
    u8 spriteId;
    struct MonIconSpriteTemplate iconTemplate =
    {
        .oam = &sMonIconOamData,
        .image = NULL,
        .anims = sMonIconAnims,
        .affineAnims = sMonIconAffineAnims,
        .callback = callback,
        .paletteTag = POKE_ICON_BASE_PAL_TAG + gMonIconPaletteIndices[species],
    };

    iconTemplate.image = GetMonIconTiles(species, 0);
    spriteId = CreateMonIconSprite(&iconTemplate, x, y, subpriority);

    UpdateMonIconFrame(&gSprites[spriteId]);

    return spriteId;
}

u16 GetIconSpecies(u16 species, u32 personality)
{
    u16 result;

    if (species == SPECIES_UNOWN)
    {
        result = GetUnownSpeciesId(personality);
    }
    else
    {
        if (species > NUM_SPECIES)
            result = INVALID_ICON_SPECIES;
        else
            result = species;
    }

    return result;
}

u16 GetUnownLetterByPersonality(u32 personality)
{
    if (!personality)
        return 0;
    else
        return GET_UNOWN_LETTER(personality);
}

u16 GetIconSpeciesNoPersonality(u16 species)
{
    u16 value;

    if (MailSpeciesToSpecies(species, &value) == SPECIES_UNOWN)
    {
        value += SPECIES_UNOWN_B; // TODO
        return value;
    }
    else
    {
        if (species > NUM_SPECIES)
            species = INVALID_ICON_SPECIES;

        return GetIconSpecies(species, 0);
    }
}

const u8 *GetMonIconPtr(u16 species, u32 personality)
{
    return GetMonIconTiles(GetIconSpecies(species, personality), personality);
}

void FreeAndDestroyMonIconSprite(struct Sprite *sprite)
{
    sub_80D328C(sprite);
}

void LoadMonIconPalettes(void)
{
    u8 i;
    for (i = 0; i < ARRAY_COUNT(gMonIconPaletteTable); i++)
        LoadSpritePalette(&gMonIconPaletteTable[i]);
}

// unused
void SafeLoadMonIconPalette(u16 species)
{
    u8 palIndex;
    if (species > NUM_SPECIES)
        species = INVALID_ICON_SPECIES;
    palIndex = gMonIconPaletteIndices[species];
    if (IndexOfSpritePaletteTag(gMonIconPaletteTable[palIndex].tag) == 0xFF)
        LoadSpritePalette(&gMonIconPaletteTable[palIndex]);
}

void LoadMonIconPalette(u16 species)
{
    u8 palIndex = gMonIconPaletteIndices[species];
    if (IndexOfSpritePaletteTag(gMonIconPaletteTable[palIndex].tag) == 0xFF)
        LoadSpritePalette(&gMonIconPaletteTable[palIndex]);
}

void FreeMonIconPalettes(void)
{
    u8 i;
    for (i = 0; i < ARRAY_COUNT(gMonIconPaletteTable); i++)
        FreeSpritePaletteByTag(gMonIconPaletteTable[i].tag);
}

// unused
void SafeFreeMonIconPalette(u16 species)
{
    u8 palIndex;
    if (species > NUM_SPECIES)
        species = INVALID_ICON_SPECIES;
    palIndex = gMonIconPaletteIndices[species];
    FreeSpritePaletteByTag(gMonIconPaletteTable[palIndex].tag);
}

void FreeMonIconPalette(u16 species)
{
    u8 palIndex;
    palIndex = gMonIconPaletteIndices[species];
    FreeSpritePaletteByTag(gMonIconPaletteTable[palIndex].tag);
}

void SpriteCB_MonIcon(struct Sprite *sprite)
{
    UpdateMonIconFrame(sprite);
}

const u8* GetMonIconTiles(u16 species, u32 personality)
{
    const u8* iconSprite = gMonIconTable[species];
    if (SpeciesHasGenderDifference[species] && GetGenderFromSpeciesAndPersonality(species, personality) == MON_FEMALE)
    {
        iconSprite = gMonIconTableFemale[species];
    }
    return iconSprite;
}

void sub_80D304C(u16 offset)
{
    s32 i;
    const struct SpritePalette* monIconPalettePtr;

    if (offset <= 0xA0)
    {
        monIconPalettePtr = gMonIconPaletteTable;
        for(i = 5; i >= 0; i--)
        {
            LoadPalette(monIconPalettePtr->data, offset, 0x20);
            offset += 0x10;
            monIconPalettePtr++;
        }
    }
}

u8 GetValidMonIconPalIndex(u16 species)
{
    if (species > NUM_SPECIES)
        species = INVALID_ICON_SPECIES;
    return gMonIconPaletteIndices[species];
}

u8 GetMonIconPaletteIndexFromSpecies(u16 species)
{
    return gMonIconPaletteIndices[species];
}

const u16* GetValidMonIconPalettePtr(u16 species)
{
    if (species > NUM_SPECIES)
        species = INVALID_ICON_SPECIES;
    return gMonIconPaletteTable[gMonIconPaletteIndices[species]].data;
}

u8 UpdateMonIconFrame(struct Sprite *sprite)
{
    u8 result = 0;

    if (sprite->animDelayCounter == 0)
    {
        s16 frame = sprite->anims[sprite->animNum][sprite->animCmdIndex].frame.imageValue;

        switch (frame)
        {
        case -1:
            break;
        case -2:
            sprite->animCmdIndex = 0;
            break;
        default:
            RequestSpriteCopy(
                // pointer arithmetic is needed to get the correct pointer to perform the sprite copy on.
                // because sprite->images is a struct def, it has to be casted to (u8 *) before any
                // arithmetic can be performed.
                (u8 *)sprite->images + (sSpriteImageSizes[sprite->oam.shape][sprite->oam.size] * frame),
                (u8 *)(OBJ_VRAM0 + sprite->oam.tileNum * TILE_SIZE_4BPP),
                sSpriteImageSizes[sprite->oam.shape][sprite->oam.size]);
            sprite->animDelayCounter = sprite->anims[sprite->animNum][sprite->animCmdIndex].frame.duration & 0xFF;
            sprite->animCmdIndex++;
            result = sprite->animCmdIndex;
            break;
        }
    }
    else
    {
        sprite->animDelayCounter--;
    }
    return result;
}

static u8 CreateMonIconSprite(struct MonIconSpriteTemplate *iconTemplate, s16 x, s16 y, u8 subpriority)
{
    u8 spriteId;

    struct SpriteFrameImage image = { NULL, sSpriteImageSizes[iconTemplate->oam->shape][iconTemplate->oam->size] };

    struct SpriteTemplate spriteTemplate =
    {
        .tileTag = 0xFFFF,
        .paletteTag = iconTemplate->paletteTag,
        .oam = iconTemplate->oam,
        .anims = iconTemplate->anims,
        .images = &image,
        .affineAnims = iconTemplate->affineAnims,
        .callback = iconTemplate->callback,
    };

    spriteId = CreateSprite(&spriteTemplate, x, y, subpriority);
    gSprites[spriteId].animPaused = TRUE;
    gSprites[spriteId].animBeginning = FALSE;
    gSprites[spriteId].images = (const struct SpriteFrameImage *)iconTemplate->image;
    return spriteId;
}

void sub_80D328C(struct Sprite *sprite)
{
    struct SpriteFrameImage image = { NULL, sSpriteImageSizes[sprite->oam.shape][sprite->oam.size] };
    sprite->images = &image;
    DestroySprite(sprite);
}

void SetPartyHPBarSprite(struct Sprite *sprite, u8 animNum)
{
    sprite->animNum = animNum;
    sprite->animDelayCounter = 0;
    sprite->animCmdIndex = 0;
}
