#define TRAINER_HILL_OTID 0x10000000

// NOTE: Each of these macros turn data into one byte. Therefore ranges for all arguments is 0-15
// See struct TrHillDisplay for more info about each
#define COORDS_XY(x,y)      ((y<<4)|(x))
#define TRAINER_DIRS(a, b)  (((a-1)<<4)|(b-1))
#define TRAINER_RANGE(a, b) ((a<<4)|(b))

static const struct TrHillTag sDataTagJPDefault = {
    .numTrainers = NUM_TRAINER_HILL_TRAINERS_JP,
    .unused1 = 1,
    .numFloors = NUM_TRAINER_HILL_FLOORS_JP,
    .checksum = 0x0
};


static const struct TrHillFloor sDataTagJPDefault_Floors[] = {
    [0] = {
        .trainerNum1 = 0,
        .trainerNum2 = 0,
        .trainers = {
            [0] = {
                .name = __("シゲノブ$$$$   "),
                .facilityClass = FACILITY_CLASS_YOUNGSTER,
                .unused = 0x1,
                .speechBefore = { EC_WORD_TRAINER, EC_WORD_MRS, EC_WORD_EXCL, EC_WORD_THEY_WERE, EC_WORD_OPPONENT, EC_WORD_CAN },
                .speechWin = { EC_WORD_TRAINER, EC_WORD_MRS, EC_WORD_WEAK, EC_WORD_OF, EC_WORD_WERE, EC_WORD_ELLIPSIS_ELLIPSIS_ELLIPSIS },
                .speechLose = { EC_WORD_TRAINER, EC_WORD_MRS, EC_WORD_STRONG, EC_WORD_HEY_QUES, EC_WORD_EXCL, EC_WORD_EXCL },
                .speechAfter = { EC_WORD_FABULOUS, EC_WORD_CAN_T, EC_WORD_TRAINER, EC_WORD_MRS, EC_WORD_ELLIPSIS_ELLIPSIS_ELLIPSIS, EC_WORD_EXCITING },
                .mons = {
                    [0] = {
                        .species = SPECIES_ZIGZAGOON,
                        .heldItem = ITEM_SITRUS_BERRY,
                        .moves = { MOVE_HEADBUTT, MOVE_PIN_MISSILE, MOVE_GROWL, MOVE_TAIL_WHIP },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 110,
                        .attackEV = 100,
                        .defenseEV = 100,
                        .speedEV = 100,
                        .spAttackEV = 0,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x80,
                        .nickname = __("ジグザグマ$$$$$$"),
                        .friendship = 255
                    },
                    [1] = {
                        .species = SPECIES_SHROOMISH,
                        .heldItem = ITEM_PECHA_BERRY,
                        .moves = { MOVE_MEGA_DRAIN, MOVE_LEECH_SEED, MOVE_POISON_POWDER, MOVE_GROWTH },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 120,
                        .attackEV = 0,
                        .defenseEV = 120,
                        .speedEV = 0,
                        .spAttackEV = 150,
                        .spDefenseEV = 120,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x0,
                        .nickname = __("キノココ$$$$$$$"),
                        .friendship = 255
                    },
                    [2] = {
                        .species = SPECIES_SANDSHREW,
                        .heldItem = ITEM_QUICK_CLAW,
                        .moves = { MOVE_SCRATCH, MOVE_POISON_STING, MOVE_SAND_ATTACK, MOVE_SWIFT },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 110,
                        .attackEV = 100,
                        .defenseEV = 100,
                        .speedEV = 0,
                        .spAttackEV = 0,
                        .spDefenseEV = 200,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x80,
                        .nickname = __("サンド$$$$$$$$"),
                        .friendship = 255
                    },
                    [3] = NULL_BATTLE_TOWER_POKEMON,
                    [4] = NULL_BATTLE_TOWER_POKEMON,
                    [5] = NULL_BATTLE_TOWER_POKEMON
                }
            },
            [1] = {
                .name = __("アカリ$$$$$   "),
                .facilityClass = FACILITY_CLASS_PARASOL_LADY,
                .unused = 0x1,
                .speechBefore = { EC_WORD_SHE, EC_WORD_EVERY, EC_WORD_TYPE, EC_WORD_ADORE, EC_WORD_FOR, EC_WORD_EXCL },
                .speechWin = { EC_WORD_SURPRISE, EC_WORD_THESE, EC_WORD_WONDER, EC_WORD_GIGGLE, EC_WORD_ELLIPSIS, EC_WORD_EXCL },
                .speechLose = { EC_WORD_OH_QUES, EC_MOVE(EARTHQUAKE), EC_WORD_EXISTS, EC_WORD_OF, EC_WORD_WITHOUT, EC_WORD_EXCL },
                .speechAfter = { EC_WORD_YOU_RE, EC_WORD_PROBABLY, EC_WORD_END, EC_WORD_UNTIL, EC_WORD_GOING, EC_WORD_ANYWHERE },
                .mons = {
                    [0] = NULL_BATTLE_TOWER_POKEMON,
                    [1] = NULL_BATTLE_TOWER_POKEMON,
                    [2] = NULL_BATTLE_TOWER_POKEMON,
                    [3] = {
                        .species = SPECIES_WINGULL,
                        .heldItem = ITEM_CHERI_BERRY,
                        .moves = { MOVE_WATER_GUN, MOVE_WING_ATTACK, MOVE_GROWL, MOVE_SUPERSONIC },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 110,
                        .attackEV = 100,
                        .defenseEV = 100,
                        .speedEV = 0,
                        .spAttackEV = 100,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x0,
                        .nickname = __("キャモメ$$$$$$$"),
                        .friendship = 255
                    },
                    [4] = {
                        .species = SPECIES_NUMEL,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = { MOVE_EMBER, MOVE_DIG, MOVE_TACKLE, MOVE_FOCUS_ENERGY },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 110,
                        .attackEV = 100,
                        .defenseEV = 100,
                        .speedEV = 0,
                        .spAttackEV = 100,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x96,
                        .nickname = __("ドンメル$$$$$$$"),
                        .friendship = 255
                    },
                    [5] = {
                        .species = SPECIES_SURSKIT,
                        .heldItem = ITEM_PERSIM_BERRY,
                        .moves = { MOVE_BUBBLE_BEAM, MOVE_MUD_SHOT, MOVE_QUICK_ATTACK, MOVE_AGILITY },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 100,
                        .attackEV = 100,
                        .defenseEV = 100,
                        .speedEV = 0,
                        .spAttackEV = 100,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x0,
                        .nickname = __("アメタマ$$$$$$$"),
                        .friendship = 255
                    },
                }
            },
        },
        .display = {
            .metatileData = {
                0x31, 0x35, 0x35, 0x3b, 0x26, 0x26, 0x1b, 0x1c, 0x1d, 0x25, 0x39, 0x3a, 0x3b, 0x3b, 0x3b, 0x08, 
                0x31, 0x2b, 0x2b, 0x3b, 0x34, 0x34, 0x2b, 0x2b, 0x34, 0x33, 0x3f, 0x3f, 0x3f, 0x3f, 0x3b, 0x08, 
                0x31, 0x2b, 0x2b, 0x3b, 0x3b, 0x3b, 0x34, 0x34, 0x3b, 0x33, 0x3f, 0x3f, 0x3f, 0x3b, 0x3b, 0x08, 
                0x31, 0x2b, 0x2b, 0x35, 0x35, 0x35, 0x35, 0x35, 0x3b, 0x33, 0x3f, 0x3f, 0x3f, 0x3b, 0x3f, 0x08, 
                0x31, 0x2b, 0x34, 0x34, 0x34, 0x2b, 0x34, 0x34, 0x3b, 0x2c, 0x3f, 0x3f, 0x3f, 0x3b, 0x3b, 0x08, 
                0x31, 0x2b, 0x3b, 0x35, 0x3b, 0x2b, 0x3b, 0x35, 0x3b, 0x35, 0x3b, 0x3f, 0x3f, 0x3f, 0x3b, 0x08, 
                0x31, 0x2b, 0x3b, 0x2b, 0x3b, 0x2b, 0x3b, 0x2b, 0x3b, 0x2b, 0x3b, 0x3f, 0x3f, 0x3f, 0x3b, 0x08, 
                0x31, 0x34, 0x3b, 0x2b, 0x3b, 0x34, 0x3b, 0x2b, 0x35, 0x2b, 0x3b, 0x3b, 0x3f, 0x3b, 0x3b, 0x08, 
                0x31, 0x3b, 0x3b, 0x34, 0x3b, 0x3b, 0x3b, 0x34, 0x34, 0x34, 0x3f, 0x3b, 0x3b, 0x3b, 0x3f, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
            },
            .collisionData = { 0x0381, 0x6fc1, 0x6341, 0x6041, 0x7f41, 0x4401, 0x5541, 0x5541, 0x11c1, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff, 0xffff },
            .coords = { COORDS_XY(8,2), COORDS_XY(8,7) },
            .direction = TRAINER_DIRS(DIR_SOUTH, DIR_NORTH),
            .range = TRAINER_RANGE(2, 3)
        }
    },
    [1] = {
        .trainerNum1 = 0,
        .trainerNum2 = 0,
        .trainers = {
            [0] = {
                .name = __("シゲゾウ$$$$   "),
                .facilityClass = FACILITY_CLASS_NINJA_BOY,
                .unused = 0x1,
                .speechBefore = { EC_WORD_WIN, EC_WORD_UNTIL, EC_WORD_DAUGHTER, EC_MOVE(CONSTRICT), EC_WORD_OF, EC_WORD_WERE },
                .speechWin = { EC_MOVE(CONSTRICT), EC_WORD_UNTIL, EC_WORD_LESS, EC_WORD_NONE, EC_WORD_HEY_QUES, EC_WORD_EXCL },
                .speechLose = { EC_WORD_YET, EC_WORD_ELLIPSIS_ELLIPSIS_ELLIPSIS, EC_WORD_URGH, EC_WORD_ALMOST, EC_WORD_POWER, EC_WORD_NONE },
                .speechAfter = { EC_WORD_HUH_QUES, EC_WORD_TOMORROW, EC_WORD_SMELL, EC_WORD_ABSOLUTELY, EC_WORD_THOSE, EC_WORD_EXCL_EXCL },
                .mons = {
                    [0] = {
                        .species = SPECIES_ELECTRIKE,
                        .heldItem = ITEM_CHERI_BERRY,
                        .moves = { MOVE_SPARK, MOVE_THUNDER_WAVE, MOVE_QUICK_ATTACK, MOVE_ROAR },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 120,
                        .attackEV = 120,
                        .defenseEV = 0,
                        .speedEV = 120,
                        .spAttackEV = 150,
                        .spDefenseEV = 0,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 1,
                        .personality = 0x0,
                        .nickname = __("ラクライ$$$$$$$"),
                        .friendship = 255
                    },
                    [1] = {
                        .species = SPECIES_CORPHISH,
                        .heldItem = ITEM_QUICK_CLAW,
                        .moves = { MOVE_KNOCK_OFF, MOVE_CRABHAMMER, MOVE_TAUNT, MOVE_PROTECT },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 100,
                        .attackEV = 110,
                        .defenseEV = 100,
                        .speedEV = 0,
                        .spAttackEV = 100,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 1,
                        .personality = 0x96,
                        .nickname = __("ヘイガニ$$$$$$$"),
                        .friendship = 255
                    },
                    [2] = {
                        .species = SPECIES_BALTOY,
                        .heldItem = ITEM_PERSIM_BERRY,
                        .moves = { MOVE_PSYBEAM, MOVE_ROCK_TOMB, MOVE_MUD_SLAP, MOVE_HARDEN },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 100,
                        .attackEV = 100,
                        .defenseEV = 100,
                        .speedEV = 0,
                        .spAttackEV = 110,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x96,
                        .nickname = __("ヤジロン$$$$$$$"),
                        .friendship = 255
                    },
                    [3] = NULL_BATTLE_TOWER_POKEMON,
                    [4] = NULL_BATTLE_TOWER_POKEMON,
                    [5] = NULL_BATTLE_TOWER_POKEMON,
                }
            },
            [1] = {
                .name = __("ナミコ$$$$$   "),
                .facilityClass = FACILITY_CLASS_BEAUTY,
                .unused = 0x1,
                .speechBefore = { EC_WORD_SISTER, EC_WORD_ALTHOUGH, EC_WORD_KIND, EC_WORD_MATCH, EC_WORD_PLEASE, EC_WORD_WITHOUT },
                .speechWin = { EC_WORD_KIND, EC_WORD_ELLIPSIS, EC_WORD_INSTEAD, EC_WORD_DEEP, EC_WORD_WEAK, EC_WORD_QUES },
                .speechLose = { EC_WORD_AWFUL, EC_WORD_GWAH, EC_WORD_HOPELESS, EC_WORD_CAN_T_WIN, EC_WORD_IS, EC_WORD_NONE },
                .speechAfter = { EC_WORD_AWW, EC_EMPTY_WORD, EC_EMPTY_WORD, EC_WORD_ALMOST, EC_WORD_GOOD, EC_WORD_ANYWHERE },
                .mons = {
                    [0] = NULL_BATTLE_TOWER_POKEMON,
                    [1] = NULL_BATTLE_TOWER_POKEMON,
                    [2] = NULL_BATTLE_TOWER_POKEMON,
                    [3] = {
                        .species = SPECIES_SPHEAL,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = { MOVE_ICE_BALL, MOVE_BODY_SLAM, MOVE_WATER_GUN, MOVE_ENCORE },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 100,
                        .attackEV = 100,
                        .defenseEV = 100,
                        .speedEV = 0,
                        .spAttackEV = 110,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x80,
                        .nickname = __("タマザラシ$$$$$$"),
                        .friendship = 255
                    },
                    [4] = {
                        .species = SPECIES_SPOINK,
                        .heldItem = ITEM_PERSIM_BERRY,
                        .moves = { MOVE_PSYWAVE, MOVE_FUTURE_SIGHT, MOVE_CONFUSE_RAY, MOVE_MAGIC_COAT },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 100,
                        .attackEV = 0,
                        .defenseEV = 100,
                        .speedEV = 100,
                        .spAttackEV = 110,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 1,
                        .personality = 0xf,
                        .nickname = __("バネブー$$$$$$$"),
                        .friendship = 255
                    },
                    [5] = {
                        .species = SPECIES_POOCHYENA,
                        .heldItem = ITEM_PECHA_BERRY,
                        .moves = { MOVE_BITE, MOVE_POISON_FANG, MOVE_SWAGGER, MOVE_SCARY_FACE },
                        .level = 0,
                        .ppBonuses = 0x0,
                        .hpEV = 70,
                        .attackEV = 80,
                        .defenseEV = 80,
                        .speedEV = 80,
                        .spAttackEV = 80,
                        .spDefenseEV = 80,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x96,
                        .nickname = __("ポチエナ$$$$$$$"),
                        .friendship = 255
                    },
                }
            },
        },
        .display = {
            .metatileData = {
                0x31, 0x3b, 0x35, 0x3b, 0x39, 0x26, 0x1b, 0x1c, 0x1d, 0x25, 0x39, 0x3a, 0x3b, 0x35, 0x3b, 0x08, 
                0x31, 0x3b, 0x2b, 0x3b, 0x3b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x3b, 0x3b, 0x3b, 0x2b, 0x3b, 0x08, 
                0x31, 0x3b, 0x2b, 0x3b, 0x3b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x3e, 0x3e, 0x3b, 0x2b, 0x3b, 0x08, 
                0x31, 0x3b, 0x2b, 0x3b, 0x3b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x3b, 0x3b, 0x3b, 0x2b, 0x3b, 0x08, 
                0x3f, 0x3b, 0x2b, 0x3b, 0x3b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x3b, 0x3e, 0x3e, 0x2b, 0x3b, 0x08, 
                0x31, 0x3b, 0x2b, 0x3b, 0x3b, 0x2b, 0x2b, 0x2c, 0x2b, 0x2b, 0x3b, 0x3b, 0x3b, 0x2b, 0x3b, 0x08, 
                0x31, 0x3f, 0x2b, 0x3b, 0x3b, 0x2c, 0x2c, 0x3b, 0x2c, 0x2c, 0x3e, 0x3e, 0x3b, 0x2b, 0x3b, 0x08, 
                0x31, 0x3b, 0x2b, 0x35, 0x35, 0x35, 0x35, 0x3b, 0x35, 0x35, 0x35, 0x35, 0x35, 0x2b, 0x3b, 0x08, 
                0x3f, 0x3b, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x3b, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x3b, 0x08, 
                0x31, 0x3b, 0x3f, 0x3f, 0x3b, 0x35, 0x35, 0x3b, 0x35, 0x35, 0x35, 0x35, 0x35, 0x35, 0x35, 0x08, 
                0x31, 0x3f, 0x3f, 0x3f, 0x3b, 0x2b, 0x2b, 0x35, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x08, 
                0x31, 0x3f, 0x3f, 0x3b, 0x3b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x2b, 0x08, 
                0x31, 0x3b, 0x3b, 0x3b, 0x3f, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x2c, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 
            },
            .collisionData = { 0x0381, 0x27c5, 0x27c5, 0x27c5, 0x27c5, 0x27c5, 0x26c5, 0x2005, 0x3efd, 0x1, 0x6ff, 0x7ff, 0x7ff, 0xffff, 0xffff, 0xffff },
            .coords = { COORDS_XY(7,6), COORDS_XY(7,10) },
            .direction = TRAINER_DIRS(DIR_SOUTH, DIR_NORTH),
            .range = TRAINER_RANGE(3, 3)
        }
    },
};

static const struct TrHillTag sDataTagNormal =
{
    .numTrainers = NUM_TRAINER_HILL_TRAINERS,
    .unused1 = 2,
    .numFloors = NUM_TRAINER_HILL_FLOORS,
    .checksum = 0x00051E05
};

static const struct TrHillFloor sDataTagNormal_Floors[] =
{
    [0] =
    {
        .trainerNum1 = 17,
        .trainerNum2 = 18,
        .trainers =
        {
            [0] =
            {
                .name = _("ALAINA"),
                .facilityClass = FACILITY_CLASS_HEX_MANIAC,
                .unused = 0,
                .speechBefore = {EC_WORD_YOU, EC_WORD_CAN, EC_WORD_CHOOSE, EC_WORD_NOT, EC_WORD_TO, EC_WORD_BELIEVE},
                .speechWin = {EC_WORD_A, EC_WORD_SUPER, EC_WORD_NATURAL, EC_WORD_POWER, EC_WORD_HAS, EC_WORD_COME},
                .speechLose = {EC_WORD_THIS, EC_WORD_MUST_BE, EC_WORD_A, EC_MOVE(NIGHTMARE), EC_WORD_EXCL, EC_EMPTY_WORD},
                .speechAfter = {EC_WORD_I, EC_WORD_WILL, EC_WORD_DISAPPEAR, EC_WORD_IN, EC_WORD_THE, EC_WORD_DARK},
                .mons =
                {
                    [0] =
                    {
                        .species = SPECIES_MISDREAVUS,
                        .heldItem = ITEM_BRIGHT_POWDER,
                        .moves = {MOVE_SHADOW_BALL, MOVE_PSYCHIC, MOVE_THUNDERBOLT, MOVE_CONFUSE_RAY},
                        .level = 0,
                        .ppBonuses = 0,
                        .attackEV = 155,
                        .speedEV = 255,
                        .spAttackEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 24,
                        .attackIV = 24,
                        .defenseIV = 24,
                        .speedIV = 24,
                        .spAttackIV = 24,
                        .spDefenseIV = 24,
                        .abilityNum = 0,
                        .personality = 0x0,
                        .nickname = _("MISDREAVUS"),
                        .friendship = 255,
                    },
                    [1] =
                    {
                        .species = SPECIES_SOLROCK,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = {MOVE_PSYCHIC, MOVE_FLAMETHROWER, MOVE_ROCK_SLIDE, MOVE_CALM_MIND},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 200,
                        .defenseEV = 100,
                        .spAttackEV = 110,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 24,
                        .attackIV = 24,
                        .defenseIV = 24,
                        .speedIV = 24,
                        .spAttackIV = 24,
                        .spDefenseIV = 24,
                        .abilityNum = 0,
                        .personality = 0xF,
                        .nickname = _("SOLROCK"),
                        .friendship = 255,
                    },
                    [2] =
                    {
                        .species = SPECIES_CLAYDOL,
                        .heldItem = ITEM_SHELL_BELL,
                        .moves = {MOVE_EARTHQUAKE, MOVE_PSYCHIC, MOVE_SHADOW_BALL, MOVE_ICE_BEAM},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 255,
                        .defenseEV = 135,
                        .spDefenseEV = 120,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 24,
                        .attackIV = 24,
                        .defenseIV = 24,
                        .speedIV = 24,
                        .spAttackIV = 24,
                        .spDefenseIV = 24,
                        .abilityNum = 0,
                        .personality = 0xC,
                        .nickname = _("CLAYDOL"),
                        .friendship = 255,
                    },
                    [3] =
                    {
                        .species = SPECIES_WEEZING,
                        .heldItem = ITEM_SALAC_BERRY,
                        .moves = {MOVE_SLUDGE_BOMB, MOVE_SHADOW_BALL, MOVE_FRUSTRATION, MOVE_DESTINY_BOND},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 110,
                        .attackEV = 200,
                        .spDefenseEV = 200,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 24,
                        .attackIV = 24,
                        .defenseIV = 24,
                        .speedIV = 24,
                        .spAttackIV = 24,
                        .spDefenseIV = 24,
                        .abilityNum = 0,
                        .personality = 0x80,
                        .nickname = _("WEEZING"),
                        .friendship = 0,
                    },
                    [4] =
                    {
                        .species = SPECIES_LUNATONE,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = {MOVE_PSYCHIC, MOVE_ICE_BEAM, MOVE_ROCK_SLIDE, MOVE_CALM_MIND},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 200,
                        .defenseEV = 100,
                        .spAttackEV = 110,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 24,
                        .attackIV = 24,
                        .defenseIV = 24,
                        .speedIV = 24,
                        .spAttackIV = 24,
                        .spDefenseIV = 24,
                        .abilityNum = 0,
                        .personality = 0xF,
                        .nickname = _("LUNATONE"),
                        .friendship = 255,
                    },
                    [5] =
                    {
                        .species = SPECIES_FLYGON,
                        .heldItem = ITEM_CHOICE_BAND,
                        .moves = {MOVE_EARTHQUAKE, MOVE_DRAGON_CLAW, MOVE_CRUNCH, MOVE_FLAMETHROWER},
                        .level = 0,
                        .ppBonuses = 0,
                        .attackEV = 155,
                        .speedEV = 255,
                        .spAttackEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 24,
                        .attackIV = 24,
                        .defenseIV = 24,
                        .speedIV = 24,
                        .spAttackIV = 24,
                        .spDefenseIV = 24,
                        .abilityNum = 0,
                        .personality = 0x83,
                        .nickname = _("FLYGON"),
                        .friendship = 255,
                    },
                },
            },
            [1] =
            {
                .name = _("ALFONSO"),
                .facilityClass = FACILITY_CLASS_CYCLING_TRIATHLETE_M,
                .unused = 0,
                .speechBefore = {EC_WORD_I, EC_WORD_NEVER, EC_WORD_TAKE, EC_WORD_A, EC_WORD_TRAIN, EC_EMPTY_WORD},
                .speechWin = {EC_WORD_I_AM, EC_WORD_FEELING, EC_MOVE2(SUPERSONIC), EC_WORD_ON, EC_WORD_MY, EC_WORD_BIKE},
                .speechLose = {EC_WORD_MY, EC_WORD_BIKE, EC_WORD_EXCL, EC_WORD_YOU_VE, EC_WORD_DESTROYED, EC_WORD_IT},
                .speechAfter = {EC_WORD_A, EC_WORD_BIKE, EC_WORD_OVER, EC_WORD_ANY, EC_WORD_TRAIN, EC_WORD_EXCL},
                .mons =
                {
                    [0] =
                    {
                        .species = SPECIES_SEALEO,
                        .heldItem = ITEM_NEVER_MELT_ICE,
                        .moves = {MOVE_BLIZZARD, MOVE_ICE_BALL, MOVE_ENCORE, MOVE_HAIL},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 255,
                        .spAttackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0xF,
                        .nickname = _("SEALEO"),
                        .friendship = 255,
                    },
                    [1] =
                    {
                        .species = SPECIES_AMPHAROS,
                        .heldItem = ITEM_MAGNET,
                        .moves = {MOVE_THUNDER, MOVE_THUNDER_WAVE, MOVE_COTTON_SPORE, MOVE_LIGHT_SCREEN},
                        .level = 0,
                        .ppBonuses = 0,
                        .speedEV = 255,
                        .spAttackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0xF,
                        .nickname = _("AMPHAROS"),
                        .friendship = 255,
                    },
                    [2] =
                    {
                        .species = SPECIES_MACHOKE,
                        .heldItem = ITEM_BLACK_BELT,
                        .moves = {MOVE_DYNAMIC_PUNCH, MOVE_MUD_SLAP, MOVE_COUNTER, MOVE_SCARY_FACE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 255,
                        .attackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x4E,
                        .nickname = _("MACHOKE"),
                        .friendship = 255,
                    },
                    [3] =
                    {
                        .species = SPECIES_FLAREON,
                        .heldItem = ITEM_CHARCOAL,
                        .moves = {MOVE_FIRE_BLAST, MOVE_BITE, MOVE_QUICK_ATTACK, MOVE_SAND_ATTACK},
                        .level = 0,
                        .ppBonuses = 0,
                        .speedEV = 255,
                        .spAttackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x28,
                        .nickname = _("FLAREON"),
                        .friendship = 255,
                    },
                    [4] =
                    {
                        .species = SPECIES_MAGNETON,
                        .heldItem = ITEM_MAGNET,
                        .moves = {MOVE_ZAP_CANNON, MOVE_THUNDER_WAVE, MOVE_SCREECH, MOVE_METAL_SOUND},
                        .level = 0,
                        .ppBonuses = 0,
                        .speedEV = 255,
                        .spAttackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x0,
                        .nickname = _("MAGNETON"),
                        .friendship = 255,
                    },
                    [5] =
                    {
                        .species = SPECIES_PINSIR,
                        .heldItem = ITEM_QUICK_CLAW,
                        .moves = {MOVE_GUILLOTINE, MOVE_BRICK_BREAK, MOVE_SWAGGER, MOVE_FEINT_ATTACK},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 200,
                        .defenseEV = 155,
                        .spDefenseEV = 155,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x85,
                        .nickname = _("PINSIR"),
                        .friendship = 255,
                    },
                },
            },
        },
            .display = {
            .metatileData = {
                0x31, 0x3B, 0x35, 0x35, 0x26, 0x26, 0x1B, 0x1C, 0x1D, 0x25, 0x26, 0x3A, 0x3B, 0x35, 0x3B, 0x08, 
                0x31, 0x3B, 0x2C, 0x2C, 0x2C, 0x2B, 0x24, 0x24, 0x24, 0x24, 0x2C, 0x3B, 0x3B, 0x2C, 0x3B, 0x08, 
                0x2D, 0x3B, 0x3B, 0x3B, 0x3B, 0x2B, 0x3B, 0x3B, 0x3B, 0x3B, 0x35, 0x3B, 0x35, 0x35, 0x3B, 0x08, 
                0x33, 0x3B, 0x32, 0x21, 0x30, 0x2B, 0x3B, 0x32, 0x21, 0x30, 0x2B, 0x3B, 0x2C, 0x2B, 0x3B, 0x08, 
                0x33, 0x35, 0x3B, 0x3B, 0x3B, 0x2C, 0x35, 0x3B, 0x3B, 0x3B, 0x2B, 0x3B, 0x3B, 0x2B, 0x3B, 0x08, 
                0x34, 0x2C, 0x3B, 0x32, 0x21, 0x30, 0x2B, 0x32, 0x30, 0x3B, 0x2B, 0x32, 0x30, 0x2C, 0x3B, 0x08, 
                0x31, 0x35, 0x3B, 0x3B, 0x35, 0x3B, 0x2C, 0x3B, 0x3B, 0x35, 0x2C, 0x3B, 0x3B, 0x35, 0x35, 0x08, 
                0x31, 0x2C, 0x32, 0x30, 0x2B, 0x32, 0x30, 0x35, 0x3B, 0x2B, 0x32, 0x21, 0x30, 0x2C, 0x2C, 0x08, 
                0x31, 0x35, 0x3B, 0x3B, 0x2B, 0x3B, 0x3B, 0x2B, 0x3B, 0x2C, 0x3B, 0x35, 0x3B, 0x3B, 0x3B, 0x08, 
                0x31, 0x2C, 0x32, 0x30, 0x2B, 0x32, 0x30, 0x2B, 0x3B, 0x32, 0x30, 0x2B, 0x32, 0x30, 0x3B, 0x08, 
                0x31, 0x35, 0x35, 0x35, 0x2B, 0x3B, 0x3B, 0x2B, 0x3B, 0x3B, 0x35, 0x2B, 0x3B, 0x35, 0x35, 0x08, 
                0x31, 0x2B, 0x2C, 0x2C, 0x2C, 0x32, 0x30, 0x2B, 0x32, 0x30, 0x2C, 0x2C, 0x3B, 0x2C, 0x2C, 0x08, 
                0x31, 0x2B, 0x3B, 0x3B, 0x35, 0x3B, 0x3B, 0x2B, 0x3B, 0x3B, 0x35, 0x3B, 0x3B, 0x3B, 0x3B, 0x08, 
                0x31, 0x2C, 0x32, 0x30, 0x2B, 0x3B, 0x3B, 0x2C, 0x32, 0x30, 0x2C, 0x32, 0x30, 0x3B, 0x35, 0x08, 
                0x31, 0x3B, 0x3B, 0x3B, 0x2C, 0x3B, 0x3B, 0x3B, 0x3B, 0x3B, 0x3B, 0x3B, 0x3B, 0x3B, 0x2C, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
            },
            .collisionData = {0x0381, 0x3FE5, 0x0401, 0xBDED, 0x8425, 0xDFBD, 0x0221, 0x7E7F, 0x0941, 0x7F7D, 0x0911, 0x7FF7, 0x4101, 0x79F9, 0x0803, 0xFFFF},
            .coords = {COORDS_XY(11,1), COORDS_XY(13,2)},
            .direction = TRAINER_DIRS(DIR_WEST, DIR_NORTH),
            .range = TRAINER_RANGE(2, 1),
            }
    },
    [1] =
    {
        .trainerNum1 = 19,
        .trainerNum2 = 20,
        .trainers =
        {
            [0] =
            {
                .name = _("THEODORE"),
                .facilityClass = FACILITY_CLASS_BLACK_BELT,
                .unused = 0,
                .speechBefore = {EC_WORD_MY, EC_WORD_POWER, EC_WORD_WILL, EC_MOVE2(STOMP), EC_WORD_YOU, EC_WORD_EXCL},
                .speechWin = {EC_WORD_WAAAH, EC_WORD_HAHAHA, EC_WORD_EXCL, EC_WORD_WAAAH, EC_WORD_HAHAHA, EC_WORD_EXCL_EXCL},
                .speechLose = {EC_WORD_I, EC_WORD_WENT, EC_WORD_AT, EC_WORD_IT, EC_WORD_TOO, EC_WORD_HARD},
                .speechAfter = {EC_WORD_YES_SIR_EXCL, EC_WORD_YOU, EC_WORD_LOOK, EC_WORD_NICE, EC_WORD_AND, EC_WORD_GUTSY},
                .mons =
                {
                    [0] =
                    {
                        .species = SPECIES_MEDITITE,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = {MOVE_FOCUS_PUNCH, MOVE_PROTECT, MOVE_NONE, MOVE_NONE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .attackEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 20,
                        .defenseIV = 20,
                        .speedIV = 20,
                        .spAttackIV = 20,
                        .spDefenseIV = 20,
                        .abilityNum = 0,
                        .personality = 0x80,
                        .nickname = _("MEDITITE"),
                        .friendship = 255,
                    },
                    [1] =
                    {
                        .species = SPECIES_HERACROSS,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = {MOVE_FOCUS_PUNCH, MOVE_PROTECT, MOVE_NONE, MOVE_NONE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .attackEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 20,
                        .defenseIV = 20,
                        .speedIV = 20,
                        .spAttackIV = 20,
                        .spDefenseIV = 20,
                        .abilityNum = 1,
                        .personality = 0x80,
                        .nickname = _("HERACROSS"),
                        .friendship = 255,
                    },
                    [2] =
                    {
                        .species = SPECIES_HITMONTOP,
                        .heldItem = ITEM_SHELL_BELL,
                        .moves = {MOVE_FOCUS_PUNCH, MOVE_PROTECT, MOVE_NONE, MOVE_NONE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .attackEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 20,
                        .defenseIV = 20,
                        .speedIV = 20,
                        .spAttackIV = 20,
                        .spDefenseIV = 20,
                        .abilityNum = 0,
                        .personality = 0x3,
                        .nickname = _("HITMONTOP"),
                        .friendship = 255,
                    },
                    [3] =
                    {
                        .species = SPECIES_MACHOP,
                        .heldItem = ITEM_SALAC_BERRY,
                        .moves = {MOVE_FOCUS_PUNCH, MOVE_REVENGE, MOVE_NONE, MOVE_NONE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .attackEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 20,
                        .defenseIV = 20,
                        .speedIV = 20,
                        .spAttackIV = 20,
                        .spDefenseIV = 20,
                        .abilityNum = 0,
                        .personality = 0x4E,
                        .nickname = _("MACHOP"),
                        .friendship = 255,
                    },
                    [4] =
                    {
                        .species = SPECIES_PINSIR,
                        .heldItem = ITEM_SALAC_BERRY,
                        .moves = {MOVE_FOCUS_PUNCH, MOVE_REVENGE, MOVE_NONE, MOVE_NONE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .attackEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 20,
                        .defenseIV = 20,
                        .speedIV = 20,
                        .spAttackIV = 20,
                        .spDefenseIV = 20,
                        .abilityNum = 0,
                        .personality = 0x80,
                        .nickname = _("PINSIR"),
                        .friendship = 255,
                    },
                    [5] =
                    {
                        .species = SPECIES_HITMONCHAN,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = {MOVE_FOCUS_PUNCH, MOVE_REVENGE, MOVE_NONE, MOVE_NONE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .attackEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 20,
                        .defenseIV = 20,
                        .speedIV = 20,
                        .spAttackIV = 20,
                        .spDefenseIV = 20,
                        .abilityNum = 0,
                        .personality = 0x3,
                        .nickname = _("HITMONCHAN"),
                        .friendship = 255,
                    },
                },
            },
            [1] =
            {
                .name = _("JAYDEN"),
                .facilityClass = FACILITY_CLASS_PKMN_BREEDER_F,
                .unused = 0,
                .speechBefore = {EC_WORD_SOME, EC_WORD_THINGS, EC_WORD_YOU, EC_WORD_CAN_T, EC_WORD_DO, EC_WORD_ALONE},
                .speechWin = {EC_WORD_YOU, EC_WORD_WIN, EC_WORD_AS, EC_WORD_A, EC_WORD_GROUP, EC_EMPTY_WORD},
                .speechLose = {EC_WORD_WE, EC_WORD_COULDN_T, EC_WORD_WIN, EC_WORD_TOGETHER, EC_WORD_QUES, EC_EMPTY_WORD},
                .speechAfter = {EC_WORD_MAYBE, EC_WORD_I, EC_WORD_NEED, EC_WORD_A, EC_WORD_BOY, EC_WORD_FRIEND},
                .mons =
                {
                    [0] =
                    {
                        .species = SPECIES_VULPIX,
                        .heldItem = ITEM_LUM_BERRY,
                        .moves = {MOVE_WILL_O_WISP, MOVE_CONFUSE_RAY, MOVE_TAIL_WHIP, MOVE_OVERHEAT},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .defenseEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0xF,
                        .nickname = _("VULPIX"),
                        .friendship = 255,
                    },
                    [1] =
                    {
                        .species = SPECIES_MINUN,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = {MOVE_THUNDER_WAVE, MOVE_CHARM, MOVE_ENCORE, MOVE_SPARK},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .defenseEV = 252,
                        .spAttackEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x8C,
                        .nickname = _("MINUN"),
                        .friendship = 255,
                    },
                    [2] =
                    {
                        .species = SPECIES_ROSELIA,
                        .heldItem = ITEM_LUM_BERRY,
                        .moves = {MOVE_TOXIC, MOVE_LEECH_SEED, MOVE_SWEET_SCENT, MOVE_GIGA_DRAIN},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .defenseEV = 252,
                        .spAttackEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 1,
                        .personality = 0xF,
                        .nickname = _("ROSELIA"),
                        .friendship = 255,
                    },
                    [3] =
                    {
                        .species = SPECIES_MR_MIME,
                        .heldItem = ITEM_SITRUS_BERRY,
                        .moves = {MOVE_SAFEGUARD, MOVE_REFLECT, MOVE_LIGHT_SCREEN, MOVE_PSYCHIC},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .defenseEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x82,
                        .nickname = _("MR. MIME"),
                        .friendship = 255,
                    },
                    [4] =
                    {
                        .species = SPECIES_PLUSLE,
                        .heldItem = ITEM_SITRUS_BERRY,
                        .moves = {MOVE_RAIN_DANCE, MOVE_LIGHT_SCREEN, MOVE_HELPING_HAND, MOVE_THUNDER},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .defenseEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x5,
                        .nickname = _("PLUSLE"),
                        .friendship = 255,
                    },
                    [5] =
                    {
                        .species = SPECIES_TOGEPI,
                        .heldItem = ITEM_SITRUS_BERRY,
                        .moves = {MOVE_LIGHT_SCREEN, MOVE_REFLECT, MOVE_FOLLOW_ME, MOVE_METRONOME},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .defenseEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 1,
                        .personality = 0x37,
                        .nickname = _("TOGEPI"),
                        .friendship = 255,
                    },
                },
            },
        },
            .display = {
            .metatileData = {
                0xD1, 0xD5, 0xD5, 0xD5, 0xD9, 0xD9, 0x1B, 0x1C, 0x1D, 0xC5, 0xC6, 0xCE, 0xD5, 0xDB, 0xD5, 0x08, 
                0xD1, 0xCB, 0xC4, 0xC4, 0xDB, 0xDB, 0xC4, 0xC4, 0xC4, 0xCC, 0xCC, 0xCC, 0xCB, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xDB, 0x17, 0x17, 0x17, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0xCB, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xD5, 0x17, 0x17, 0x17, 0xD5, 0xD5, 0xD5, 0xD5, 0xDF, 0xCB, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xCB, 0x17, 0x17, 0x1F, 0xCB, 0xCB, 0xC4, 0xC4, 0xDB, 0xCB, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xCB, 0xDB, 0xDB, 0xC4, 0xCB, 0xCB, 0xDF, 0xD5, 0xD5, 0xCB, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xCB, 0xDB, 0xDB, 0xDB, 0xCB, 0xCB, 0xDB, 0xC4, 0xC4, 0xCB, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xCB, 0xDB, 0xDB, 0xDB, 0xCB, 0xCB, 0xD5, 0xD5, 0xDF, 0xCB, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xCB, 0xDB, 0xDB, 0xDB, 0xCB, 0xCB, 0xC4, 0xC4, 0xDB, 0xCB, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xCB, 0xDB, 0xDB, 0xDB, 0xCB, 0xCB, 0xDF, 0xD5, 0xD5, 0xCB, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xCB, 0xDB, 0xDB, 0xDB, 0xC4, 0xC4, 0xDB, 0xC4, 0xC4, 0xC4, 0xDB, 0xCB, 0x08, 
                0xD1, 0xCB, 0xDB, 0xCB, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0xDB, 0xD5, 0xD5, 0xD5, 0xCB, 0x08, 
                0xD1, 0xC4, 0xDB, 0xCB, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0x9B, 0xDB, 0xCB, 0xCB, 0xCB, 0xCB, 0x08, 
                0xD1, 0xDB, 0xDB, 0xCB, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0x9B, 0xDB, 0xCB, 0xCB, 0xCB, 0xCB, 0x08, 
                0xD1, 0xDB, 0xDB, 0xC4, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0x9B, 0xDB, 0xC4, 0xC4, 0xC4, 0xC4, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
            },
            .collisionData = {0x0381, 0x73FB, 0x400B, 0x400B, 0x51EB, 0x538B, 0x51BB, 0x518B, 0x51EB, 0x518B, 0x51BB, 0x5003, 0x501F, 0x101F, 0x101F, 0xFFFF},
            .coords = {COORDS_XY(4,11), COORDS_XY(9,14)},
            .direction = TRAINER_DIRS(DIR_SOUTH, DIR_EAST),
            .range = TRAINER_RANGE(3, 5),
            }
    },
    [2] =
    {
        .trainerNum1 = 21,
        .trainerNum2 = 22,
        .trainers =
        {
            [0] =
            {
                .name = _("SALVADORE"),
                .facilityClass = FACILITY_CLASS_PKMN_BREEDER_M,
                .unused = 0,
                .speechBefore = {EC_WORD_LET_S, EC_MOVE2(WRAP), EC_WORD_THINGS, EC_WORD_UP, EC_WORD_HERE, EC_WORD_HEY_QUES},
                .speechWin = {EC_WORD_WOULD, EC_WORD_YOU, EC_WORD_LIKE, EC_WORD_TO, EC_WORD_GO_HOME, EC_WORD_QUES},
                .speechLose = {EC_WORD_I, EC_WORD_WILL, EC_WORD_MAKE, EC_WORD_YOU, EC_WORD_SORRY, EC_WORD_FRIEND},
                .speechAfter = {EC_WORD_HAVEN_T, EC_WORD_YOU, EC_WORD_DONE, EC_WORD_ENOUGH, EC_WORD_QUES, EC_EMPTY_WORD},
                .mons =
                {
                    [0] =
                    {
                        .species = SPECIES_VAPOREON,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = {MOVE_HAZE, MOVE_HELPING_HAND, MOVE_TICKLE, MOVE_WATER_PULSE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 250,
                        .defenseEV = 130,
                        .spDefenseEV = 130,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x37,
                        .nickname = _("VAPOREON"),
                        .friendship = 0,
                    },
                    [1] =
                    {
                        .species = SPECIES_DODRIO,
                        .heldItem = ITEM_KINGS_ROCK,
                        .moves = {MOVE_HAZE, MOVE_TRI_ATTACK, MOVE_TAUNT, MOVE_TORMENT},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .speedEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 1,
                        .personality = 0x8A,
                        .nickname = _("DODRIO"),
                        .friendship = 0,
                    },
                    [2] =
                    {
                        .species = SPECIES_OMASTAR,
                        .heldItem = ITEM_QUICK_CLAW,
                        .moves = {MOVE_HAZE, MOVE_HYDRO_PUMP, MOVE_TICKLE, MOVE_ATTRACT},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 250,
                        .defenseEV = 130,
                        .spDefenseEV = 130,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x14,
                        .nickname = _("OMASTAR"),
                        .friendship = 255,
                    },
                    [3] =
                    {
                        .species = SPECIES_LICKITUNG,
                        .heldItem = ITEM_CHESTO_BERRY,
                        .moves = {MOVE_BELLY_DRUM, MOVE_REST, MOVE_MUD_SLAP, MOVE_SWAGGER},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 250,
                        .defenseEV = 130,
                        .spDefenseEV = 130,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x8,
                        .nickname = _("LICKITUNG"),
                        .friendship = 255,
                    },
                    [4] =
                    {
                        .species = SPECIES_SLOWBRO,
                        .heldItem = ITEM_LEFTOVERS,
                        .moves = {MOVE_BELLY_DRUM, MOVE_MUD_SLAP, MOVE_SWAGGER, MOVE_AMNESIA},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 250,
                        .defenseEV = 130,
                        .spDefenseEV = 130,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 1,
                        .personality = 0x94,
                        .nickname = _("SLOWBRO"),
                        .friendship = 0,
                    },
                    [5] =
                    {
                        .species = SPECIES_LINOONE,
                        .heldItem = ITEM_LUM_BERRY,
                        .moves = {MOVE_BELLY_DRUM, MOVE_REST, MOVE_MUD_SLAP, MOVE_SWAGGER},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 250,
                        .defenseEV = 130,
                        .spDefenseEV = 130,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 20,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x8,
                        .nickname = _("LINOONE"),
                        .friendship = 255,
                    },
                },
            },
            [1] =
            {
                .name = _("VERONICA"),
                .facilityClass = FACILITY_CLASS_PKMN_BREEDER_F,
                .unused = 0,
                .speechBefore = {EC_WORD_I_AM, EC_WORD_THE, EC_WORD_STRONG, EC_WORD_BEAUTY, EC_WORD_AROUND, EC_WORD_HERE},
                .speechWin = {EC_WORD_I_AM, EC_WORD_STRONG, EC_WORD_THAT_S, EC_WORD_WHY, EC_WORD_EXCL, EC_EMPTY_WORD},
                .speechLose = {EC_WORD_I_AM, EC_WORD_TIRED, EC_WORD_TODAY, EC_WORD_THAT_S, EC_WORD_WHY, EC_WORD_EXCL},
                .speechAfter = {EC_WORD_WHY, EC_WORD_YES, EC_WORD_I_AM, EC_WORD_ANGRY, EC_WORD_THANK_YOU, EC_WORD_EXCL},
                .mons =
                {
                    [0] =
                    {
                        .species = SPECIES_SKITTY,
                        .heldItem = ITEM_FOCUS_BAND,
                        .moves = {MOVE_PSYCH_UP, MOVE_DOUBLE_EDGE, MOVE_SHADOW_BALL, MOVE_IRON_TAIL},
                        .level = 0,
                        .ppBonuses = 0,
                        .attackEV = 252,
                        .speedEV = 252,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = MAX_PER_STAT_IVS,
                        .attackIV = MAX_PER_STAT_IVS,
                        .defenseIV = MAX_PER_STAT_IVS,
                        .speedIV = MAX_PER_STAT_IVS,
                        .spAttackIV = MAX_PER_STAT_IVS,
                        .spDefenseIV = MAX_PER_STAT_IVS,
                        .abilityNum = 0,
                        .personality = 0xCB,
                        .nickname = _("SKITTY"),
                        .friendship = 255,
                    },
                    [1] =
                    {
                        .species = SPECIES_MEDICHAM,
                        .heldItem = ITEM_QUICK_CLAW,
                        .moves = {MOVE_PSYCH_UP, MOVE_HIGH_JUMP_KICK, MOVE_MEGA_KICK, MOVE_ROCK_SLIDE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .attackEV = 6,
                        .speedEV = 252,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0xD,
                        .nickname = _("MEDICHAM"),
                        .friendship = 0,
                    },
                    [2] =
                    {
                        .species = SPECIES_STANTLER,
                        .heldItem = ITEM_SHELL_BELL,
                        .moves = {MOVE_PSYCH_UP, MOVE_RETURN, MOVE_EARTHQUAKE, MOVE_SHADOW_BALL},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .attackEV = 252,
                        .speedEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x3,
                        .nickname = _("STANTLER"),
                        .friendship = 0,
                    },
                    [3] =
                    {
                        .species = SPECIES_NIDOQUEEN,
                        .heldItem = ITEM_SALAC_BERRY,
                        .moves = {MOVE_SUPERPOWER, MOVE_BITE, MOVE_CHARM, MOVE_FLATTER},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 252,
                        .attackEV = 252,
                        .spDefenseEV = 6,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 20,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x3,
                        .nickname = _("NIDOQUEEN"),
                        .friendship = 0,
                    },
                    [4] =
                    {
                        .species = SPECIES_NINETALES,
                        .heldItem = ITEM_SALAC_BERRY,
                        .moves = {MOVE_OVERHEAT, MOVE_QUICK_ATTACK, MOVE_SPITE, MOVE_TAIL_WHIP},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 6,
                        .speedEV = 252,
                        .spAttackEV = 252,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0xD7,
                        .nickname = _("NINETALES"),
                        .friendship = 255,
                    },
                    [5] =
                    {
                        .species = SPECIES_CHARIZARD,
                        .heldItem = ITEM_SALAC_BERRY,
                        .moves = {MOVE_OVERHEAT, MOVE_BEAT_UP, MOVE_SCARY_FACE, MOVE_GROWL},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 6,
                        .speedEV = 252,
                        .spAttackEV = 252,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x28,
                        .nickname = _("CHARIZARD"),
                        .friendship = 255,
                    },
                },
            },
        },
            .display = {
            .metatileData = {
                0x31, 0x35, 0x35, 0x35, 0x26, 0x26, 0x13, 0x14, 0x15, 0x38, 0x26, 0x2E, 0x35, 0x35, 0x3B, 0x08, 
                0x69, 0x63, 0x64, 0x64, 0x64, 0x64, 0x71, 0x71, 0x71, 0x72, 0x64, 0x64, 0x64, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x43, 0x41, 0x40, 0x41, 0x42, 0x41, 0x41, 0x4A, 0x42, 0x41, 0x41, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x64, 0x73, 0x73, 0x73, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x41, 0x43, 0x4B, 0x43, 0x43, 0x41, 0x42, 0x42, 0x40, 0x41, 0x40, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x73, 0x73, 0x64, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x41, 0x40, 0x42, 0x42, 0x41, 0x41, 0x42, 0x4A, 0x42, 0x41, 0x42, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x64, 0x73, 0x73, 0x73, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x41, 0x42, 0x41, 0x43, 0x4B, 0x41, 0x41, 0x41, 0x40, 0x43, 0x41, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x73, 0x73, 0x73, 0x73, 0x64, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x63, 0x73, 0x08, 
                0x69, 0x63, 0x41, 0x40, 0x43, 0x41, 0x42, 0x42, 0x41, 0x4A, 0x42, 0x41, 0x42, 0x63, 0x73, 0x08, 
                0x69, 0x64, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x64, 0x73, 0x73, 0x73, 0x64, 0x73, 0x08, 
                0x69, 0x43, 0x43, 0x41, 0x42, 0x42, 0x41, 0x43, 0x41, 0x41, 0x40, 0x42, 0x41, 0x42, 0x73, 0x08, 
                0x69, 0x42, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x42, 0x73, 0x08,
            },
            .collisionData = {0x0381, 0x7C3D, 0x4005, 0x4005, 0x4005, 0x4045, 0x4005, 0x4805, 0x4005, 0x4045, 0x4005, 0x4205, 0x4005, 0x4045, 0x1, 0x1},
            .coords = {COORDS_XY(5,2), COORDS_XY(9,2)},
            .direction = TRAINER_DIRS(DIR_WEST, DIR_EAST),
            .range = TRAINER_RANGE(3, 3),
            }
    },
    [3] =
    {
        .trainerNum1 = 23,
        .trainerNum2 = 24,
        .trainers =
        {
            [0] =
            {
                .name = _("KEENAN"),
                .facilityClass = FACILITY_CLASS_PSYCHIC_M,
                .unused = 0,
                .speechBefore = {EC_WORD_YOU, EC_WORD_LOOK, EC_WORD_SO, EC_WORD_HURRIED, EC_EMPTY_WORD, EC_EMPTY_WORD},
                .speechWin = {EC_WORD_TOO, EC_WORD_BAD, EC_EMPTY_WORD, EC_WORD_TIME, EC_WORD_IS, EC_WORD_UP},
                .speechLose = {EC_WORD_DOES, EC_WORD_THE, EC_WORD_TIME, EC_WORD_WORRY, EC_WORD_YOU, EC_WORD_QUES},
                .speechAfter = {EC_WORD_YOU, EC_WORD_DON_T, EC_WORD_HAVE, EC_WORD_TIME, EC_WORD_TO, EC_WORD_CHAT},
                .mons =
                {
                    [0] =
                    {
                        .species = SPECIES_ALAKAZAM,
                        .heldItem = ITEM_PETAYA_BERRY,
                        .moves = {MOVE_SKILL_SWAP, MOVE_FIRE_PUNCH, MOVE_ICE_PUNCH, MOVE_REFLECT},
                        .level = 0,
                        .ppBonuses = 0,
                        .speedEV = 255,
                        .spAttackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x41,
                        .nickname = _("ALAKAZAM"),
                        .friendship = 255,
                    },
                    [1] =
                    {
                        .species = SPECIES_BLISSEY,
                        .heldItem = ITEM_LEFTOVERS,
                        .moves = {MOVE_SKILL_SWAP, MOVE_EGG_BOMB, MOVE_THUNDERBOLT, MOVE_SING},
                        .level = 0,
                        .ppBonuses = 0,
                        .defenseEV = 255,
                        .spAttackEV = 155,
                        .spDefenseEV = 100,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 1,
                        .personality = 0xF,
                        .nickname = _("BLISSEY"),
                        .friendship = 255,
                    },
                    [2] =
                    {
                        .species = SPECIES_GRUMPIG,
                        .heldItem = ITEM_TWISTED_SPOON,
                        .moves = {MOVE_SKILL_SWAP, MOVE_PSYCHIC, MOVE_CONFUSE_RAY, MOVE_REST},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 110,
                        .defenseEV = 200,
                        .spAttackEV = 200,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x8C,
                        .nickname = _("GRUMPIG"),
                        .friendship = 255,
                    },
                    [3] =
                    {
                        .species = SPECIES_GARDEVOIR,
                        .heldItem = ITEM_SALAC_BERRY,
                        .moves = {MOVE_SKILL_SWAP, MOVE_DREAM_EATER, MOVE_HYPNOSIS, MOVE_PROTECT},
                        .level = 0,
                        .ppBonuses = 0,
                        .speedEV = 255,
                        .spAttackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 1,
                        .personality = 0xF,
                        .nickname = _("GARDEVOIR"),
                        .friendship = 255,
                    },
                    [4] =
                    {
                        .species = SPECIES_VENOMOTH,
                        .heldItem = ITEM_BRIGHT_POWDER,
                        .moves = {MOVE_SKILL_SWAP, MOVE_SIGNAL_BEAM, MOVE_SLEEP_POWDER, MOVE_TOXIC},
                        .level = 0,
                        .ppBonuses = 0,
                        .attackEV = 255,
                        .speedEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0x80,
                        .nickname = _("VENOMOTH"),
                        .friendship = 255,
                    },
                    [5] =
                    {
                        .species = SPECIES_ESPEON,
                        .heldItem = ITEM_LUM_BERRY,
                        .moves = {MOVE_SKILL_SWAP, MOVE_PSYBEAM, MOVE_SWIFT, MOVE_ATTRACT},
                        .level = 0,
                        .ppBonuses = 0,
                        .speedEV = 255,
                        .spAttackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 5,
                        .attackIV = 5,
                        .defenseIV = 5,
                        .speedIV = 5,
                        .spAttackIV = 5,
                        .spDefenseIV = 5,
                        .abilityNum = 0,
                        .personality = 0xF,
                        .nickname = _("ESPEON"),
                        .friendship = 255,
                    },
                },
            },
            [1] =
            {
                .name = _("KRISTINA"),
                .facilityClass = FACILITY_CLASS_AROMA_LADY,
                .unused = 0,
                .speechBefore = {EC_WORD_IT_S, EC_WORD_HOT, EC_WORD_ELLIPSIS, EC_WORD_WHAT, EC_WORD_A, EC_WORD_STENCH},
                .speechWin = {EC_WORD_I, EC_WORD_REALLY, EC_WORD_LIKE, EC_WORD_AN, EC_WORD_OFFENSIVE, EC_WORD_STENCH},
                .speechLose = {EC_WORD_I, EC_MOVE2(COVET), EC_WORD_ANY, EC_WORD_STRONG, EC_WORD_STENCH, EC_EMPTY_WORD},
                .speechAfter = {EC_WORD_MY, EC_WORD_SENSE, EC_WORD_OF, EC_WORD_SMELL, EC_WORD_ISN_T, EC_WORD_NORMAL},
                .mons =
                {
                    [0] =
                    {
                        .species = SPECIES_WEEZING,
                        .heldItem = ITEM_POISON_BARB,
                        .moves = {MOVE_TOXIC, MOVE_SLUDGE_BOMB, MOVE_SMOKESCREEN, MOVE_HAZE},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 255,
                        .attackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x3,
                        .nickname = _("WEEZING"),
                        .friendship = 255,
                    },
                    [1] =
                    {
                        .species = SPECIES_GLOOM,
                        .heldItem = ITEM_MIRACLE_SEED,
                        .moves = {MOVE_PETAL_DANCE, MOVE_SYNTHESIS, MOVE_SUNNY_DAY, MOVE_SOLAR_BEAM},
                        .level = 0,
                        .ppBonuses = 0,
                        .speedEV = 255,
                        .spAttackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x8C,
                        .nickname = _("GLOOM"),
                        .friendship = 255,
                    },
                    [2] =
                    {
                        .species = SPECIES_MUK,
                        .heldItem = ITEM_BRIGHT_POWDER,
                        .moves = {MOVE_SCREECH, MOVE_DISABLE, MOVE_SLUDGE_BOMB, MOVE_ACID_ARMOR},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 255,
                        .attackEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x3,
                        .nickname = _("MUK"),
                        .friendship = 255,
                    },
                    [3] =
                    {
                        .species = SPECIES_TROPIUS,
                        .heldItem = ITEM_WHITE_HERB,
                        .moves = {MOVE_SUNNY_DAY, MOVE_SOLAR_BEAM, MOVE_SWEET_SCENT, MOVE_AERIAL_ACE},
                        .level = 0,
                        .ppBonuses = 0,
                        .attackEV = 120,
                        .speedEV = 255,
                        .spAttackEV = 135,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x83,
                        .nickname = _("TROPIUS"),
                        .friendship = 255,
                    },
                    [4] =
                    {
                        .species = SPECIES_BELLOSSOM,
                        .heldItem = ITEM_MENTAL_HERB,
                        .moves = {MOVE_SWEET_SCENT, MOVE_PETAL_DANCE, MOVE_STUN_SPORE, MOVE_SLUDGE_BOMB},
                        .level = 0,
                        .ppBonuses = 0,
                        .hpEV = 255,
                        .speedEV = 255,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x6,
                        .nickname = _("BELLOSSOM"),
                        .friendship = 255,
                    },
                    [5] =
                    {
                        .species = SPECIES_MEGANIUM,
                        .heldItem = ITEM_MIRACLE_SEED,
                        .moves = {MOVE_RAZOR_LEAF, MOVE_BODY_SLAM, MOVE_LEECH_SEED, MOVE_SYNTHESIS},
                        .level = 0,
                        .ppBonuses = 0,
                        .attackEV = 200,
                        .speedEV = 110,
                        .spAttackEV = 200,
                        .otId = TRAINER_HILL_OTID,
                        .hpIV = 15,
                        .attackIV = 15,
                        .defenseIV = 15,
                        .speedIV = 15,
                        .spAttackIV = 15,
                        .spDefenseIV = 15,
                        .abilityNum = 0,
                        .personality = 0x1F,
                        .nickname = _("MEGANIUM"),
                        .friendship = 255,
                    },
                },
            },
        },
            .display = {
            .metatileData = {
                0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x26, 0x1B, 0x1C, 0x1D, 0x25, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                0x1F, 0x17, 0x17, 0x17, 0x17, 0x2B, 0x24, 0x24, 0x24, 0x2B, 0x3B, 0x3B, 0x3B, 0x3B, 0x3B, 0x08, 
                0x33, 0x17, 0x17, 0x17, 0x17, 0x2B, 0x3B, 0x3B, 0x3B, 0x2B, 0x35, 0x35, 0x3B, 0x35, 0x35, 0x08, 
                0x33, 0x17, 0x17, 0x17, 0x17, 0x2B, 0x3B, 0x3B, 0x3B, 0x2B, 0x2C, 0x2C, 0x3B, 0x2C, 0x2C, 0x08, 
                0x33, 0x17, 0x1F, 0x17, 0x17, 0x2B, 0x3B, 0x3B, 0x3B, 0x2B, 0x17, 0x17, 0x17, 0x17, 0x17, 0x08, 
                0x34, 0x17, 0x2C, 0x17, 0x17, 0x2B, 0x3B, 0x3B, 0x3B, 0x2C, 0x17, 0x17, 0x17, 0x17, 0x17, 0x08, 
                0x17, 0x17, 0x17, 0x17, 0x17, 0x2C, 0x3B, 0x3B, 0x3B, 0x3B, 0x17, 0x17, 0x17, 0x17, 0x17, 0x08, 
                0x1F, 0x17, 0x17, 0x17, 0x17, 0x1F, 0x17, 0x3B, 0x3B, 0x3B, 0x17, 0x17, 0x17, 0x17, 0x17, 0x08, 
                0x33, 0x17, 0x17, 0x17, 0x17, 0x2B, 0x3B, 0x3B, 0x3B, 0x35, 0x1F, 0x17, 0x17, 0x1F, 0x17, 0x08, 
                0x34, 0x17, 0x17, 0x17, 0x17, 0x2B, 0x3B, 0x3B, 0x3B, 0x2B, 0x2C, 0x3B, 0x3B, 0x2B, 0x17, 0x08, 
                0x17, 0x17, 0x17, 0x1F, 0x17, 0x2B, 0x3B, 0x3B, 0x3B, 0x2B, 0x17, 0x17, 0x17, 0x2C, 0x17, 0x08, 
                0x1F, 0x17, 0x17, 0x2C, 0x17, 0x2B, 0x3B, 0x3B, 0x3B, 0x2B, 0x17, 0x3B, 0x1F, 0x3B, 0x17, 0x08, 
                0x33, 0x17, 0x17, 0x17, 0x17, 0x2B, 0x3B, 0x3B, 0x3B, 0x2B, 0x17, 0x3B, 0x2B, 0x3B, 0x17, 0x08, 
                0x33, 0x17, 0x17, 0x17, 0x17, 0x2B, 0x35, 0x35, 0x35, 0x2B, 0x17, 0x3B, 0x2C, 0x3B, 0x17, 0x08, 
                0x34, 0x17, 0x17, 0x17, 0x17, 0x2C, 0x2C, 0x2C, 0x2C, 0x2C, 0x17, 0x17, 0x17, 0x17, 0x17, 0x08, 
                0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
            },
            .collisionData = {0x0381, 0x7C1, 0x8441, 0x8477, 0x8441, 0xA441, 0x0401, 0x1, 0x8401, 0x8465, 0x0445, 0x1441, 0x8449, 0x8449, 0x87C1, 0xFFFF},
            .coords = {COORDS_XY(7,4), COORDS_XY(7,10)},
            .direction = TRAINER_DIRS(DIR_SOUTH, DIR_NORTH),
            .range = TRAINER_RANGE(3, 3),
            }
    },
};

static const struct TrHillTag sDataTagVariety =
{
    .numTrainers = NUM_TRAINER_HILL_TRAINERS,
    .unused1 = 1,
    .numFloors = NUM_TRAINER_HILL_FLOORS,
    .checksum = 0x00054C15
};

static const struct TrHillFloor sDataTagVariety_Floors[] = {
    [0] =
        {
            .trainerNum1 = 41,
            .trainerNum2 = 42,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("TERRANCE"),
                            .facilityClass = FACILITY_CLASS_GENTLEMAN,
                            .unused = 0,
                            .speechBefore = {EC_WORD_GOOD, EC_WORD_CHILDREN, EC_WORD_WILL, EC_WORD_GET, EC_WORD_A, EC_MOVE2(PRESENT)},
                            .speechWin = {EC_WORD_THANK_YOU, EC_WORD_MY, EC_WORD_VICTORY, EC_WORD_IS, EC_WORD_YOUR, EC_MOVE2(PRESENT)},
                            .speechLose = {EC_WORD_THAT_WAS, EC_WORD_MY, EC_MOVE2(PRESENT), EC_WORD_TO, EC_WORD_YOU, EC_EMPTY_WORD},
                            .speechAfter = {EC_WORD_WELL_THEN, EC_WORD_CONGRATS, EC_WORD_MY, EC_WORD_FRIEND, EC_EMPTY_WORD, EC_EMPTY_WORD},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_DELIBIRD,
                                            .heldItem = ITEM_SITRUS_BERRY,
                                            .moves = {MOVE_PRESENT, MOVE_SPLASH, MOVE_HAIL, MOVE_PROTECT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0x8A,
                                            .nickname = _("DELIBIRD"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_CLEFAIRY,
                                            .heldItem = ITEM_SITRUS_BERRY,
                                            .moves = {MOVE_PRESENT, MOVE_COSMIC_POWER, MOVE_LIGHT_SCREEN, MOVE_MOONLIGHT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xD,
                                            .nickname = _("CLEFAIRY"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_PIKACHU,
                                            .heldItem = ITEM_SITRUS_BERRY,
                                            .moves = {MOVE_PRESENT, MOVE_GROWL, MOVE_TAIL_WHIP, MOVE_AGILITY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xD,
                                            .nickname = _("PIKACHU"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_MARILL,
                                            .heldItem = ITEM_SITRUS_BERRY,
                                            .moves = {MOVE_PRESENT, MOVE_DEFENSE_CURL, MOVE_TAIL_WHIP, MOVE_ENDURE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0x8A,
                                            .nickname = _("MARILL"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_JIGGLYPUFF,
                                            .heldItem = ITEM_SITRUS_BERRY,
                                            .moves = {MOVE_PRESENT, MOVE_SING, MOVE_DISABLE, MOVE_REST},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xD,
                                            .nickname = _("JIGGLYPUFF"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_TOGETIC,
                                            .heldItem = ITEM_SITRUS_BERRY,
                                            .moves = {MOVE_PRESENT, MOVE_CHARM, MOVE_SWEET_KISS, MOVE_WISH},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0x26,
                                            .nickname = _("TOGETIC"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("ELIZABETH"),
                            .facilityClass = FACILITY_CLASS_LADY,
                            .unused = 0,
                            .speechBefore = {EC_WORD_CAN, EC_WORD_YOU, EC_WORD_SENSE, EC_WORD_ME, EC_MOVE(FOCUS_ENERGY), EC_WORD_QUES},
                            .speechWin = {EC_WORD_THAT_S, EC_WORD_ABOUT, EC_WORD_RIGHT, EC_WORD_I, EC_WORD_WOULD, EC_WORD_THINK},
                            .speechLose = {EC_WORD_THINGS, EC_WORD_DON_T, EC_WORD_ALWAYS, EC_WORD_WORK, EC_WORD_OUT, EC_EMPTY_WORD},
                            .speechAfter = {EC_WORD_I_AM, EC_WORD_FEELING, EC_WORD_LONESOME, EC_WORD_AND, EC_WORD_SAD, EC_WORD_NOW},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_WIGGLYTUFF,
                                            .heldItem = ITEM_SILK_SCARF,
                                            .moves = {MOVE_ROLLOUT, MOVE_DEFENSE_CURL, MOVE_SING, MOVE_DREAM_EATER},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0xC1,
                                            .nickname = _("WIGGLYTUFF"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_SABLEYE,
                                            .heldItem = ITEM_SILK_SCARF,
                                            .moves = {MOVE_ASTONISH, MOVE_FEINT_ATTACK, MOVE_DETECT, MOVE_CONFUSE_RAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x87,
                                            .nickname = _("SABLEYE"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_GRUMPIG,
                                            .heldItem = ITEM_SILK_SCARF,
                                            .moves = {MOVE_PSYBEAM, MOVE_MAGIC_COAT, MOVE_BOUNCE, MOVE_FUTURE_SIGHT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0xA,
                                            .nickname = _("GRUMPIG"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_CORSOLA,
                                            .heldItem = ITEM_SILK_SCARF,
                                            .moves = {MOVE_BUBBLE_BEAM, MOVE_ROCK_BLAST, MOVE_REFLECT, MOVE_LIGHT_SCREEN},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x12,
                                            .nickname = _("CORSOLA"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_CLAMPERL,
                                            .heldItem = ITEM_SILK_SCARF,
                                            .moves = {MOVE_WHIRLPOOL, MOVE_IRON_DEFENSE, MOVE_ENDURE, MOVE_CONFUSE_RAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0xA,
                                            .nickname = _("CLAMPERL"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_STARMIE,
                                            .heldItem = ITEM_SILK_SCARF,
                                            .moves = {MOVE_DIVE, MOVE_ICY_WIND, MOVE_SWIFT, MOVE_SKILL_SWAP},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0xA,
                                            .nickname = _("STARMIE"),
                                            .friendship = 255,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x39, 0x13, 0x14, 0x15, 0x38, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x40, 0x41, 0x41, 0x41, 0x41, 0x41, 0x42, 0x43, 0x43, 0x43, 0x43, 0x43, 0x40, 0x41, 0x41, 0x08, 
                    0x40, 0xFB, 0x43, 0x41, 0x41, 0x41, 0x42, 0x40, 0x43, 0x41, 0x42, 0x42, 0x42, 0xFB, 0x41, 0x08, 
                    0x40, 0x41, 0xFB, 0x41, 0x41, 0x41, 0x42, 0x40, 0x43, 0x41, 0x43, 0x43, 0xFB, 0x40, 0x41, 0x08, 
                    0x40, 0x41, 0x41, 0xFB, 0x43, 0x41, 0x42, 0x40, 0x43, 0x43, 0x43, 0xFB, 0x41, 0x40, 0x41, 0x08, 
                    0x40, 0x41, 0x41, 0x41, 0x42, 0x42, 0x42, 0x40, 0x43, 0x43, 0x43, 0x41, 0x41, 0x40, 0x41, 0x08, 
                    0x40, 0x41, 0x41, 0x41, 0xFE, 0xFE, 0xFE, 0xFB, 0xFE, 0xFE, 0xFE, 0x41, 0x41, 0x40, 0x41, 0x08, 
                    0x40, 0x41, 0x41, 0x41, 0xFE, 0xFE, 0xFE, 0xFB, 0xFE, 0xFE, 0xFE, 0x41, 0x41, 0x40, 0x41, 0x08, 
                    0x40, 0x41, 0x42, 0x41, 0xFE, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFE, 0x41, 0x41, 0x40, 0x41, 0x08, 
                    0x40, 0x41, 0x41, 0x41, 0xFE, 0xFE, 0xFE, 0xFB, 0xFE, 0xFE, 0xFE, 0x41, 0x41, 0x40, 0x41, 0x08, 
                    0x40, 0x41, 0x41, 0x41, 0xFE, 0xFE, 0xFE, 0xFB, 0xFE, 0xFE, 0xFE, 0x41, 0x41, 0x40, 0x41, 0x08, 
                    0x40, 0x42, 0x41, 0x43, 0x43, 0x43, 0x41, 0x40, 0x42, 0x42, 0x42, 0x42, 0x41, 0x40, 0x41, 0x08, 
                    0x40, 0x40, 0x41, 0xFB, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0xFB, 0x41, 0x40, 0x41, 0x08, 
                    0x40, 0x40, 0xFB, 0x43, 0x43, 0x41, 0x41, 0x41, 0x42, 0x42, 0x42, 0x42, 0xFB, 0x40, 0x41, 0x08, 
                    0x40, 0xFB, 0x43, 0x43, 0x41, 0x41, 0x43, 0x43, 0x43, 0x43, 0x43, 0x43, 0x43, 0xFB, 0x41, 0x08, 
                    0x40, 0x42, 0x42, 0x42, 0x42, 0x42, 0x42, 0x42, 0x42, 0x42, 0x42, 0x42, 0x42, 0x42, 0x42, 0x08,
                },
                .collisionData = {0x0381, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1},
                .coords = {COORDS_XY(5,8), COORDS_XY(9,8)},
                .direction = TRAINER_DIRS(DIR_WEST, DIR_EAST),
                .range = TRAINER_RANGE(3, 3),
            }
        },
    [1] =
        {
            .trainerNum1 = 43,
            .trainerNum2 = 44,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("ANNABELL"),
                            .facilityClass = FACILITY_CLASS_PARASOL_LADY,
                            .unused = 0,
                            .speechBefore = {EC_WORD_HI, EC_WORD_ARE, EC_WORD_YOU, EC_WORD_FEELING, EC_WORD_UPBEAT, EC_WORD_QUES},
                            .speechWin = {EC_WORD_AREN_T, EC_WORD_YOU, EC_WORD_FEELING, EC_WORD_WELL, EC_WORD_QUES, EC_EMPTY_WORD},
                            .speechLose = {EC_WORD_I, EC_WORD_CAN, EC_WORD_SEE, EC_WORD_YOU_RE, EC_WORD_FEELING, EC_WORD_GREAT},
                            .speechAfter = {EC_WORD_YOUR, EC_WORD_POKEMON, EC_WORD_LOOK, EC_WORD_READY, EC_WORD_TO, EC_WORD_HUSTLE},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_JIGGLYPUFF,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_SING, MOVE_HYPER_VOICE, MOVE_ATTRACT, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 85,
                                            .attackEV = 85,
                                            .defenseEV = 85,
                                            .speedEV = 85,
                                            .spAttackEV = 85,
                                            .spDefenseEV = 85,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("JIGGLYPUFF"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_JYNX,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_PERISH_SONG, MOVE_FAKE_TEARS, MOVE_ATTRACT, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 85,
                                            .attackEV = 85,
                                            .defenseEV = 85,
                                            .speedEV = 85,
                                            .spAttackEV = 85,
                                            .spDefenseEV = 85,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("JYNX"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_EXPLOUD,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_HOWL, MOVE_HYPER_VOICE, MOVE_ATTRACT, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 85,
                                            .attackEV = 85,
                                            .defenseEV = 85,
                                            .speedEV = 85,
                                            .spAttackEV = 85,
                                            .spDefenseEV = 85,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x96,
                                            .nickname = _("EXPLOUD"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_ABSOL,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_SWORDS_DANCE, MOVE_SLASH, MOVE_ATTRACT, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 85,
                                            .attackEV = 85,
                                            .defenseEV = 85,
                                            .speedEV = 85,
                                            .spAttackEV = 85,
                                            .spDefenseEV = 85,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x96,
                                            .nickname = _("ABSOL"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_PIDGEOTTO,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_FEATHER_DANCE, MOVE_AERIAL_ACE, MOVE_ATTRACT, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 85,
                                            .attackEV = 85,
                                            .defenseEV = 85,
                                            .speedEV = 85,
                                            .spAttackEV = 85,
                                            .spDefenseEV = 85,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x96,
                                            .nickname = _("PIDGEOTTO"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_ALTARIA,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_DRAGON_DANCE, MOVE_AERIAL_ACE, MOVE_ATTRACT, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 85,
                                            .attackEV = 85,
                                            .defenseEV = 85,
                                            .speedEV = 85,
                                            .spAttackEV = 85,
                                            .spDefenseEV = 85,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("ALTARIA"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("COLEMAN"),
                            .facilityClass = FACILITY_CLASS_COLLECTOR,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_SEEK, EC_WORD_SOMEONE, EC_WORD_AS, EC_WORD_MY, EC_WORD_IDOL},
                            .speechWin = {EC_WORD_DID, EC_WORD_MY, EC_WORD_IDOL, EC_WORD_SEE, EC_WORD_THAT, EC_WORD_QUES},
                            .speechLose = {EC_WORD_YOU_RE, EC_WORD_LIKE, EC_WORD_A, EC_MOVE(HEAL_BELL), EC_WORD_TO_ME, EC_WORD_EXCL},
                            .speechAfter = {EC_WORD_I_VE, EC_WORD_DECIDED, EC_WORD_I_CHOOSE_YOU, EC_WORD_AS, EC_WORD_MY, EC_WORD_IDOL},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_CHIMECHO,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_UPROAR, MOVE_ATTRACT, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0xD,
                                            .nickname = _("CHIMECHO"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_WHISMUR,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_UPROAR, MOVE_ATTRACT, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0xD,
                                            .nickname = _("WHISMUR"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_YANMA,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_UPROAR, MOVE_ATTRACT, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x8A,
                                            .nickname = _("YANMA"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_ILLUMISE,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_ENCORE, MOVE_ATTRACT, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 6,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0xA,
                                            .nickname = _("ILLUMISE"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_SPHEAL,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_ENCORE, MOVE_ATTRACT, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0xA,
                                            .nickname = _("SPHEAL"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_VIGOROTH,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_ENCORE, MOVE_ATTRACT, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 6,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x87,
                                            .nickname = _("VIGOROTH"),
                                            .friendship = 255,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x39, 0x13, 0x14, 0x15, 0x38, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x91, 0x9B, 0x9C, 0x96, 0x40, 0x40, 0x96, 0x9B, 0x96, 0x40, 0x40, 0x96, 0x9B, 0x9C, 0x9B, 0x08, 
                    0x9C, 0x9B, 0x96, 0x40, 0xDB, 0xDB, 0x40, 0x96, 0x40, 0xDB, 0xDB, 0x42, 0x96, 0x9B, 0x9B, 0x08, 
                    0x91, 0x96, 0x40, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0x42, 0x96, 0x9C, 0x08, 
                    0x91, 0x42, 0xDB, 0xDB, 0xD6, 0xD6, 0xD6, 0xDB, 0xD6, 0xD6, 0xD6, 0xDB, 0xDB, 0x42, 0x9B, 0x08, 
                    0x96, 0x42, 0xDB, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xDB, 0x42, 0x96, 0x08, 
                    0x96, 0x42, 0xDB, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xDB, 0x42, 0x96, 0x08, 
                    0x96, 0x42, 0xDB, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xDB, 0x42, 0x96, 0x08, 
                    0x96, 0x42, 0xDB, 0xDB, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xDB, 0xDB, 0x42, 0x96, 0x08, 
                    0x91, 0x96, 0x42, 0xDB, 0xDB, 0xD6, 0xD6, 0xD6, 0xD6, 0xD6, 0xDB, 0xDB, 0x42, 0x96, 0x9B, 0x08, 
                    0x91, 0x9B, 0x96, 0x42, 0xDB, 0xDB, 0xD6, 0xD6, 0xD6, 0xDB, 0xDB, 0x42, 0x96, 0x9B, 0x9C, 0x08, 
                    0x9C, 0x9B, 0x9B, 0x96, 0x42, 0xDB, 0xDB, 0xD6, 0xDB, 0xDB, 0x42, 0x96, 0x9B, 0x9B, 0x9B, 0x08, 
                    0x96, 0x9B, 0x9B, 0x9B, 0x96, 0x41, 0xDB, 0xDB, 0xDB, 0x42, 0x96, 0x9B, 0x9B, 0x9C, 0x96, 0x08, 
                    0xD6, 0x96, 0x9C, 0x9B, 0x9B, 0x96, 0x41, 0xDB, 0x42, 0x96, 0x9B, 0x9B, 0x9B, 0x96, 0xD6, 0x08, 
                    0x9C, 0xD6, 0x96, 0x9B, 0x9C, 0x9B, 0x96, 0x40, 0x96, 0x9B, 0x9C, 0x9B, 0x96, 0xD6, 0x9C, 0x08, 
                    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
                },
                .collisionData = {0x0381, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0x1, 0xFFFF},
                .coords = {COORDS_XY(3,8), COORDS_XY(11,8)},
                .direction = TRAINER_DIRS(DIR_WEST, DIR_EAST),
                .range = TRAINER_RANGE(7, 7),
            }
        },
    [2] =
        {
            .trainerNum1 = 45,
            .trainerNum2 = 46,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("ENRIQUE"),
                            .facilityClass = FACILITY_CLASS_RICH_BOY,
                            .unused = 0,
                            .speechBefore = {EC_WORD_GET, EC_WORD_READY, EC_WORD_FOR, EC_WORD_AN, EC_WORD_AWESOME, EC_WORD_TIME},
                            .speechWin = {EC_WORD_HEY, EC_WORD_HEY, EC_WORD_EXCL, EC_WORD_WHAT_S_UP_QUES, EC_EMPTY_WORD, EC_EMPTY_WORD},
                            .speechLose = {EC_WORD_AWW, EC_WORD_COULDN_T, EC_WORD_YOU, EC_WORD_LET_ME_WIN, EC_WORD_QUES, EC_EMPTY_WORD},
                            .speechAfter = {EC_WORD_DID, EC_WORD_YOU, EC_WORD_SEE, EC_WORD_MY, EC_WORD_SKILL, EC_WORD_QUES},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_WOOPER,
                                            .heldItem = ITEM_FIGY_BERRY,
                                            .moves = {MOVE_RAIN_DANCE, MOVE_YAWN, MOVE_SURF, MOVE_HAZE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 6,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x8C,
                                            .nickname = _("WOOPER"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_POLIWAG,
                                            .heldItem = ITEM_WIKI_BERRY,
                                            .moves = {MOVE_SURF, MOVE_ICE_BEAM, MOVE_MIST, MOVE_HYPNOSIS},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 6,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0x3,
                                            .nickname = _("POLIWAG"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_PSYDUCK,
                                            .heldItem = ITEM_AGUAV_BERRY,
                                            .moves = {MOVE_HYPNOSIS, MOVE_SURF, MOVE_DISABLE, MOVE_SEISMIC_TOSS},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 250,
                                            .defenseEV = 130,
                                            .spDefenseEV = 130,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("PSYDUCK"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_RHYDON,
                                            .heldItem = ITEM_SOFT_SAND,
                                            .moves = {MOVE_EARTHQUAKE, MOVE_MAGNITUDE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .speedEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x96,
                                            .nickname = _("RHYDON"),
                                            .friendship = 0,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_RHYHORN,
                                            .heldItem = ITEM_SOFT_SAND,
                                            .moves = {MOVE_EARTHQUAKE, MOVE_MAGNITUDE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("RHYHORN"),
                                            .friendship = 0,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_CUBONE,
                                            .heldItem = ITEM_SOFT_SAND,
                                            .moves = {MOVE_EARTHQUAKE, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0x80,
                                            .nickname = _("CUBONE"),
                                            .friendship = 0,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("COLLEEN"),
                            .facilityClass = FACILITY_CLASS_LADY,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_WOULD, EC_WORD_LIKE, EC_WORD_TO, EC_WORD_ROCK, EC_WORD_OUT},
                            .speechWin = {EC_WORD_I, EC_WORD_LIKE, EC_WORD_THIS, EC_WORD_ROCK, EC_WORD_THING, EC_WORD_EXCL},
                            .speechLose = {EC_WORD_GO_AHEAD, EC_WORD_AND, EC_WORD_CRUSH, EC_WORD_MY, EC_WORD_DREAM, EC_WORD_ELLIPSIS},
                            .speechAfter = {EC_WORD_I, EC_WORD_WANT, EC_WORD_TO, EC_WORD_GO, EC_WORD_WILD, EC_WORD_SOMETIME},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_MAGNEMITE,
                                            .heldItem = ITEM_MAGNET,
                                            .moves = {MOVE_THUNDER, MOVE_ZAP_CANNON, MOVE_SPARK, MOVE_THUNDER_SHOCK},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x8C,
                                            .nickname = _("MAGNEMITE"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_ELECTABUZZ,
                                            .heldItem = ITEM_MAGNET,
                                            .moves = {MOVE_THUNDER, MOVE_THUNDERBOLT, MOVE_THUNDER_PUNCH, MOVE_SHOCK_WAVE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x41,
                                            .nickname = _("ELECTABUZZ"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_FLAAFFY,
                                            .heldItem = ITEM_MAGNET,
                                            .moves = {MOVE_THUNDER, MOVE_THUNDERBOLT, MOVE_SHOCK_WAVE, MOVE_THUNDER_SHOCK},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 250,
                                            .spAttackEV = 130,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("FLAAFFY"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_BALTOY,
                                            .heldItem = ITEM_SILK_SCARF,
                                            .moves = {MOVE_EXPLOSION, MOVE_SELF_DESTRUCT, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("BALTOY"),
                                            .friendship = 0,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_PINECO,
                                            .heldItem = ITEM_SILK_SCARF,
                                            .moves = {MOVE_EXPLOSION, MOVE_SELF_DESTRUCT, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .speedEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("PINECO"),
                                            .friendship = 0,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_VOLTORB,
                                            .heldItem = ITEM_SILK_SCARF,
                                            .moves = {MOVE_EXPLOSION, MOVE_SELF_DESTRUCT, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x80,
                                            .nickname = _("VOLTORB"),
                                            .friendship = 0,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x39, 0x1B, 0x1C, 0x1D, 0x38, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x91, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x84, 0x84, 0x84, 0x9A, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x08, 
                    0x17, 0xB2, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xB0, 0x17, 0xB2, 0xA1, 0x08, 
                    0x17, 0xBB, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0xBB, 0x17, 0x17, 0x17, 0x08, 
                    0x17, 0xB2, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xB0, 0x17, 0xB2, 0xA1, 0x08, 
                    0x17, 0x17, 0x17, 0xBB, 0x17, 0x17, 0x17, 0x17, 0x17, 0xBB, 0x17, 0x17, 0x17, 0x17, 0x17, 0x08, 
                    0x17, 0xB2, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xB0, 0x17, 0xB2, 0xA1, 0x08, 
                    0x17, 0x17, 0xBB, 0x17, 0x17, 0x17, 0x17, 0xBB, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x08, 
                    0x17, 0xB2, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xB0, 0x17, 0xB2, 0xA1, 0x08, 
                    0x17, 0x17, 0x17, 0x17, 0x17, 0xBB, 0x17, 0x17, 0x17, 0x17, 0xBB, 0x17, 0x17, 0x17, 0x17, 0x08, 
                    0x17, 0xB2, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xB0, 0x17, 0xB2, 0xA1, 0x08, 
                    0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0xBB, 0x17, 0x17, 0xBB, 0x17, 0x17, 0x08, 
                    0x17, 0xB2, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xB0, 0x17, 0xB2, 0xA1, 0x08, 
                    0x17, 0x17, 0x17, 0x17, 0xBB, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x17, 0x08, 
                    0x17, 0xB2, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xA1, 0xB0, 0x17, 0xB2, 0xA1, 0x08, 
                    0x17, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0x17, 0xBB, 0xBB, 0x08,
                },
                .collisionData = {0x0381, 0x0381, 0x7FF7, 0x1, 0x7FF7, 0x1, 0x7FF7, 0x1, 0x7FF7, 0x1, 0x7FF7, 0x1, 0x7FF7, 0x1, 0x7FF7, 0x1},
                .coords = {COORDS_XY(9,1), COORDS_XY(14,1)},
                .direction = TRAINER_DIRS(DIR_WEST, DIR_EAST),
                .range = TRAINER_RANGE(4, 4),
            }
        },
    [3] =
        {
            .trainerNum1 = 47,
            .trainerNum2 = 48,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("KIMBERLY"),
                            .facilityClass = FACILITY_CLASS_POKEFAN_F,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_LIKE, EC_WORD_WORKING, EC_WORD_AT, EC_WORD_MY, EC_WORD_OWN_TEMPO},
                            .speechWin = {EC_WORD_I_AM, EC_WORD_NOT, EC_WORD_BAD, EC_WORD_YOU, EC_WORD_THINK, EC_WORD_QUES},
                            .speechLose = {EC_WORD_YOU_RE, EC_WORD_A, EC_WORD_MEAN, EC_WORD_KID, EC_WORD_IT, EC_WORD_SEEMS},
                            .speechAfter = {EC_WORD_MY, EC_WORD_OWN_TEMPO, EC_WORD_IS, EC_WORD_RIGHT, EC_WORD_FOR, EC_WORD_ME},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_UNOWN,
                                            .heldItem = ITEM_MIRACLE_SEED,
                                            .moves = {MOVE_HIDDEN_POWER, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 30,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = 30,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = 30,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x202,
                                            .nickname = _("UNOWN"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_UNOWN,
                                            .heldItem = ITEM_MYSTIC_WATER,
                                            .moves = {MOVE_HIDDEN_POWER, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 30,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = 30,
                                            .spAttackIV = 30,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x10001,
                                            .nickname = _("UNOWN"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_UNOWN,
                                            .heldItem = ITEM_BLACK_BELT,
                                            .moves = {MOVE_HIDDEN_POWER, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 30,
                                            .attackIV = 30,
                                            .defenseIV = 30,
                                            .speedIV = 30,
                                            .spAttackIV = 30,
                                            .spDefenseIV = 30,
                                            .abilityNum = 0,
                                            .personality = 0x102,
                                            .nickname = _("UNOWN"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_SPINDA,
                                            .heldItem = ITEM_LUM_BERRY,
                                            .moves = {MOVE_TEETER_DANCE, MOVE_DIZZY_PUNCH, MOVE_CALM_MIND, MOVE_BATON_PASS},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 110,
                                            .defenseEV = 200,
                                            .spDefenseEV = 200,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x88FE980F,
                                            .nickname = _("SPINDA"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_PLUSLE,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_HELPING_HAND, MOVE_THUNDERBOLT, MOVE_AGILITY, MOVE_BATON_PASS},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("PLUSLE"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_VOLBEAT,
                                            .heldItem = ITEM_SITRUS_BERRY,
                                            .moves = {MOVE_HELPING_HAND, MOVE_SIGNAL_BEAM, MOVE_SOLAR_BEAM, MOVE_MOONLIGHT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0xF,
                                            .nickname = _("VOLBEAT"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("FRANCISCO"),
                            .facilityClass = FACILITY_CLASS_POKEFAN_M,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_STUDY, EC_WORD_EVERY, EC_WORD_DAY, EC_WORD_TOO, EC_EMPTY_WORD},
                            .speechWin = {EC_WORD_SO, EC_WORD_THAT_S, EC_WORD_HOW, EC_WORD_YOU, EC_WORD_LOSE, EC_EMPTY_WORD},
                            .speechLose = {EC_WORD_NO, EC_WORD_WONDER, EC_WORD_I, EC_WORD_COULDN_T, EC_WORD_BEAT, EC_WORD_YOU},
                            .speechAfter = {EC_WORD_YES, EC_WORD_ELLIPSIS, EC_EMPTY_WORD, EC_WORD_THANK_YOU, EC_EMPTY_WORD, EC_EMPTY_WORD},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_SPINDA,
                                            .heldItem = ITEM_LUM_BERRY,
                                            .moves = {MOVE_TEETER_DANCE, MOVE_DIZZY_PUNCH, MOVE_CALM_MIND, MOVE_BATON_PASS},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 110,
                                            .defenseEV = 200,
                                            .spDefenseEV = 200,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xE2880098,
                                            .nickname = _("SPINDA"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_MINUN,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_HELPING_HAND, MOVE_THUNDERBOLT, MOVE_AGILITY, MOVE_BATON_PASS},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x8C,
                                            .nickname = _("MINUN"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_ILLUMISE,
                                            .heldItem = ITEM_SITRUS_BERRY,
                                            .moves = {MOVE_HELPING_HAND, MOVE_WISH, MOVE_THUNDERBOLT, MOVE_MOONLIGHT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("ILLUMISE"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_UNOWN,
                                            .heldItem = ITEM_CHARCOAL,
                                            .moves = {MOVE_HIDDEN_POWER, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 30,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = 30,
                                            .speedIV = 30,
                                            .spAttackIV = 30,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x302,
                                            .nickname = _("UNOWN"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_UNOWN,
                                            .heldItem = ITEM_SOFT_SAND,
                                            .moves = {MOVE_HIDDEN_POWER, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .attackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = 30,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = 30,
                                            .spDefenseIV = 30,
                                            .abilityNum = 0,
                                            .personality = 0x203,
                                            .nickname = _("UNOWN"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_UNOWN,
                                            .heldItem = ITEM_TWISTED_SPOON,
                                            .moves = {MOVE_HIDDEN_POWER, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = 30,
                                            .speedIV = 30,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x301,
                                            .nickname = _("UNOWN"),
                                            .friendship = 255,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x39, 0x13, 0x14, 0x15, 0x38, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x69, 0x40, 0x6D, 0x41, 0x73, 0x41, 0x5E, 0x41, 0x71, 0x42, 0x45, 0x45, 0x45, 0x45, 0x45, 0x08, 
                    0x43, 0x73, 0x64, 0x73, 0x43, 0x73, 0x64, 0x73, 0x41, 0x73, 0x45, 0x45, 0x45, 0x45, 0x45, 0x08, 
                    0x65, 0x40, 0x73, 0x42, 0x6D, 0x42, 0x73, 0x42, 0x6D, 0x42, 0x45, 0x45, 0x45, 0x45, 0x45, 0x08, 
                    0x6C, 0x73, 0x41, 0x73, 0x64, 0x73, 0x40, 0x73, 0x64, 0x73, 0x40, 0x73, 0x42, 0x73, 0x41, 0x08, 
                    0x69, 0x40, 0x6D, 0x41, 0x73, 0x40, 0x6D, 0x43, 0x73, 0x43, 0x6D, 0x42, 0x73, 0x42, 0x6D, 0x08, 
                    0x40, 0x73, 0x64, 0x73, 0x43, 0x73, 0x64, 0x73, 0x41, 0x73, 0x64, 0x73, 0x41, 0x73, 0x64, 0x08, 
                    0x65, 0x42, 0x73, 0x42, 0x6D, 0x41, 0x73, 0x40, 0x6D, 0x42, 0x73, 0x41, 0x6D, 0x42, 0x73, 0x08, 
                    0x6C, 0x73, 0x41, 0x73, 0x64, 0x73, 0x42, 0x73, 0x64, 0x73, 0x42, 0x73, 0x64, 0x73, 0x42, 0x08, 
                    0x69, 0x40, 0x6D, 0x41, 0x73, 0x41, 0x6D, 0x42, 0x73, 0x43, 0x6D, 0x41, 0x73, 0x43, 0x6D, 0x08, 
                    0x40, 0x73, 0x64, 0x73, 0x43, 0x73, 0x64, 0x73, 0x43, 0x73, 0x64, 0x73, 0x42, 0x73, 0x64, 0x08, 
                    0x65, 0x42, 0x73, 0x42, 0x6D, 0x43, 0x73, 0x42, 0x6D, 0x42, 0x73, 0x41, 0x6D, 0x40, 0x73, 0x08, 
                    0x6C, 0x73, 0x40, 0x73, 0x64, 0x73, 0x41, 0x73, 0x64, 0x73, 0x42, 0x73, 0x64, 0x73, 0x42, 0x08, 
                    0x69, 0x40, 0x6D, 0x42, 0x73, 0x42, 0x6D, 0x43, 0x73, 0x40, 0x6D, 0x41, 0x73, 0x40, 0x6D, 0x08, 
                    0x40, 0x73, 0x64, 0x73, 0x40, 0x73, 0x64, 0x73, 0x40, 0x73, 0x64, 0x73, 0x43, 0x73, 0x64, 0x08, 
                    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
                },
                .collisionData = {0x0381, 0x1, 0x2201, 0x1, 0x8881, 0x1, 0x2223, 0x1, 0x8889, 0x1, 0x2223, 0x1, 0x8889, 0x1, 0x2223, 0xFFFF},
                .coords = {COORDS_XY(10,2), COORDS_XY(14,2)},
                .direction = TRAINER_DIRS(DIR_WEST, DIR_EAST),
                .range = TRAINER_RANGE(3, 3),
            }
        },
};

static const struct TrHillTag sDataTagUnique =
{
    .numTrainers = NUM_TRAINER_HILL_TRAINERS,
    .unused1 = 3,
    .numFloors = NUM_TRAINER_HILL_FLOORS,
    .checksum = 0x000652F3
};

static const struct TrHillFloor sDataTagUnique_Floors[] = {
    [0] =
        {
            .trainerNum1 = 49,
            .trainerNum2 = 50,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("MEREDITH"),
                            .facilityClass = FACILITY_CLASS_PKMN_RANGER_F,
                            .unused = 0,
                            .speechBefore = {EC_WORD_UM, EC_WORD_ELLIPSIS, EC_EMPTY_WORD, EC_WORD_I, EC_WORD_ELLIPSIS, EC_EMPTY_WORD},
                            .speechWin = {EC_WORD_OH, EC_WORD_EXCL, EC_EMPTY_WORD, EC_WORD_UM, EC_WORD_ELLIPSIS, EC_WORD_YAY},
                            .speechLose = {EC_WORD_UM, EC_WORD_ELLIPSIS, EC_EMPTY_WORD, EC_WORD_WAAAH, EC_WORD_ELLIPSIS, EC_EMPTY_WORD},
                            .speechAfter = {EC_WORD_UM, EC_WORD_ELLIPSIS, EC_EMPTY_WORD, EC_WORD_I_AM, EC_WORD_SAD, EC_WORD_ELLIPSIS},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_SUNFLORA,
                                            .heldItem = ITEM_PERSIM_BERRY,
                                            .moves = {MOVE_PETAL_DANCE, MOVE_GRASS_WHISTLE, MOVE_LIGHT_SCREEN, MOVE_SUNNY_DAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .defenseEV = 155,
                                            .spDefenseEV = 100,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("SUNFLORA"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_TANGELA,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_GIGA_DRAIN, MOVE_SLEEP_POWDER, MOVE_AMNESIA, MOVE_SUNNY_DAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spDefenseEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x91,
                                            .nickname = _("TANGELA"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_VENUSAUR,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_SOLAR_BEAM, MOVE_EARTHQUAKE, MOVE_SYNTHESIS, MOVE_SUNNY_DAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 100,
                                            .attackEV = 110,
                                            .defenseEV = 100,
                                            .spAttackEV = 100,
                                            .spDefenseEV = 100,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x1F,
                                            .nickname = _("VENUSAUR"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_LANTURN,
                                            .heldItem = ITEM_PERSIM_BERRY,
                                            .moves = {MOVE_SPARK, MOVE_WATER_PULSE, MOVE_CONFUSE_RAY, MOVE_RAIN_DANCE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("LANTURN"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_MANECTRIC,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_THUNDERBOLT, MOVE_HEADBUTT, MOVE_BITE, MOVE_RAIN_DANCE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0xF,
                                            .nickname = _("MANECTRIC"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_RAIKOU,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_THUNDER, MOVE_CRUNCH, MOVE_ROAR, MOVE_RAIN_DANCE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("RAIKOU"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("BERNARD"),
                            .facilityClass = FACILITY_CLASS_KINDLER,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_CAN_T, EC_WORD_LOSE, EC_WORD_THIS, EC_WORD_HOT, EC_WORD_BATTLE},
                            .speechWin = {EC_WORD_DID, EC_WORD_I, EC_WORD_COOL, EC_WORD_YOU, EC_WORD_DOWN, EC_WORD_QUES},
                            .speechLose = {EC_WORD_AIYEEH, EC_WORD_EXCL, EC_WORD_YOU_RE, EC_WORD_A, EC_WORD_TERRIBLE, EC_WORD_PERSON},
                            .speechAfter = {EC_WORD_YOUR, EC_WORD_HEART, EC_WORD_MUST_BE, EC_WORD_ICE, EC_WORD_COLD, EC_WORD_ELLIPSIS},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_RELICANTH,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_ANCIENT_POWER, MOVE_WATER_PULSE, MOVE_MUD_SPORT, MOVE_RAIN_DANCE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 155,
                                            .defenseEV = 100,
                                            .spDefenseEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x2F,
                                            .nickname = _("RELICANTH"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_GOLDUCK,
                                            .heldItem = ITEM_LAX_INCENSE,
                                            .moves = {MOVE_SURF, MOVE_PSYBEAM, MOVE_BRICK_BREAK, MOVE_RAIN_DANCE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("GOLDUCK"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_BLASTOISE,
                                            .heldItem = ITEM_SHELL_BELL,
                                            .moves = {MOVE_HYDRO_PUMP, MOVE_BITE, MOVE_MIRROR_COAT, MOVE_RAIN_DANCE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x28,
                                            .nickname = _("BLASTOISE"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_MAGCARGO,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_HEAT_WAVE, MOVE_ROCK_SLIDE, MOVE_PROTECT, MOVE_SUNNY_DAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spDefenseEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0x93,
                                            .nickname = _("MAGCARGO"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_RAPIDASH,
                                            .heldItem = ITEM_KINGS_ROCK,
                                            .moves = {MOVE_FIRE_BLAST, MOVE_BOUNCE, MOVE_QUICK_ATTACK, MOVE_SUNNY_DAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0xF,
                                            .nickname = _("RAPIDASH"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_MOLTRES,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_SKY_ATTACK, MOVE_AERIAL_ACE, MOVE_ROAR, MOVE_SUNNY_DAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("MOLTRES"),
                                            .friendship = 255,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0xF1, 0xF5, 0xFB, 0xF5, 0xE6, 0xE6, 0x1B, 0x14, 0x15, 0xF8, 0xF9, 0xFA, 0xFB, 0xFB, 0xFB, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xEC, 0xEC, 0xEC, 0xF9, 0xE6, 0xEE, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xFB, 0x9B, 0x9B, 0xFB, 0xEB, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEB, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0x9B, 0xDB, 0xDB, 0x9B, 0xEC, 0xFB, 0xF5, 0xF5, 0xF5, 0xFB, 0xEB, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xF5, 0x95, 0x95, 0xF5, 0xF5, 0xF5, 0xEB, 0xEC, 0xEB, 0xFB, 0xEB, 0x08, 
                    0xED, 0xEB, 0xFB, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0x08, 
                    0xF4, 0xEB, 0xFB, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xFB, 0xEB, 0xFB, 0xEC, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xFB, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xEB, 0xF5, 0xFB, 0x08, 
                    0xF1, 0xEC, 0xFB, 0xEB, 0xFB, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEB, 0xFB, 0x08, 
                    0xF1, 0xF5, 0xF5, 0xEB, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xFB, 0xEB, 0xFB, 0x08, 
                    0xF1, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEB, 0xFB, 0xEB, 0xFB, 0x08, 
                    0xF1, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xEB, 0xFB, 0xEB, 0xFB, 0x08, 
                    0xF1, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xFB, 0xEC, 0xFB, 0x08, 
                    0xF1, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0x08, 
                    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
                },
                .collisionData = {0x0381, 0x5E01, 0x50FF, 0x5083, 0x503B, 0x5FEB, 0xC02B, 0x5FEB, 0x5009, 0x57FD, 0x1005, 0x7FF5, 0x15, 0x7FF5, 0x1, 0xFFFF},
                .coords = {COORDS_XY(4,3), COORDS_XY(7,3)},
                .direction = TRAINER_DIRS(DIR_WEST, DIR_EAST),
                .range = TRAINER_RANGE(2, 2),
            }
        },
    [1] =
        {
            .trainerNum1 = 51,
            .trainerNum2 = 52,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("ABRAHAM"),
                            .facilityClass = FACILITY_CLASS_RUIN_MANIAC,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_LIKE, EC_WORD_THIS, EC_WORD_POKEMON, EC_WORD_THE, EC_WORD_BEST},
                            .speechWin = {EC_WORD_WHAT, EC_WORD_A, EC_WORD_GREAT, EC_WORD_POKEMON, EC_WORD_IT, EC_WORD_IS},
                            .speechLose = {EC_WORD_I, EC_WORD_CONFUSED, EC_WORD_A, EC_WORD_MOVE, EC_WORD_WITH, EC_WORD_ANOTHER},
                            .speechAfter = {EC_WORD_I, EC_WORD_WANT, EC_WORD_A, EC_MOVE(SKETCH), EC_WORD_OF, EC_WORD_YOU},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_SMEARGLE,
                                            .heldItem = ITEM_SCOPE_LENS,
                                            .moves = {MOVE_EARTHQUAKE, MOVE_SHADOW_BALL, MOVE_AERIAL_ACE, MOVE_IMPRISON},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 6,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x8A,
                                            .nickname = _("SMEARGLE"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_SMEARGLE,
                                            .heldItem = ITEM_CHESTO_BERRY,
                                            .moves = {MOVE_REST, MOVE_THUNDER_WAVE, MOVE_FLAMETHROWER, MOVE_IMPRISON},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .spAttackEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x87,
                                            .nickname = _("SMEARGLE"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_SMEARGLE,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_TEETER_DANCE, MOVE_LOCK_ON, MOVE_SHEER_COLD, MOVE_EXPLOSION},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 6,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xD,
                                            .nickname = _("SMEARGLE"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_SMEARGLE,
                                            .heldItem = ITEM_SCOPE_LENS,
                                            .moves = {MOVE_PSYCHIC, MOVE_SURF, MOVE_THUNDERBOLT, MOVE_IMPRISON},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .spAttackEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xA,
                                            .nickname = _("SMEARGLE"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_SMEARGLE,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_TOXIC, MOVE_PROTECT, MOVE_WILL_O_WISP, MOVE_IMPRISON},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 6,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xA,
                                            .nickname = _("SMEARGLE"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_SMEARGLE,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_TEETER_DANCE, MOVE_LOCK_ON, MOVE_SHEER_COLD, MOVE_DESTINY_BOND},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 6,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x8A,
                                            .nickname = _("SMEARGLE"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("LUC"),
                            .facilityClass = FACILITY_CLASS_TUBER_M,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_HAVE, EC_WORD_A, EC_WORD_COOL, EC_WORD_RARE, EC_WORD_MOVE},
                            .speechWin = {EC_WORD_MY, EC_WORD_RARE, EC_WORD_MOVE, EC_WORD_IS, EC_WORD_COOL, EC_WORD_ISN_T_IT_QUES},
                            .speechLose = {EC_WORD_DIDN_T, EC_WORD_YOU, EC_WORD_SEE, EC_WORD_MY, EC_WORD_MOVE, EC_WORD_QUES},
                            .speechAfter = {EC_WORD_I_AM, EC_WORD_SMART, EC_WORD_BUT, EC_WORD_ALSO, EC_WORD_WEAK, EC_WORD_ELLIPSIS},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_STARYU,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_CAMOUFLAGE, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .spAttackEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 1,
                                            .personality = 0xA,
                                            .nickname = _("STARYU"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_MEOWTH,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_PAY_DAY, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .spAttackEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0xD,
                                            .nickname = _("MEOWTH"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_BLAZIKEN,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_BLAZE_KICK, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x28,
                                            .nickname = _("BLAZIKEN"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_CUBONE,
                                            .heldItem = ITEM_THICK_CLUB,
                                            .moves = {MOVE_BONEMERANG, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 6,
                                            .spDefenseEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 1,
                                            .personality = 0x16,
                                            .nickname = _("CUBONE"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_BEEDRILL,
                                            .heldItem = ITEM_SHELL_BELL,
                                            .moves = {MOVE_TWINEEDLE, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 0,
                                            .personality = 0x8A,
                                            .nickname = _("BEEDRILL"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_RATICATE,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_SUPER_FANG, MOVE_NONE, MOVE_NONE, MOVE_NONE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 6,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 5,
                                            .attackIV = 5,
                                            .defenseIV = 5,
                                            .speedIV = 5,
                                            .spAttackIV = 5,
                                            .spDefenseIV = 5,
                                            .abilityNum = 1,
                                            .personality = 0xD,
                                            .nickname = _("RATICATE"),
                                            .friendship = 255,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x26, 0x1B, 0x1C, 0x1D, 0x25, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x2D, 0x3B, 0x3B, 0x3B, 0x35, 0x2C, 0x23, 0x24, 0x23, 0x2C, 0x35, 0x3B, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x94, 0x87, 0x9B, 0x87, 0x8C, 0x87, 0x8B, 0x87, 0x8C, 0x87, 0x8C, 0x87, 0x95, 0x87, 0x9B, 0x08, 
                    0x91, 0x8F, 0x9B, 0x8F, 0x9B, 0x8F, 0x8C, 0x8F, 0x9B, 0x8F, 0x9B, 0x8F, 0x8C, 0x8F, 0x95, 0x08, 
                    0x8D, 0x97, 0x95, 0x97, 0x9B, 0x97, 0x95, 0x97, 0x95, 0x97, 0x9B, 0x97, 0x9B, 0x97, 0x8C, 0x08, 
                    0x94, 0x87, 0x8C, 0x87, 0x9B, 0x87, 0x8C, 0x87, 0x8B, 0x87, 0x9B, 0x9B, 0x9B, 0x87, 0x9B, 0x08, 
                    0x91, 0x8F, 0x9B, 0x8F, 0x95, 0x8F, 0x9B, 0x8F, 0x8C, 0x8F, 0x95, 0x9B, 0x95, 0x8F, 0x9B, 0x08, 
                    0x91, 0x97, 0x95, 0x97, 0x8C, 0x97, 0x95, 0x97, 0x95, 0x97, 0x8C, 0x9B, 0x8C, 0x97, 0x95, 0x08, 
                    0x91, 0x87, 0x8C, 0x87, 0x9B, 0x87, 0x8C, 0x87, 0x8B, 0x87, 0x9B, 0x87, 0x95, 0x87, 0x8C, 0x08, 
                    0x8D, 0x8F, 0x9B, 0x8F, 0x95, 0x8F, 0x9B, 0x8F, 0x8C, 0x8F, 0x9B, 0x8F, 0x8C, 0x8F, 0x9B, 0x08, 
                    0x94, 0x97, 0x95, 0x97, 0x8C, 0x97, 0x9B, 0x97, 0x95, 0x97, 0x95, 0x97, 0x9B, 0x97, 0x9B, 0x08, 
                    0x91, 0x87, 0x8C, 0x87, 0x95, 0x87, 0x95, 0x87, 0x8B, 0x87, 0x8C, 0x87, 0x9B, 0x87, 0x9B, 0x08, 
                    0x91, 0x8F, 0x95, 0x8F, 0x8B, 0x8F, 0x8C, 0x8F, 0x8C, 0x8F, 0x9B, 0x8F, 0x95, 0x8F, 0x9B, 0x08, 
                    0x91, 0x97, 0x8C, 0x97, 0x8C, 0x97, 0x9B, 0x97, 0x9B, 0x97, 0x9B, 0x97, 0x8B, 0x97, 0x9B, 0x08, 
                    0x91, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x8C, 0x9B, 0x9B, 0x08, 
                    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
                },
                .collisionData = {0x0381, 0x7C1, 0x8AA1, 0x0209, 0x5557, 0xA281, 0x81, 0x5D6D, 0x2283, 0x89, 0xDD55, 0x20A1, 0xA81, 0x7D5D, 0x9, 0xFFFF},
                .coords = {COORDS_XY(9,6), COORDS_XY(13,6)},
                .direction = TRAINER_DIRS(DIR_WEST, DIR_EAST),
                .range = TRAINER_RANGE(3, 3),
            }
        },
    [2] =
        {
            .trainerNum1 = 53,
            .trainerNum2 = 54,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("BREYDEN"),
                            .facilityClass = FACILITY_CLASS_YOUNGSTER,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_CARE, EC_WORD_FOR, EC_WORD_MY, EC_WORD_POKEMON, EC_WORD_A_LOT},
                            .speechWin = {EC_WORD_MY, EC_WORD_POKEMON, EC_WORD_CAN, EC_WORD_DO, EC_WORD_IT, EC_WORD_ALL},
                            .speechLose = {EC_WORD_IT_S, EC_WORD_NOT, EC_WORD_LIKE, EC_WORD_WE, EC_WORD_ALWAYS, EC_WORD_WIN},
                            .speechAfter = {EC_WORD_I, EC_WORD_LIKE, EC_WORD_THE, EC_WORD_BEGINNING, EC_WORD_POKEMON, EC_WORD_BEST},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_CHARMELEON,
                                            .heldItem = ITEM_CHARCOAL,
                                            .moves = {MOVE_FIRE_SPIN, MOVE_DRAGON_RAGE, MOVE_FLAMETHROWER, MOVE_SLASH},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .spAttackEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 25,
                                            .attackIV = 25,
                                            .defenseIV = 25,
                                            .speedIV = 25,
                                            .spAttackIV = 25,
                                            .spDefenseIV = 25,
                                            .abilityNum = 0,
                                            .personality = 0x32,
                                            .nickname = _("CHARMELEON"),
                                            .friendship = 100,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_WARTORTLE,
                                            .heldItem = ITEM_MYSTIC_WATER,
                                            .moves = {MOVE_HYDRO_PUMP, MOVE_SKULL_BASH, MOVE_RAIN_DANCE, MOVE_PROTECT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 250,
                                            .defenseEV = 130,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 25,
                                            .attackIV = 25,
                                            .defenseIV = 25,
                                            .speedIV = 25,
                                            .spAttackIV = 25,
                                            .spDefenseIV = 25,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("WARTORTLE"),
                                            .friendship = 100,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_IVYSAUR,
                                            .heldItem = ITEM_MIRACLE_SEED,
                                            .moves = {MOVE_SOLAR_BEAM, MOVE_SYNTHESIS, MOVE_GROWTH, MOVE_SWEET_SCENT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 6,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 25,
                                            .attackIV = 25,
                                            .defenseIV = 25,
                                            .speedIV = 25,
                                            .spAttackIV = 25,
                                            .spDefenseIV = 25,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("IVYSAUR"),
                                            .friendship = 100,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_BAYLEEF,
                                            .heldItem = ITEM_MIRACLE_SEED,
                                            .moves = {MOVE_SOLAR_BEAM, MOVE_SAFEGUARD, MOVE_LIGHT_SCREEN, MOVE_BODY_SLAM},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 250,
                                            .attackEV = 130,
                                            .spAttackEV = 130,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 25,
                                            .attackIV = 25,
                                            .defenseIV = 25,
                                            .speedIV = 25,
                                            .spAttackIV = 25,
                                            .spDefenseIV = 25,
                                            .abilityNum = 0,
                                            .personality = 0x32,
                                            .nickname = _("BAYLEEF"),
                                            .friendship = 100,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_CROCONAW,
                                            .heldItem = ITEM_MYSTIC_WATER,
                                            .moves = {MOVE_SCARY_FACE, MOVE_SLASH, MOVE_HYDRO_PUMP, MOVE_SCREECH},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .speedEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 25,
                                            .attackIV = 25,
                                            .defenseIV = 25,
                                            .speedIV = 25,
                                            .spAttackIV = 25,
                                            .spDefenseIV = 25,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("CROCONAW"),
                                            .friendship = 100,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_QUILAVA,
                                            .heldItem = ITEM_CHARCOAL,
                                            .moves = {MOVE_QUICK_ATTACK, MOVE_FLAMETHROWER, MOVE_FLAME_WHEEL, MOVE_SWIFT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x28,
                                            .nickname = _("QUILAVA"),
                                            .friendship = 100,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("ANIYA"),
                            .facilityClass = FACILITY_CLASS_TUBER_F,
                            .unused = 0,
                            .speechBefore = {EC_WORD_SOMEONE, EC_WORD_GOT, EC_WORD_ME, EC_WORD_A_LITTLE, EC_WORD_POKEMON, EC_WORD_EGG},
                            .speechWin = {EC_WORD_I_AM, EC_WORD_NOT, EC_WORD_NEW, EC_WORD_AT, EC_WORD_THIS, EC_WORD_EXCL},
                            .speechLose = {EC_WORD_OH, EC_WORD_BUT, EC_WORD_ELLIPSIS, EC_WORD_WHY, EC_WORD_WHY, EC_WORD_QUES_EXCL},
                            .speechAfter = {EC_WORD_YOU_RE, EC_WORD_BUSY, EC_WORD_QUES, EC_WORD_DON_T, EC_WORD_GIVE_UP, EC_WORD_EXCL},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_SMOOCHUM,
                                            .heldItem = ITEM_PETAYA_BERRY,
                                            .moves = {MOVE_ICE_BEAM, MOVE_PSYCHIC, MOVE_SWEET_KISS, MOVE_FAKE_TEARS},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x32,
                                            .nickname = _("SMOOCHUM"),
                                            .friendship = 50,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_AZURILL,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_SURF, MOVE_SING, MOVE_RAIN_DANCE, MOVE_BLIZZARD},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0xC8,
                                            .nickname = _("AZURILL"),
                                            .friendship = 50,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_ELEKID,
                                            .heldItem = ITEM_KINGS_ROCK,
                                            .moves = {MOVE_FIRE_PUNCH, MOVE_THUNDER, MOVE_ICE_PUNCH, MOVE_THUNDER_WAVE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("ELEKID"),
                                            .friendship = 50,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_CLEFFA,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_MEGA_KICK, MOVE_SWEET_KISS, MOVE_SING, MOVE_METRONOME},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("CLEFFA"),
                                            .friendship = 50,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_WYNAUT,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_ENCORE, MOVE_COUNTER, MOVE_MIRROR_COAT, MOVE_DESTINY_BOND},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 250,
                                            .defenseEV = 130,
                                            .spDefenseEV = 130,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x84,
                                            .nickname = _("WYNAUT"),
                                            .friendship = 50,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_MAGBY,
                                            .heldItem = ITEM_SCOPE_LENS,
                                            .moves = {MOVE_FIRE_BLAST, MOVE_CONFUSE_RAY, MOVE_THUNDER_PUNCH, MOVE_BARRIER},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("MAGBY"),
                                            .friendship = 50,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x39, 0x13, 0x14, 0x15, 0x38, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x69, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x7C, 0x46, 0x08, 
                    0x69, 0x46, 0x7A, 0x73, 0x73, 0x73, 0x79, 0x73, 0x73, 0x73, 0x7D, 0x73, 0x73, 0x73, 0x46, 0x08, 
                    0x69, 0x46, 0x73, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x08, 
                    0x69, 0x46, 0x73, 0x73, 0x7B, 0x73, 0x7C, 0x73, 0x7B, 0x73, 0x7A, 0x73, 0x73, 0x73, 0x46, 0x08, 
                    0x69, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x7A, 0x46, 0x08, 
                    0x69, 0x46, 0x73, 0x73, 0x73, 0x73, 0x73, 0x7D, 0x7C, 0x73, 0x7C, 0x73, 0x7B, 0x73, 0x46, 0x08, 
                    0x69, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x7C, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x08, 
                    0x91, 0x46, 0x9B, 0x9B, 0x9B, 0x46, 0x3B, 0x3B, 0x3B, 0x46, 0x9B, 0x9B, 0x9B, 0x9B, 0x46, 0x08, 
                    0x91, 0x46, 0x9B, 0x9B, 0x9B, 0x46, 0x3B, 0x3B, 0x3B, 0x46, 0x9B, 0x9B, 0x9B, 0x9B, 0x46, 0x08, 
                    0x91, 0x46, 0x9B, 0x9B, 0x9B, 0x46, 0x3B, 0x3B, 0x3B, 0x46, 0x9B, 0x9B, 0x9B, 0x9B, 0x46, 0x08, 
                    0xF1, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x78, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x08, 
                    0xF1, 0x46, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0x7A, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0x7C, 0x08, 
                    0xF1, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0xFB, 0x08, 
                    0x7C, 0xFB, 0x7B, 0xFB, 0x7A, 0xFB, 0x79, 0xFB, 0xB3, 0xFB, 0x7D, 0xFB, 0x7E, 0xFB, 0x7D, 0x08, 
                    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
                },
                .collisionData = {0x0381, 0x7FFB, 0x4003, 0x5FFF, 0x4003, 0x7FFB, 0x4003, 0x7EFF, 0x4443, 0x4443, 0x4443, 0x7EFF, 0x4001, 0x7FFD, 0x1, 0xFFFF},
                .coords = {COORDS_XY(6,9), COORDS_XY(8,9)},
                .direction = TRAINER_DIRS(DIR_WEST, DIR_EAST),
                .range = TRAINER_RANGE(1, 1),
            }
        },
    [3] =
        {
            .trainerNum1 = 56,
            .trainerNum2 = 55,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("DANE"),
                            .facilityClass = FACILITY_CLASS_BIRD_KEEPER,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_GOT, EC_WORD_MY, EC_WORD_POKEMON, EC_WORD_ON, EC_WORD_VACATION},
                            .speechWin = {EC_WORD_IT_S, EC_WORD_GREAT, EC_WORD_TO, EC_WORD_TRAVEL, EC_WORD_AND, EC_WORD_BATTLE},
                            .speechLose = {EC_WORD_WAAAH, EC_WORD_EXCL, EC_WORD_MY, EC_WORD_RARE, EC_WORD_POKEMON, EC_WORD_EXCL},
                            .speechAfter = {EC_WORD_OFF, EC_WORD_ON, EC_WORD_ANOTHER, EC_WORD_VACATION, EC_WORD_I, EC_WORD_GO},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_SUDOWOODO,
                                            .heldItem = ITEM_SITRUS_BERRY,
                                            .moves = {MOVE_ROCK_SLIDE, MOVE_BLOCK, MOVE_TOXIC, MOVE_EXPLOSION},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 100,
                                            .attackEV = 255,
                                            .spDefenseEV = 155,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x80,
                                            .nickname = _("SUDOWOODO"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_SLOWKING,
                                            .heldItem = ITEM_SCOPE_LENS,
                                            .moves = {MOVE_SURF, MOVE_PSYCHIC, MOVE_BLIZZARD, MOVE_DISABLE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .defenseEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0x8C,
                                            .nickname = _("SLOWKING"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_ENTEI,
                                            .heldItem = ITEM_PETAYA_BERRY,
                                            .moves = {MOVE_FLAMETHROWER, MOVE_CALM_MIND, MOVE_FIRE_SPIN, MOVE_ROAR},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("ENTEI"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_HITMONCHAN,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_MEGA_PUNCH, MOVE_DETECT, MOVE_COUNTER, MOVE_SKY_UPPERCUT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("HITMONCHAN"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_MANTINE,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_SURF, MOVE_CONFUSE_RAY, MOVE_ATTRACT, MOVE_AERIAL_ACE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .defenseEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0x6,
                                            .nickname = _("MANTINE"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_ZAPDOS,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_THUNDERBOLT, MOVE_DRILL_PECK, MOVE_THUNDER_WAVE, MOVE_AGILITY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x18,
                                            .nickname = _("ZAPDOS"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("STEPHANIE"),
                            .facilityClass = FACILITY_CLASS_SWIMMING_TRIATHLETE_F,
                            .unused = 0,
                            .speechBefore = {EC_WORD_THIS, EC_WORD_IS, EC_WORD_HOW, EC_WORD_YOU, EC_WORD_WIN, EC_WORD_EXCL},
                            .speechWin = {EC_WORD_DO, EC_WORD_YOU, EC_WORD_UNDERSTAND, EC_WORD_HOW, EC_WORD_NOW, EC_WORD_QUES},
                            .speechLose = {EC_WORD_YES, EC_WORD_THAT_S, EC_WORD_HOW, EC_WORD_YOU, EC_WORD_DO, EC_WORD_IT},
                            .speechAfter = {EC_WORD_YOU, EC_WORD_BEAT, EC_WORD_ME, EC_WORD_GOOD, EC_WORD_FOR, EC_WORD_YOU},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_HITMONLEE,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_MEGA_KICK, MOVE_MIND_READER, MOVE_FOCUS_ENERGY, MOVE_HIGH_JUMP_KICK},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .attackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("HITMONLEE"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_PORYGON2,
                                            .heldItem = ITEM_SCOPE_LENS,
                                            .moves = {MOVE_LOCK_ON, MOVE_BLIZZARD, MOVE_CONVERSION_2, MOVE_PSYCHIC},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("PORYGON2"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_SUICUNE,
                                            .heldItem = ITEM_PETAYA_BERRY,
                                            .moves = {MOVE_SURF, MOVE_CALM_MIND, MOVE_MIRROR_COAT, MOVE_MIST},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("SUICUNE"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_HOUNDOOM,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_FLAMETHROWER, MOVE_CRUNCH, MOVE_ROAR, MOVE_WILL_O_WISP},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 1,
                                            .personality = 0xF,
                                            .nickname = _("HOUNDOOM"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_STANTLER,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_CONFUSE_RAY, MOVE_SWAGGER, MOVE_PSYCH_UP, MOVE_TAKE_DOWN},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("STANTLER"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_ARTICUNO,
                                            .heldItem = ITEM_NEVER_MELT_ICE,
                                            .moves = {MOVE_BLIZZARD, MOVE_SHEER_COLD, MOVE_MIST, MOVE_AERIAL_ACE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = 15,
                                            .attackIV = 15,
                                            .defenseIV = 15,
                                            .speedIV = 15,
                                            .spAttackIV = 15,
                                            .spDefenseIV = 15,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("ARTICUNO"),
                                            .friendship = 255,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0xF1, 0xFB, 0xFB, 0xFB, 0xF9, 0xF9, 0x1B, 0x1C, 0x1D, 0xE5, 0xE6, 0xEE, 0xF5, 0xFB, 0xFB, 0x08, 
                    0xED, 0xF5, 0xF5, 0xF5, 0xFB, 0xFB, 0xEC, 0xEC, 0xEC, 0xEB, 0xEC, 0xEC, 0xEC, 0xFB, 0xFB, 0x08, 
                    0xF4, 0xEC, 0xEC, 0xEC, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xEB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0x08, 
                    0xF1, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xF5, 0xFB, 0xEB, 0xFB, 0xF5, 0xF5, 0xF5, 0xF5, 0x08, 
                    0xF1, 0xEB, 0xEC, 0xEC, 0xEC, 0xEB, 0xEC, 0xEC, 0xFB, 0xEC, 0xFB, 0xEC, 0xEC, 0xEC, 0xEC, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xFB, 0xFB, 0xEB, 0xFB, 0xF5, 0xFB, 0xF5, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xF5, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xF5, 0xF5, 0xF5, 0xFB, 0xFB, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xEC, 0xEC, 0xEC, 0xFB, 0xFB, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xF5, 0xF5, 0xF5, 0xF5, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEC, 0xEC, 0xEC, 0xEC, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0x08, 
                    0xF1, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xFB, 0xEB, 0xF5, 0xF5, 0xF5, 0xF5, 0xFB, 0x08, 
                    0xF1, 0xEC, 0xFB, 0xEB, 0xFB, 0xEC, 0xFB, 0xEB, 0xFB, 0xEC, 0xEC, 0xEC, 0xEC, 0xEC, 0xFB, 0x08, 
                    0xF1, 0xFB, 0xFB, 0xEB, 0xFB, 0xFB, 0xFB, 0xEB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0x08, 
                    0xF1, 0xFB, 0xFB, 0xEC, 0xFB, 0xFB, 0xFB, 0xEC, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0xFB, 0x08, 
                    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
                },
                .collisionData = {0x0381, 0x3F9, 0xF041, 0x41, 0x7F5F, 0x4401, 0x4541, 0x5579, 0x5541, 0x555F, 0x5541, 0x5541, 0x557D, 0x1101, 0x1101, 0xFFFF},
                .coords = {COORDS_XY(8,2), COORDS_XY(11,5)},
                .direction = TRAINER_DIRS(DIR_WEST, DIR_NORTH),
                .range = TRAINER_RANGE(3, 3),
            }
        },
};

static const struct TrHillTag sDataTagExpert =
{
    .numTrainers = NUM_TRAINER_HILL_TRAINERS,
    .unused1 = 1,
    .numFloors = NUM_TRAINER_HILL_FLOORS,
    .checksum = 0x00061F3F
};

static const struct TrHillFloor sDataTagExpert_Floors[] = {
    [0] =
        {
            .trainerNum1 = 57,
            .trainerNum2 = 58,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("ALFRED"),
                            .facilityClass = FACILITY_CLASS_COOLTRAINER_M,
                            .unused = 0,
                            .speechBefore = {EC_WORD_WE, EC_WORD_CAN, EC_WORD_TAKE, EC_WORD_ON, EC_WORD_ANY, EC_WORD_TYPE},
                            .speechWin = {EC_WORD_WE, EC_WORD_DID, EC_WORD_BETTER, EC_WORD_THAN, EC_WORD_OKAY, EC_WORD_EXCL},
                            .speechLose = {EC_WORD_OUR, EC_WORD_STRATEGY, EC_WORD_ISN_T, EC_WORD_ANY, EC_WORD_GOOD, EC_WORD_ELLIPSIS},
                            .speechAfter = {EC_WORD_WE, EC_WORD_NEED, EC_WORD_TO, EC_WORD_TRAIN, EC_WORD_A_LOT, EC_WORD_MORE},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_SNORLAX,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_MEGA_KICK, MOVE_SHADOW_BALL, MOVE_BRICK_BREAK, MOVE_EARTHQUAKE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .defenseEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x35,
                                            .nickname = _("SNORLAX"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_MILTANK,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_DOUBLE_EDGE, MOVE_SHADOW_BALL, MOVE_ATTRACT, MOVE_MILK_DRINK},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("MILTANK"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_URSARING,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_DOUBLE_EDGE, MOVE_CRUNCH, MOVE_BRICK_BREAK, MOVE_AERIAL_ACE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .spAttackEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x7F,
                                            .nickname = _("URSARING"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_SLAKING,
                                            .heldItem = ITEM_CHESTO_BERRY,
                                            .moves = {MOVE_HYPER_BEAM, MOVE_SHADOW_BALL, MOVE_BRICK_BREAK, MOVE_REST},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .spDefenseEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x80,
                                            .nickname = _("SLAKING"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_KANGASKHAN,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_MEGA_KICK, MOVE_SHADOW_BALL, MOVE_ATTRACT, MOVE_FAKE_OUT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("KANGASKHAN"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_ZANGOOSE,
                                            .heldItem = ITEM_SCOPE_LENS,
                                            .moves = {MOVE_CRUSH_CLAW, MOVE_SHADOW_BALL, MOVE_BRICK_BREAK, MOVE_ROAR},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x80,
                                            .nickname = _("ZANGOOSE"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("EDIE"),
                            .facilityClass = FACILITY_CLASS_PSYCHIC_F,
                            .unused = 0,
                            .speechBefore = {EC_WORD_WE, EC_WORD_WILL_BE_HERE, EC_WORD_FOR, EC_WORD_ANY, EC_WORD_TRAINER, EC_WORD_CHALLENGE},
                            .speechWin = {EC_WORD_MY, EC_WORD_STRATEGY, EC_WORD_WORKS, EC_WORD_TO, EC_WORD_PERFECTION, EC_WORD_EXCL},
                            .speechLose = {EC_WORD_NO, EC_WORD_NO, EC_WORD_ELLIPSIS, EC_WORD_THAT, EC_WORD_WON_T, EC_WORD_DO},
                            .speechAfter = {EC_WORD_THERE, EC_WORD_SHOULD, EC_WORD_BE, EC_WORD_A, EC_WORD_TRAINER, EC_WORD_SCHOOL},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_SLOWKING,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_PSYCHIC, MOVE_SURF, MOVE_ICE_BEAM, MOVE_SKILL_SWAP},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 200,
                                            .defenseEV = 110,
                                            .spAttackEV = 200,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 1,
                                            .personality = 0xF,
                                            .nickname = _("SLOWKING"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_ESPEON,
                                            .heldItem = ITEM_LUM_BERRY,
                                            .moves = {MOVE_PSYCHIC, MOVE_BITE, MOVE_CALM_MIND, MOVE_REFLECT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x28,
                                            .nickname = _("ESPEON"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_STARMIE,
                                            .heldItem = ITEM_SHELL_BELL,
                                            .moves = {MOVE_PSYCHIC, MOVE_SURF, MOVE_THUNDERBOLT, MOVE_ICE_BEAM},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 1,
                                            .personality = 0xF,
                                            .nickname = _("STARMIE"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_GENGAR,
                                            .heldItem = ITEM_LUM_BERRY,
                                            .moves = {MOVE_PSYCHIC, MOVE_THUNDERBOLT, MOVE_FIRE_PUNCH, MOVE_ICE_PUNCH},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("GENGAR"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_GARDEVOIR,
                                            .heldItem = ITEM_SALAC_BERRY,
                                            .moves = {MOVE_PSYCHIC, MOVE_THUNDERBOLT, MOVE_MAGICAL_LEAF, MOVE_DESTINY_BOND},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("GARDEVOIR"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_ALAKAZAM,
                                            .heldItem = ITEM_LUM_BERRY,
                                            .moves = {MOVE_PSYCHIC, MOVE_RECOVER, MOVE_THUNDER_WAVE, MOVE_ATTRACT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("ALAKAZAM"),
                                            .friendship = 255,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x39, 0x1B, 0x14, 0x15, 0x38, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x31, 0x3B, 0x3B, 0x3B, 0x3B, 0x3B, 0x2C, 0x39, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x31, 0x3B, 0x08, 0x08, 0x08, 0x08, 0x08, 0x3B, 0x08, 0x08, 0x08, 0x08, 0x08, 0x3B, 0x3B, 0x08, 
                    0x69, 0x73, 0x08, 0x4D, 0x4D, 0x4D, 0x4D, 0xD1, 0x4D, 0x4D, 0x4D, 0x4D, 0x08, 0x69, 0x73, 0x08, 
                    0x40, 0x3B, 0x08, 0x55, 0x55, 0x55, 0x55, 0xD1, 0x55, 0x55, 0x55, 0x55, 0x08, 0x31, 0x41, 0x08, 
                    0x69, 0x41, 0x08, 0xC5, 0xD9, 0xD9, 0xD9, 0x9A, 0xD9, 0xD9, 0xD9, 0xC6, 0x08, 0x41, 0x73, 0x08, 
                    0x69, 0x3B, 0x08, 0xD4, 0xDB, 0x9B, 0x73, 0x73, 0x73, 0x9B, 0xDB, 0xCC, 0x08, 0x31, 0x73, 0x08, 
                    0x69, 0x3B, 0x08, 0xCD, 0x9B, 0x73, 0x73, 0x44, 0x73, 0x73, 0x9B, 0xD5, 0x08, 0x31, 0x73, 0x08, 
                    0x69, 0x3B, 0x08, 0xD4, 0xDB, 0x9B, 0x73, 0x73, 0x73, 0x9B, 0xDB, 0xCC, 0x08, 0x31, 0x73, 0x08, 
                    0x69, 0x41, 0x08, 0xD1, 0xDB, 0xDB, 0xDB, 0x9B, 0xDB, 0xDB, 0xDB, 0xDB, 0x08, 0x41, 0x73, 0x08, 
                    0x40, 0x3B, 0x08, 0x08, 0xC7, 0xC7, 0xDB, 0xDB, 0xDB, 0xC7, 0xC7, 0x08, 0x08, 0x31, 0x41, 0x08, 
                    0x69, 0x3B, 0x4D, 0x4D, 0x67, 0x67, 0xDB, 0xDB, 0xDB, 0x67, 0x67, 0x4D, 0x4D, 0x31, 0x73, 0x08, 
                    0x69, 0x3B, 0x55, 0x55, 0xD7, 0xD7, 0xD1, 0xDB, 0xDB, 0xD7, 0xD7, 0x55, 0x55, 0x31, 0x73, 0x08, 
                    0x69, 0x3B, 0x39, 0x39, 0x39, 0x39, 0x3A, 0x3B, 0x3B, 0x39, 0x39, 0x39, 0x39, 0x3A, 0x73, 0x08, 
                    0x69, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x73, 0x08, 
                    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
                },
                .collisionData = {0x0381, 0x0201, 0x3EF9, 0x3EF9, 0x3EF9, 0x2009, 0x3019, 0x2009, 0x3019, 0x2009, 0x3019, 0x3019, 0x3C79, 0x1, 0x1, 0xFFFF},
                .coords = {COORDS_XY(4,7), COORDS_XY(10,7)},
                .direction = TRAINER_DIRS(DIR_WEST, DIR_EAST),
                .range = TRAINER_RANGE(5, 5),
            }
        },
    [1] =
        {
            .trainerNum1 = 59,
            .trainerNum2 = 60,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("RODERICK"),
                            .facilityClass = FACILITY_CLASS_COOLTRAINER_M,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_CAN, EC_WORD_WIN, EC_WORD_WITHOUT, EC_WORD_A, EC_MOVE2(HELPING_HAND)},
                            .speechWin = {EC_WORD_IT_S, EC_WORD_ONLY, EC_WORD_NATURAL, EC_WORD_THAT, EC_WORD_I, EC_WORD_WIN},
                            .speechLose = {EC_WORD_LOSING, EC_WORD_HAS, EC_WORD_ME, EC_WORD_FEELING, EC_WORD_SMALL, EC_WORD_ELLIPSIS},
                            .speechAfter = {EC_WORD_YOU, EC_WORD_HAVE, EC_WORD_A, EC_WORD_GOOD, EC_WORD_TEACHER, EC_WORD_QUES},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_SWELLOW,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_AERIAL_ACE, MOVE_AGILITY, MOVE_FACADE, MOVE_ATTRACT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("SWELLOW"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_MACHAMP,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_LOW_KICK, MOVE_ROCK_SLIDE, MOVE_FACADE, MOVE_ATTRACT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("MACHAMP"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_URSARING,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_PROTECT, MOVE_ROCK_SLIDE, MOVE_FACADE, MOVE_ATTRACT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("URSARING"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_KINGLER,
                                            .heldItem = ITEM_PERSIM_BERRY,
                                            .moves = {MOVE_RETURN, MOVE_PROTECT, MOVE_CRABHAMMER, MOVE_ATTRACT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x80,
                                            .nickname = _("KINGLER"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_TYRANITAR,
                                            .heldItem = ITEM_PERSIM_BERRY,
                                            .moves = {MOVE_ROCK_SLIDE, MOVE_CRUNCH, MOVE_EARTHQUAKE, MOVE_ATTRACT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .defenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x80,
                                            .nickname = _("TYRANITAR"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_DRAGONITE,
                                            .heldItem = ITEM_PERSIM_BERRY,
                                            .moves = {MOVE_BODY_SLAM, MOVE_THUNDER_WAVE, MOVE_EARTHQUAKE, MOVE_ATTRACT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .attackEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x80,
                                            .nickname = _("DRAGONITE"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("ALICIA"),
                            .facilityClass = FACILITY_CLASS_COOLTRAINER_F,
                            .unused = 0,
                            .speechBefore = {EC_WORD_I, EC_WORD_DON_T, EC_WORD_NEED, EC_WORD_A, EC_MOVE2(HELPING_HAND), EC_WORD_EXCL},
                            .speechWin = {EC_WORD_SORRY, EC_WORD_EXCL, EC_WORD_TOO, EC_WORD_BAD, EC_WORD_FOR, EC_WORD_YOU},
                            .speechLose = {EC_WORD_WHAT, EC_WORD_AN, EC_WORD_UNBELIEVABLE, EC_MOVE2(STRUGGLE), EC_WORD_THAT_WAS, EC_WORD_EXCL},
                            .speechAfter = {EC_WORD_I, EC_WORD_CAN, EC_WORD_BE, EC_WORD_YOUR, EC_WORD_POKEMON, EC_WORD_TEACHER},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_JOLTEON,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_THUNDERBOLT, MOVE_THUNDER_WAVE, MOVE_ATTRACT, MOVE_SWAGGER},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0xA,
                                            .nickname = _("JOLTEON"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_ALAKAZAM,
                                            .heldItem = ITEM_KINGS_ROCK,
                                            .moves = {MOVE_PSYCHIC, MOVE_ICE_PUNCH, MOVE_ATTRACT, MOVE_SWAGGER},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0xA,
                                            .nickname = _("ALAKAZAM"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_STARMIE,
                                            .heldItem = ITEM_SCOPE_LENS,
                                            .moves = {MOVE_SURF, MOVE_PSYCHIC, MOVE_CONFUSE_RAY, MOVE_SWAGGER},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 1,
                                            .personality = 0xA,
                                            .nickname = _("STARMIE"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_DUSCLOPS,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_PURSUIT, MOVE_PROTECT, MOVE_ATTRACT, MOVE_WILL_O_WISP},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x82,
                                            .nickname = _("DUSCLOPS"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_NINETALES,
                                            .heldItem = ITEM_WHITE_HERB,
                                            .moves = {MOVE_OVERHEAT, MOVE_CONFUSE_RAY, MOVE_WILL_O_WISP, MOVE_ATTRACT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .speedEV = 252,
                                            .spAttackEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0xD2,
                                            .nickname = _("NINETALES"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_BANETTE,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_SHADOW_BALL, MOVE_FEINT_ATTACK, MOVE_ATTRACT, MOVE_WILL_O_WISP},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x85,
                                            .nickname = _("BANETTE"),
                                            .friendship = 255,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x39, 0x13, 0x14, 0x15, 0x38, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x91, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x78, 0x08, 
                    0x91, 0x46, 0x9B, 0x7C, 0x9B, 0x9B, 0x9B, 0x7D, 0x9B, 0x7C, 0x9B, 0x7B, 0x9B, 0x7A, 0x9B, 0x08, 
                    0x91, 0x46, 0x7D, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x08, 
                    0x91, 0x46, 0x9B, 0x7E, 0x9B, 0x9B, 0x9B, 0x7D, 0x9B, 0x7E, 0x9B, 0xB3, 0x9B, 0x9B, 0x9B, 0x08, 
                    0x91, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x9B, 0x08, 
                    0x91, 0x46, 0x9B, 0x7D, 0x9B, 0x7C, 0x9B, 0x7B, 0x9B, 0x7A, 0x9B, 0x7C, 0x9B, 0x9B, 0x9B, 0x08, 
                    0x91, 0x46, 0x9B, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x08, 
                    0x91, 0x46, 0x9B, 0x7C, 0x9B, 0x7D, 0x9B, 0x7E, 0x9B, 0x9B, 0x9B, 0x7D, 0x9B, 0x7E, 0x9B, 0x08, 
                    0x91, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0x46, 0xB3, 0x08, 
                    0x91, 0xD6, 0x96, 0x9B, 0x9B, 0x96, 0xD6, 0xDB, 0xD6, 0x96, 0x9B, 0x9B, 0x96, 0xD6, 0x9B, 0x08, 
                    0xD6, 0x96, 0x9B, 0x9B, 0x96, 0xD6, 0xDB, 0xDB, 0xDB, 0xD6, 0x96, 0x9B, 0x9B, 0x96, 0xD6, 0x08, 
                    0x96, 0x9B, 0x9B, 0x96, 0xD6, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0xD6, 0x96, 0x9B, 0x9B, 0x96, 0x08, 
                    0xD6, 0x96, 0x9B, 0x9B, 0x96, 0xD6, 0xDB, 0xDB, 0xDB, 0xD6, 0x96, 0x9B, 0x9B, 0x96, 0xD6, 0x08, 
                    0x91, 0xD6, 0x96, 0x9B, 0x9B, 0x96, 0xD6, 0xDB, 0xD6, 0x96, 0x9B, 0x9B, 0x96, 0xD6, 0x9B, 0x08, 
                    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,
                },
                .collisionData = {0x0381, 0x7FFD, 0x4001, 0x5FFF, 0x4001, 0x7FFD, 0x4001, 0x5FFF, 0x4001, 0x7FFD, 0x1, 0x1, 0x1, 0x1, 0x1, 0xFFFF},
                .coords = {COORDS_XY(7,10), COORDS_XY(7,14)},
                .direction = TRAINER_DIRS(DIR_SOUTH, DIR_NORTH),
                .range = TRAINER_RANGE(3, 3),
            }
        },
    [2] =
        {
            .trainerNum1 = 61,
            .trainerNum2 = 62,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("TERRENCE"),
                            .facilityClass = FACILITY_CLASS_EXPERT_M,
                            .unused = 0,
                            .speechBefore = {EC_WORD_OH_DEAR, EC_WORD_THIS, EC_WORD_MATCH, EC_WORD_IS, EC_WORD_FOR, EC_WORD_YOU},
                            .speechWin = {EC_WORD_OH_DEAR, EC_WORD_DID, EC_WORD_YOU, EC_WORD_SEE, EC_WORD_THAT, EC_WORD_QUES},
                            .speechLose = {EC_WORD_OH_DEAR, EC_WORD_I_AM, EC_WORD_SO, EC_WORD_SORRY, EC_EMPTY_WORD, EC_EMPTY_WORD},
                            .speechAfter = {EC_WORD_IS, EC_WORD_YOUR, EC_WORD_GRANDMOTHER, EC_WORD_STRONG, EC_WORD_QUES, EC_EMPTY_WORD},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_WOBBUFFET,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_MIRROR_COAT, MOVE_COUNTER, MOVE_SAFEGUARD, MOVE_ENCORE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 250,
                                            .defenseEV = 130,
                                            .spDefenseEV = 130,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x94,
                                            .nickname = _("WOBBUFFET"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_EXPLOUD,
                                            .heldItem = ITEM_CHESTO_BERRY,
                                            .moves = {MOVE_HYPER_VOICE, MOVE_COUNTER, MOVE_REST, MOVE_ROCK_SLIDE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 250,
                                            .defenseEV = 130,
                                            .spDefenseEV = 130,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x85,
                                            .nickname = _("EXPLOUD"),
                                            .friendship = 0,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_CROBAT,
                                            .heldItem = ITEM_KINGS_ROCK,
                                            .moves = {MOVE_MEAN_LOOK, MOVE_CONFUSE_RAY, MOVE_AERIAL_ACE, MOVE_TOXIC},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 250,
                                            .defenseEV = 130,
                                            .spDefenseEV = 130,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("CROBAT"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_DUGTRIO,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_DOUBLE_TEAM, MOVE_PROTECT, MOVE_RETURN, MOVE_SLUDGE_BOMB},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 6,
                                            .attackEV = 252,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 1,
                                            .personality = 0xD,
                                            .nickname = _("DUGTRIO"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_ELECTRODE,
                                            .heldItem = ITEM_PETAYA_BERRY,
                                            .moves = {MOVE_TORMENT, MOVE_MIRROR_COAT, MOVE_THUNDERBOLT, MOVE_LIGHT_SCREEN},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 252,
                                            .spAttackEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x5,
                                            .nickname = _("ELECTRODE"),
                                            .friendship = 0,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_GENGAR,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_CONFUSE_RAY, MOVE_MEAN_LOOK, MOVE_GIGA_DRAIN, MOVE_WILL_O_WISP},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .spDefenseEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x14,
                                            .nickname = _("GENGAR"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("CARLOTTA"),
                            .facilityClass = FACILITY_CLASS_EXPERT_F,
                            .unused = 0,
                            .speechBefore = {EC_WORD_OH_DEAR, EC_WORD_I, EC_WORD_WILL, EC_WORD_WIN, EC_WORD_FOR, EC_WORD_YOU},
                            .speechWin = {EC_WORD_OH_DEAR, EC_WORD_WE, EC_WORD_HAVE, EC_WORD_A, EC_WORD_PARTY, EC_WORD_LATER},
                            .speechLose = {EC_WORD_OH_DEAR, EC_WORD_I_AM, EC_WORD_SORRY, EC_WORD_TO, EC_WORD_DISAPPOINT, EC_WORD_YOU},
                            .speechAfter = {EC_WORD_IS, EC_WORD_YOUR, EC_WORD_GRANDFATHER, EC_WORD_COOL, EC_WORD_QUES, EC_EMPTY_WORD},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_LAPRAS,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_SURF, MOVE_ICE_BEAM, MOVE_PERISH_SONG, MOVE_SING},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 250,
                                            .defenseEV = 130,
                                            .spDefenseEV = 130,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x82,
                                            .nickname = _("LAPRAS"),
                                            .friendship = 0,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_ABSOL,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_PERISH_SONG, MOVE_DOUBLE_EDGE, MOVE_PROTECT, MOVE_TORMENT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .defenseEV = 6,
                                            .speedEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x8A,
                                            .nickname = _("ABSOL"),
                                            .friendship = 0,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_ALTARIA,
                                            .heldItem = ITEM_KINGS_ROCK,
                                            .moves = {MOVE_PERISH_SONG, MOVE_PROTECT, MOVE_DRAGON_CLAW, MOVE_FIRE_BLAST},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .spAttackEV = 6,
                                            .spDefenseEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x0,
                                            .nickname = _("ALTARIA"),
                                            .friendship = 0,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_DEWGONG,
                                            .heldItem = ITEM_CHESTO_BERRY,
                                            .moves = {MOVE_ICE_BEAM, MOVE_SIGNAL_BEAM, MOVE_REST, MOVE_PERISH_SONG},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 252,
                                            .spAttackEV = 6,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x14,
                                            .nickname = _("DEWGONG"),
                                            .friendship = 0,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_POLITOED,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_HYDRO_PUMP, MOVE_BLIZZARD, MOVE_MIND_READER, MOVE_PERISH_SONG},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .spAttackEV = 6,
                                            .spDefenseEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 1,
                                            .personality = 0x14,
                                            .nickname = _("POLITOED"),
                                            .friendship = 0,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_MAROWAK,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_PERISH_SONG, MOVE_EARTHQUAKE, MOVE_COUNTER, MOVE_PROTECT},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 252,
                                            .speedEV = 6,
                                            .spDefenseEV = 252,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 1,
                                            .personality = 0x17,
                                            .nickname = _("MAROWAK"),
                                            .friendship = 0,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0xD1, 0xDB, 0xDB, 0xDB, 0xD9, 0xD9, 0x1B, 0x14, 0x15, 0x98, 0x99, 0x9A, 0x9B, 0x9B, 0x9B, 0x08, 
                    0xD1, 0xDB, 0xDB, 0xDB, 0xD5, 0xD5, 0xC3, 0xF9, 0x86, 0x8E, 0x95, 0x9B, 0x9B, 0x9B, 0x9B, 0x08, 
                    0xD1, 0xDB, 0xDB, 0xD5, 0xCB, 0xCB, 0xCB, 0xFB, 0x8B, 0x8B, 0x8B, 0x95, 0x9B, 0x9B, 0x9B, 0x08, 
                    0xD1, 0xDB, 0xD5, 0xCB, 0xCB, 0xCB, 0xCB, 0xFB, 0x8B, 0x8B, 0x8B, 0x8B, 0x95, 0x9B, 0x9B, 0x08, 
                    0xD1, 0xDB, 0xCB, 0xCB, 0xCB, 0xCB, 0xCB, 0xFB, 0x8B, 0x8B, 0x8B, 0x8B, 0x8B, 0x9B, 0x9B, 0x08, 
                    0xD1, 0xD5, 0xCB, 0xCB, 0xCB, 0xCB, 0xCB, 0xFB, 0x8C, 0x8B, 0x8B, 0x8B, 0x8B, 0x95, 0x9B, 0x08, 
                    0xD1, 0xCB, 0xCB, 0xCB, 0xCB, 0xCC, 0xCC, 0xFB, 0xFB, 0x8C, 0x8B, 0x8B, 0x8B, 0x8B, 0x9B, 0x08, 
                    0xD1, 0xCC, 0xCC, 0xCC, 0xCC, 0xFB, 0xF5, 0xFB, 0xF5, 0xFB, 0x8B, 0x8B, 0x8B, 0x8B, 0x9B, 0x08, 
                    0xD1, 0xD5, 0xD5, 0xD5, 0xD5, 0xFB, 0xEC, 0xFB, 0xEC, 0xFB, 0x8B, 0x8B, 0x8B, 0x8B, 0x9B, 0x08, 
                    0xD1, 0xCB, 0xCB, 0xCB, 0xCB, 0xF5, 0xF5, 0xFB, 0xFB, 0xF5, 0x8B, 0x8B, 0x8B, 0x8B, 0x9B, 0x08, 
                    0xD1, 0xCC, 0xCB, 0xCB, 0xCB, 0xCB, 0xCB, 0xFB, 0xFB, 0x8B, 0x8B, 0x8B, 0x8B, 0x8C, 0x9B, 0x08, 
                    0xD1, 0xDB, 0xCB, 0xCB, 0xCB, 0xCB, 0xCB, 0xFB, 0xF5, 0x8B, 0x8B, 0x8B, 0x8B, 0x9B, 0x9B, 0x08, 
                    0xD1, 0xDB, 0xCC, 0xCB, 0xCB, 0xCB, 0xCB, 0xFB, 0x8B, 0x8B, 0x8B, 0x8B, 0x8C, 0x9B, 0x9B, 0x08, 
                    0xD1, 0xDB, 0xDB, 0xCC, 0xCB, 0xCB, 0xCB, 0xFB, 0x8B, 0x8B, 0x8B, 0x8C, 0x9B, 0x9B, 0x9B, 0x08, 
                    0xD1, 0xDB, 0xDB, 0xDB, 0xCC, 0xCC, 0xCB, 0xFB, 0x8C, 0x8C, 0x8C, 0x9B, 0x9B, 0x9B, 0x9B, 0x08, 
                    0xD1, 0xDB, 0xDB, 0xDB, 0xDB, 0xDB, 0xCC, 0xFB, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x9B, 0x08,
                },
                .collisionData = {0x0381, 0x0201, 0xEE1, 0x1EF1, 0x3EF9, 0x3EF9, 0x7E7D, 0x783D, 0x2BD, 0x783D, 0x7E7D, 0x3E79, 0x3EF9, 0x1EF1, 0xEE1, 0x201},
                .coords = {COORDS_XY(7,6), COORDS_XY(7,10)},
                .direction = TRAINER_DIRS(DIR_SOUTH, DIR_NORTH),
                .range = TRAINER_RANGE(3, 3),
            }
        },
    [3] =
        {
            .trainerNum1 = 63,
            .trainerNum2 = 64,
            .trainers =
                {
                    [0] =
                        {
                            .name = _("NORA"),
                            .facilityClass = FACILITY_CLASS_PKMN_RANGER_F,
                            .unused = 0,
                            .speechBefore = {EC_WORD_WAAAH, EC_WORD_THIS, EC_WORD_IS, EC_WORD_SO, EC_WORD_EXCITING, EC_WORD_EXCL},
                            .speechWin = {EC_WORD_WAAAH, EC_WORD_I_AM, EC_WORD_SO, EC_WORD_HAPPY, EC_WORD_I, EC_WORD_WON},
                            .speechLose = {EC_WORD_WAAAH, EC_WORD_I_AM, EC_WORD_SO, EC_WORD_SAD, EC_WORD_I, EC_WORD_LOST},
                            .speechAfter = {EC_WORD_WAAAH, EC_WORD_EXCL, EC_EMPTY_WORD, EC_WORD_GO, EC_WORD_AWAY, EC_WORD_EXCL},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_FORRETRESS,
                                            .heldItem = ITEM_QUICK_CLAW,
                                            .moves = {MOVE_EXPLOSION, MOVE_EARTHQUAKE, MOVE_ATTRACT, MOVE_SPIKES},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 110,
                                            .attackEV = 200,
                                            .spDefenseEV = 200,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("FORRETRESS"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_ELECTRODE,
                                            .heldItem = ITEM_SALAC_BERRY,
                                            .moves = {MOVE_EXPLOSION, MOVE_THUNDERBOLT, MOVE_SWIFT, MOVE_LIGHT_SCREEN},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 1,
                                            .personality = 0xC,
                                            .nickname = _("ELECTRODE"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_EXEGGUTOR,
                                            .heldItem = ITEM_SHELL_BELL,
                                            .moves = {MOVE_EXPLOSION, MOVE_HYPNOSIS, MOVE_PSYCHIC, MOVE_SOLAR_BEAM},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x7F,
                                            .nickname = _("EXEGGUTOR"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_DUSCLOPS,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_IMPRISON, MOVE_PROTECT, MOVE_ICE_BEAM, MOVE_EARTHQUAKE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 110,
                                            .defenseEV = 200,
                                            .spDefenseEV = 200,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x93,
                                            .nickname = _("DUSCLOPS"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_NINETALES,
                                            .heldItem = ITEM_WHITE_HERB,
                                            .moves = {MOVE_IMPRISON, MOVE_PROTECT, MOVE_OVERHEAT, MOVE_CONFUSE_RAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0xF,
                                            .nickname = _("NINETALES"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_BANETTE,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_IMPRISON, MOVE_PROTECT, MOVE_THUNDERBOLT, MOVE_THUNDER},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x96,
                                            .nickname = _("BANETTE"),
                                            .friendship = 255,
                                        },
                                },
                        },
                    [1] =
                        {
                            .name = _("GAV"),
                            .facilityClass = FACILITY_CLASS_PKMN_RANGER_M,
                            .unused = 0,
                            .speechBefore = {EC_WORD_WAHAHAHA, EC_WORD_YOU, EC_WORD_CAN_T, EC_WORD_STOP, EC_WORD_ME, EC_WORD_EXCL},
                            .speechWin = {EC_WORD_WOWEE, EC_WORD_EXCL, EC_EMPTY_WORD, EC_WORD_I_AM, EC_WORD_AWESOME, EC_WORD_COOL},
                            .speechLose = {EC_WORD_I_AM, EC_WORD_NOT, EC_WORD_COOL, EC_WORD_AT, EC_WORD_ALL, EC_WORD_EXCL},
                            .speechAfter = {EC_WORD_HEHEHE, EC_WORD_THAT_WAS, EC_WORD_AN, EC_WORD_AWESOME, EC_WORD_BATTLE, EC_WORD_EXCL},
                            .mons =
                                {
                                    [0] =
                                        {
                                            .species = SPECIES_SALAMENCE,
                                            .heldItem = ITEM_SHELL_BELL,
                                            .moves = {MOVE_ROCK_SLIDE, MOVE_FLAMETHROWER, MOVE_DRAGON_CLAW, MOVE_AERIAL_ACE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x95,
                                            .nickname = _("SALAMENCE"),
                                            .friendship = 255,
                                        },
                                    [1] =
                                        {
                                            .species = SPECIES_GENGAR,
                                            .heldItem = ITEM_LUM_BERRY,
                                            .moves = {MOVE_PSYCHIC, MOVE_GIGA_DRAIN, MOVE_WILL_O_WISP, MOVE_DESTINY_BOND},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .speedEV = 255,
                                            .spAttackEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x8C,
                                            .nickname = _("GENGAR"),
                                            .friendship = 255,
                                        },
                                    [2] =
                                        {
                                            .species = SPECIES_GYARADOS,
                                            .heldItem = ITEM_BRIGHT_POWDER,
                                            .moves = {MOVE_DRAGON_DANCE, MOVE_HYPER_BEAM, MOVE_BITE, MOVE_EARTHQUAKE},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("GYARADOS"),
                                            .friendship = 255,
                                        },
                                    [3] =
                                        {
                                            .species = SPECIES_GENGAR,
                                            .heldItem = ITEM_SALAC_BERRY,
                                            .moves = {MOVE_EXPLOSION, MOVE_MEAN_LOOK, MOVE_SHADOW_BALL, MOVE_CONFUSE_RAY},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .attackEV = 255,
                                            .speedEV = 255,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x3,
                                            .nickname = _("GENGAR"),
                                            .friendship = 255,
                                        },
                                    [4] =
                                        {
                                            .species = SPECIES_DUSCLOPS,
                                            .heldItem = ITEM_LEFTOVERS,
                                            .moves = {MOVE_MEAN_LOOK, MOVE_CONFUSE_RAY, MOVE_WILL_O_WISP, MOVE_SHADOW_BALL},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 110,
                                            .defenseEV = 200,
                                            .spDefenseEV = 200,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x14,
                                            .nickname = _("DUSCLOPS"),
                                            .friendship = 255,
                                        },
                                    [5] =
                                        {
                                            .species = SPECIES_MISDREAVUS,
                                            .heldItem = ITEM_FOCUS_BAND,
                                            .moves = {MOVE_MEAN_LOOK, MOVE_CONFUSE_RAY, MOVE_PERISH_SONG, MOVE_SHADOW_BALL},
                                            .level = 0,
                                            .ppBonuses = 0,
                                            .hpEV = 180,
                                            .defenseEV = 180,
                                            .spDefenseEV = 150,
                                            .otId = TRAINER_HILL_OTID,
                                            .hpIV = MAX_PER_STAT_IVS,
                                            .attackIV = MAX_PER_STAT_IVS,
                                            .defenseIV = MAX_PER_STAT_IVS,
                                            .speedIV = MAX_PER_STAT_IVS,
                                            .spAttackIV = MAX_PER_STAT_IVS,
                                            .spDefenseIV = MAX_PER_STAT_IVS,
                                            .abilityNum = 0,
                                            .personality = 0x85,
                                            .nickname = _("MISDREAVUS"),
                                            .friendship = 255,
                                        },
                                },
                        },
                },
            .display = {
                .metatileData = {
                    0x31, 0x3B, 0x3B, 0x3B, 0x39, 0x39, 0x13, 0x14, 0x15, 0x39, 0x39, 0x3A, 0x3B, 0x3B, 0x3B, 0x08, 
                    0x96, 0xFB, 0xF6, 0xFD, 0xF6, 0xF6, 0xFB, 0x46, 0xFB, 0xF6, 0xF6, 0xFD, 0xF6, 0xFB, 0x9B, 0x08, 
                    0xFB, 0x9B, 0xFB, 0xF6, 0xFB, 0xFB, 0xFB, 0x46, 0xFB, 0xFB, 0xFB, 0xF6, 0xFB, 0x9B, 0xFB, 0x08, 
                    0xF6, 0xFB, 0x9B, 0xFB, 0xFB, 0x46, 0x46, 0x9B, 0x46, 0x46, 0xFB, 0xFB, 0x9B, 0xFB, 0xF6, 0x08, 
                    0xFD, 0xF6, 0xFB, 0x9B, 0x46, 0x36, 0x36, 0x9B, 0x36, 0x36, 0x46, 0x9B, 0xFB, 0xF6, 0xFD, 0x08, 
                    0xF6, 0xFB, 0xFB, 0x46, 0x9B, 0x46, 0x46, 0xDB, 0x46, 0x46, 0x9B, 0x46, 0xFB, 0xFB, 0xF6, 0x08, 
                    0xF6, 0xFB, 0x46, 0x36, 0x46, 0x9B, 0xDB, 0xD6, 0xDB, 0x9B, 0x46, 0x36, 0x46, 0xFB, 0xF6, 0x08, 
                    0xFB, 0xFB, 0x46, 0x36, 0x9B, 0xDB, 0xD6, 0xD6, 0xD6, 0xDB, 0x9B, 0x36, 0x46, 0xFB, 0xFB, 0x08, 
                    0x96, 0x9B, 0x9B, 0x9B, 0x9B, 0xD6, 0xD6, 0x96, 0xD6, 0xD6, 0xDB, 0x9B, 0x9B, 0x9B, 0x96, 0x08, 
                    0xFB, 0xFB, 0x46, 0x36, 0x9B, 0xDB, 0xD6, 0xD6, 0xD6, 0xDB, 0x9B, 0x36, 0x46, 0xFB, 0xFB, 0x08, 
                    0xF6, 0xFB, 0x46, 0x36, 0x46, 0x9B, 0xDB, 0xD6, 0xDB, 0x9B, 0x46, 0x36, 0x46, 0xFB, 0xF6, 0x08, 
                    0xF6, 0xFB, 0xFB, 0x46, 0x9B, 0x46, 0x46, 0xDB, 0x46, 0x46, 0x9B, 0x46, 0xFB, 0xFB, 0xF6, 0x08, 
                    0xFD, 0xF6, 0xFB, 0x9B, 0x46, 0x36, 0x36, 0x9B, 0x36, 0x36, 0x46, 0x9B, 0xFB, 0xF6, 0xFD, 0x08, 
                    0xF6, 0xFB, 0x9B, 0xFB, 0xFB, 0x46, 0x46, 0x9B, 0x46, 0x46, 0xFB, 0xFB, 0x9B, 0xFB, 0xF6, 0x08, 
                    0xFB, 0x9B, 0xFB, 0xF6, 0xFB, 0xFB, 0xFB, 0x46, 0xFB, 0xFB, 0xFB, 0xF6, 0xFB, 0x9B, 0xFB, 0x08, 
                    0x96, 0xFB, 0xF6, 0xFD, 0xF6, 0xF6, 0xFB, 0x46, 0xFB, 0xF6, 0xF6, 0xFD, 0xF6, 0xFB, 0x9B, 0x08,
                },
                .collisionData = {0x0381, 0x0101, 0x0101, 0x6C1, 0x0821, 0x16D1, 0x2829, 0x2009, 0x1, 0x2009, 0x2829, 0x16D1, 0x0821, 0x6C1, 0x0101, 0x101},
                .coords = {COORDS_XY(7,6), COORDS_XY(7,10)},
                .direction = TRAINER_DIRS(DIR_SOUTH, DIR_NORTH),
                .range = TRAINER_RANGE(3, 3),
            }
        },
};

#undef COORDS_XY
#undef TRAINER_DIRS
#undef TRAINER_RANGE
